#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# funciones para manejo de portadas
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use prontus_varglb;
# use glib_fildir_03;
# use lib_waitlock; en actualizar_portadas_byartic
package lib_ports;

use strict;
use utf8;

# variables para la creacion de portadas
our %AREA = ();
our %PRIO = ();
our %VB = ();
our %DIR_FECHA = ();
# no ejecutar preprocesos
our $PREPROC_NO_BG = 0;

# ---------------------------------------------------------------
# Re-genera todas las portadas de la edicion correspondiente,
# donde el articulo referido este presente.
# Se considera la ed. base, la ultima y la vigente.
sub actualizar_portadas_byartic {
    my $ts     = $_[0];    # Nombre del articulo SIN extension y sin path, a dejar sin VoBo en las secciones donde aparezca.
    my $accion = $_[1];    # ALTA_CONTROL | BORRAR

    # procesa edicion, vigente, la ultima y la base.
    my @ediciones = &get_edics4update();

    foreach my $edic (@ediciones) {
        # print STDERR "edic[$edic]\n";
        # Directorio destino de las portadas de la edicion, en site, sino existe en site no busca el xml asociado
        my $pathdir_seccs = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . "/$edic" . $prontus_varglb::DIR_PORT;

        my @entries = glib_fildir_03::lee_dir($pathdir_seccs);

        # Para cada portada.
        foreach my $entry (@entries) {
            next if (substr($entry, 0, 1) eq '.');
            my $requiere_actualizar = 0;

            # portada en el site
            my $arch_seccion = "$pathdir_seccs/$entry";

            my $arch_xml = $arch_seccion;
            $arch_xml =~ s/\/port\/(\w+?)\.\w*$/\/xml\/$1\.xml/;
            # print STDERR "arch_xml[$arch_xml]\n";

            # template de la portada
            my $path_tsec = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_PORT . "/$entry";
            next if ((!-f $arch_xml) || ($entry =~ /^preview/));

            my $lock = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/$entry";
            $lock =~ s/\.\w+$//;
            $lock = "$lock.mutex";
            my $fh = lib_waitlock::lock_file($lock);

            %AREA      = ();
            %PRIO      = ();
            %DIR_FECHA = ();
            my $xml_port = glib_fildir_03::read_file($arch_xml);

            my $port_pub = &get_port_pub($arch_xml);
            my $no_write = 0; # por defecto no_write desactivado.
            # Si la portada no esta publicada se activa el no_write.
            $no_write = 1 if ($port_pub eq '0');

            # Rescatar la info de c/artic de la portada actual
            # Solo carga la info de portada si el articulo esta publicado en ella
            if ($xml_port =~ /<file>$ts<\/file>/s) {
                while ($xml_port =~ /(<rowartic>\s*<dir>(\d+)<\/dir>\s*<file>(.*?)<\/file>\s*<area>(\d+)<\/area>\s*<ord>(\d+)<\/ord>\s*<vb>(\d+)<\/vb>\s*<pub>(\d+)<\/pub>\s*<\/rowartic>)/sg) {
                    my ($rowartic, $dirfecha, $art, $area, $prio, $vb, $pub) = ($1, $2, $3, $4, $5, $6, $7);
                    # print STDERR "ENTRA con[$rowartic, $dirfecha, $art, $area, $prio, $vb, $pub]";

                    $AREA{$prontus_varglb::PRONTUS_ID}{$entry}{$art}      = $area;       # Asocia area al articulo.
                    $PRIO{$prontus_varglb::PRONTUS_ID}{$entry}{$art}      = $prio;       # Asocia prioridad correspondiente.
                    $VB{$prontus_varglb::PRONTUS_ID}{$entry}{$art}        = $vb;         # Asocia vobo del articulo en la portada.
                    $DIR_FECHA{$prontus_varglb::PRONTUS_ID}{$entry}{$art} = $dirfecha;
                }
                if ($accion eq 'BORRAR') {
                    delete $AREA{$prontus_varglb::PRONTUS_ID}{$entry}{$ts};
                    delete $PRIO{$prontus_varglb::PRONTUS_ID}{$entry}{$ts};
                    delete $VB{$prontus_varglb::PRONTUS_ID}{$entry}{$ts};
                    delete $DIR_FECHA{$prontus_varglb::PRONTUS_ID}{$entry}{$ts};
                }
                # indico que hay que regenerar la portada
                $requiere_actualizar = 1;
            } # while art. de esta portada

            # Re-Genera la portada si en ella esta el articulo en cuestion
            if ($requiere_actualizar) {
                my $mv            = '';
                my $sin_regen_xml = 0;
                my $ts_preview    = '';
                my $resp = &make_portada($arch_seccion, $path_tsec, $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID, $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}, $edic, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);

                # Ahora proceso multivistas
                $sin_regen_xml = 1;
                $resp = '';
                foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
                    $resp = &make_portada($arch_seccion, $path_tsec, $prontus_varglb::DIR_SERVER, $prontus_varglb::PRONTUS_ID, $mv, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}, $edic, $sin_regen_xml, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'}, $ts_preview, $no_write);
                }
            }

            lib_waitlock::unlock_file($lock, $fh);
        }    # foreach portadas de la edicion
    }    # foreach
}

# ---------------------------------------------------------------
# Carga un hash con los articulos publicados en las portadas
# y últimas ediciones
sub load_artic_pubs {
    # procesa edicion, vigente, la ultima y la base.
    my (@ediciones) = &get_edics4update();
    my %hash_artics;

    my $pathdir_json = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/data/cache';
    my $path_json = "$pathdir_json/json_artics_in_ports.json";

    if (-f $path_json) {
        my $json_artics = glib_fildir_03::read_file_bin($path_json);
        my $ref_hash_artics;
        # decodifica datos de json
        eval { $ref_hash_artics = JSON::decode_json($json_artics); };
        if ($@) {
            print STDERR "Error al decodificar json: $@, file[$path_json] json:[$json_artics]\n";
            # si falla, el json es invalido y se borra para que se re-genere
            unlink($path_json);
        } else {
            return %$ref_hash_artics;
        }
    }

    # Solo si la edicion es base, se guardan sólo las portadas base
    my %ports_base;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
        foreach my $port_base (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}) {
            $ports_base{$port_base} = 1;
            # print STDERR "Base: $port_base\n";
        }
    } else {
        %ports_base = %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}};
    }

    foreach my $edic (@ediciones) {
        # print STDERR "edic[$edic]\n";
        # Dir destino de las portadas de la ed.
        my $pathdir_seccs = $prontus_varglb::DIR_SERVER .
                        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} .
                        $prontus_varglb::DIR_EDIC .
                        "/$edic" .
                        $prontus_varglb::DIR_PORT;

        # Para cada port configurada
        foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) {
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
                if ($edic eq 'base') {
                    # hay multiedicion y esta procesando edicion base, si portada no es base, la salta
                    next unless ($ports_base{$port});
                } else {
                    # hay multiedicion y esta procesando ediciones que no son la base, si portada es base, la salta
                    next if ($ports_base{$port});
                }
            }

            # portada en el site
            my $arch_seccion = "$pathdir_seccs/$port";

            my $arch_xml = $arch_seccion;
            $arch_xml =~ s/\/port\/(\w+?)\.\w*$/\/xml\/$1\.xml/;

            next if ((!-f $arch_xml) || ($port =~ /^preview/));

            my $xml_port = glib_fildir_03::read_file($arch_xml);
            $xml_port = lib_parser::ajusta_crlf($xml_port);

            # Rescatar la info de c/artic de la port actual
            while ($xml_port =~ /<rowartic>\s*<dir>\d+<\/dir>\s*<file>(\d+)<\/file>.*?<\/rowartic>/sg) {
                my $art = $1;
                $hash_artics{$art}{$edic}{$port} = 1;
            }
        }
    }

    my $ref_hash_artics = \%hash_artics;
    my $json_artics;

    $json_artics = JSON::encode_json($ref_hash_artics);
    utf8::decode($json_artics);

    glib_fildir_03::check_dir($pathdir_json);
    glib_fildir_03::write_file($path_json, $json_artics);

    return %hash_artics;
}

# ---------------------------------------------------------------
# Obtiene un arreglo con las ediciones cuyas portadas es necesario actualizar por haberse modificado o eliminado
# articulos publicados en ellas.
# Estas ediciones corresponden a:
# - ed. base
# - ed. vigente
# - ultima edicion
# - la de hoy
sub get_edics4update {
    my (@ediciones);

    # Siempre procesa la ed. base.
    my $edic_base = $prontus_varglb::DIR_BASEEDIC;
    $edic_base =~ s/^\///;
    push @ediciones, $edic_base;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {

        # Obtener ed. vigente.
        my $ed_vigente = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/ed_vigente.txt");
        if ($ed_vigente) {
            push @ediciones, $ed_vigente;
        }

        # Obtener ultima edicion
        my $ed_newer = &get_newer_edic();
        if (($ed_newer) && ($ed_newer ne $ed_vigente)) {
            push @ediciones, $ed_newer;
        }

        # Obtener la de hoy
        my $edic_hoy;    # aaaammdd
        my $fecha_hoy = glib_hrfec_03::get_date_pack4();
        $edic_hoy = substr($fecha_hoy, 0, 4) . '_' . substr($fecha_hoy, 4, 2) . '_' . substr($fecha_hoy, 6, 2) . '_1';
        if (-d "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/$edic_hoy") {
            push @ediciones, $edic_hoy if (($edic_hoy ne $ed_newer) && ($edic_hoy ne $ed_vigente));
        }
    }

    return @ediciones;
}

# --------------------------------------------------------------------
# Lee el xml de la portada y obtiene el campo port_pub
# 1 = portada publicada
# 0 = portada no publicada, no debiese existir html en site.
#
sub get_port_pub {
    my $full_path_xmlport = $_[0];
    my $buffer_port_xml   = glib_fildir_03::read_file($full_path_xmlport);
    my $port_pub = 1; # si el campo no esta se entrega por defecto que si esta publicado.

    if ($buffer_port_xml =~ /<port_pub>(\d+)<\/port_pub>/) {
        $port_pub = $1;
    }

    return $port_pub;
}

# -------------------------------------------------------------------------#
# Genera la portada de acuerdo al template escogido.
# $dest_file: full path de la portada a generar, incluyendo extension
# $path_tpl: full path de la plantilla de portada, incluyendo extension
sub make_portada {
    my ($dest_file, $path_tpl, $dir_server, $prontus_id, $mv, $public_server_name, $nom_edic, $sin_regen_xml, $control_fecha, $ts_preview, $no_write) = @_;

    # print STDERR "make_portada: path_tpl[$path_tpl] mv[$mv]\n";

    my %plts4port;
    my %port_nom_clones;
    my %port_ext_clones;

    # Agregar la portada principal tambien
    $plts4port{$path_tpl} = $dest_file;

    $path_tpl =~ /(.*)\/([^\/]+\.\w+)$/;
    my $dir_tpl = $1;
    my $nomport = $2;
    # print STDERR "nomport[$nomport]\n";
    # Agregar las plantillas paralelas
    if (exists($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport})) {
        # contiene las paralelas en un arreglo
        my $extra_ports = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport}{'plt_paralela'};
        foreach my $extra_port (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$nomport}{'plt_paralela'}}) {
            next if (!-f "$dir_tpl/$extra_port");
            my $my_dest_file = $dest_file;
            $my_dest_file =~ s/\/$nomport$/\/$extra_port/;
            $plts4port{"$dir_tpl/$extra_port"} = $my_dest_file;

            my ($nom_clon, $ext) = lib_utils::split_nom_y_extension($extra_port);
            $port_nom_clones{"$dir_tpl/$extra_port"} = $nom_clon;
            $port_ext_clones{"$dir_tpl/$extra_port"} = $ext;
        }
    }

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my $port_seo_file = "$port_seo_dir/$nomport";
    $port_seo_file =~ s/\.\w+$/.json/;

    my %seo_data;
    if (-f $port_seo_file) {
        my $jsonSeoBuffer = &glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %seo_data = %{$jsonSeoRef};
    }

    # plantilla para los links a previews.
    my $previews_plt_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/core/links_previews.html";
    my $previews_tpl = glib_fildir_03::read_file($previews_plt_path);
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    # Parsear la portada principal y sus clones
    foreach my $path_tpl_clon (keys %plts4port) {
        my ($no_procesar, $preproc) = &check_port_no_procesar($path_tpl_clon, $mv);
        # print STDERR "no_procesar[$no_procesar] preproc[$preproc]\n";

        if ($no_procesar == 1) {
            print STDERR "No se parsea plantilla, no procesar activo!\n";
            $preproc =~ s/[#;&<>|~]+//g;

            if ($preproc ne '') {
                # Se ejecuta el pre-proceso. Esperar que termine o se ejecuta en segudno plano ?
                my $dest_xml = $dest_file;
                $dest_xml =~ s/\/port\/(\w+)(\.\w+)?$/\/xml\/$1\.xml/;

                # Incluye path completo hacia el xml y path completo hacia la plantilla que se ignoró.
                my $cmd = "$rutaScript/$preproc $dest_xml $path_tpl_clon >/dev/null 2>&1"; # bg.

                if (!$PREPROC_NO_BG) {
                    $cmd .= " &";
                }

                print STDERR "PRE PROC cmd[$cmd]\n";
                system($cmd);
            }

            next;
        }

        # No se procesa la portada y no se escribe el html.
        if ($no_write == 1) {
            my $dest_file_clon = $plts4port{$path_tpl_clon};
            $dest_file_clon =~ s/\/port\//\/port-$mv\// if ($mv);

            print STDERR "no_write[$no_write] -> [$dest_file_clon] \n";

            unlink $dest_file_clon if (-e $dest_file_clon);
            next;
        }

        my $dest_file_clon = $plts4port{$path_tpl_clon};
        # print STDERR "path_tpl_clon[$path_tpl_clon]\n";
        # Carga plantilla y realiza parseos normales
        my $buffer = &generic_parse_port($path_tpl_clon, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, $mv, \%seo_data, $nomport);
        # print STDERR "buffer[$buffer]\n";

        # Escribe portada html
        $dest_file_clon =~ s/\/port\//\/port-$mv\// if ($mv);
        my $destdir = $dest_file_clon;
        $destdir =~ s/\/[\w\.]+$//;
        glib_fildir_03::check_dir($destdir);
        my $nom_clon = $port_nom_clones{$path_tpl_clon};
        my $ext_clon = $port_ext_clones{$path_tpl_clon};
        # Si es un clon el que se esta previsualizando, le pongo preview_<nomclon>_<port>.<ext>
        if (($dest_file_clon =~ /\/port(-$mv)?\/preview_/) && ($path_tpl ne $path_tpl_clon)) {
            $dest_file_clon =~ s/\/port(-$mv)?\/preview_/\/port$1\/preview_$nom_clon\_/;
        }

        # En los preview, poner links a los demas previews
        my $cual_viendo = $path_tpl_clon;
        $cual_viendo =~ /([^\/\\]+\.\w+)$/;
        $cual_viendo = $1;

        $previews_tpl =~ s/%%cual_viendo%%/$cual_viendo/;
        $previews_tpl =~ /%%loop_previews%%(.*?)%%\/loop_previews%/s;
        my $loop_previews = $1;

        if ((%port_nom_clones) && ($dest_file_clon =~ /\/port\/preview/)) {
            my $links_previews;
            my $paralela = '';

            foreach my $k (keys %plts4port) {
                my $tmp_preview = $loop_previews;

                my $k_dst = $plts4port{$k};
                if ($path_tpl ne $k) {
                    my $nom_k = $port_nom_clones{$k};
                    $k_dst =~ s/\/port\/preview_/\/port\/preview_$nom_k\_/;
                    $paralela = 'paralela';
                }
                $k =~ /([^\/\\]+\.\w+)$/;
                $k = $1;
                $k_dst =~ /([^\/\\]+\.\w+)$/;
                $k_dst = $1;

                $tmp_preview =~ s/%%k_dst%%/$k_dst/g;
                $tmp_preview =~ s/%%paralela%%/$paralela/g;
                $tmp_preview =~ s/%%k%%/$k/g;
                $links_previews = $tmp_preview . $links_previews;
            }

            $previews_tpl =~ s/%%loop_previews%%.*%%\/loop_previews%%/$links_previews/gs;

            if ($buffer =~ /(<body[^>]*?>)/) {
                my $first = $1;
                if ($first =~ /</) {
                    $buffer =~ s/(<div)/$previews_tpl$1/;
                } else {
                    $buffer =~ s/(<body[^>]*?>)/$1$previews_tpl/;
                }
            } else {
                $buffer = $previews_tpl . $buffer;
            }
        }

        $previews_tpl =~ s/%%loop_previews%%.*%%\/loop_previews%%//gs;

        $buffer =~ s/%%_vista%%/$mv/g;
        $buffer =~ s/%%.+?%%//g;
        glib_fildir_03::write_file($dest_file_clon, $buffer);
        lib_prontus::purge_cache($dest_file_clon);
    }

    # Escribe xml de la portada ppal.
    my $resp; # eventual respuesta por problemas en write_xml_port
    $resp = &write_xml_port($dest_file, &generar_rows_xml_port($control_fecha, $nomport), $no_write) if (!$sin_regen_xml);
    if ($resp) {
        return $resp; # si hubo problemas, termino acá.
    }

    # -------------------------------------------------
    # Genera xml para RSS, para todas las vistas
    # -------------------------------------------------

    # Lee archivo de template rss.
    # Se llama igual a la portada pero esta en vez de /port en /rss y tiene extension .xml
    $path_tpl =~ s/\/port(\-\w+)?\/(\w+)\.\w+?$/\/rss$1\/$2\.xml/;

    if ((-s $path_tpl) && (-f $path_tpl)) {
        my $buffer = &generic_parse_port($path_tpl, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, '', \%seo_data, $nomport);

        # Parseos especificos
        $buffer =~ s/%%_vista%%/$mv/g;

        # Escribe RSS.
        $buffer =~ s/%%.+?%%//g;
        &write_rss_port($dest_file, $nom_edic, $buffer);
    }

    # Borra cache de no publicados
    glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/data/cache");

    return '';
}

# ---------------------------------------------------------------
# Obtiene nombre de la edic mas nueva
sub get_newer_edic {
    my ($dir) = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC;

    my @entries = glib_fildir_03::lee_dir($dir);
    @entries = sort { $b cmp $a } (@entries);
    foreach my $entry (@entries) {
        if (($entry ne 'base') && (-d "$dir/$entry") && ($entry =~ /^[0-9_]+$/)) {
            return $entry;
        }
    }
    return '';
}

# -------------------------------------------------------------------------
sub check_port_no_procesar {
    my $path_tpl = $_[0];
    my $mv = $_[1];
    my $dir_macros = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/plantillas/edic/nroedic/macros";

    my $buffer = lib_prontus::carga_buffer_plt($path_tpl, $dir_macros, $mv);
    my $pproc;
    my $no_procesar = 0;

    # print STDERR "check_port_no_procesar: mv[$mv] path_tpl[$path_tpl]\n";

    if ($buffer =~ /<!--\s*NO_PROCESAR\s*-->/isg) {
        $no_procesar = 1;
        if ($buffer =~ /<!--\s*PRE_PROCESO\s*=\s*['|"]?(.+?)['|"]?\s*-->/isg) {
            $pproc = $1;
         }
    }

    return ($no_procesar, $pproc);
}

# -------------------------------------------------------------------------
# carga plantilla de portada y realiza parseos genericos, comunes a port normal y a rss
sub generic_parse_port {
    my ($path_tpl, $dir_server, $prontus_id, $public_server_name, $nom_edic, $control_fecha, $ts_preview, $mv, $seo_ref, $nomport) = @_;
    my %seo_data = %{$seo_ref};

    my ($dir_macros) = "$dir_server/$prontus_id/plantillas/edic/nroedic/macros";
    my $buffer = lib_prontus::carga_buffer_plt($path_tpl, $dir_macros, $mv);

    my $doblegato_string = glib_str_03::random_string(30);
    $buffer =~ s/##/$doblegato_string/g;

    # Obtener nombre y extensión de la plantilla.
    my @paths = split('/', $path_tpl);
    my $plt_file = $paths[$#paths];
    my ($nom_plt, $ext_plt) = lib_utils::split_nom_y_extension($paths[$#paths]);
    print STDERR "[$plt_file] [$path_tpl]\n";

    $buffer =~ s/%%_plt_nom%%/$nom_plt/ig;
    $buffer =~ s/%%_plt_ext%%/$ext_plt/ig;
    $buffer =~ s/%%_mv%%/$mv/g;

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my $port_seo_file = "$port_seo_dir/$nom_plt.json";

    if (-f $port_seo_file) {
        my $jsonSeoBuffer = glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %seo_data = %{$jsonSeoRef};
    }

    # Marcas globales
    $buffer = lib_parser::parse_globals($buffer, $nom_edic);

    # Encuentra los tags de area dentro del template (%%LOOPi%%...%%/LOOP%%),
    # y produce los contenidos.
    my %areas;
    my %area_cont;
    my %area_check;

    while ($buffer =~ /%%loop(\d+)(\([^)]+?\))?%%(.*?)%%\/loop%%/sg) {
        my ($are, $template) = ($1, $3);
        my $pure_are = $are;
        if (!exists $area_cont{$pure_are}) {
            $area_cont{$pure_are} = 1;
            $area_check{$pure_are} = 1;
        }
        if (exists $areas{$are}) {
            $are .= '_' . sprintf("%04d", $area_cont{$pure_are});
            $area_cont{$pure_are}++;
            $area_check{$are} = 1;
        }
        # Parsea el area usando $2 como template parcial.
        $areas{$are} = &parser_area($pure_are, $template, $prontus_id, $control_fecha, $ts_preview, $doblegato_string, $mv, \%seo_data, $nomport);
    }

    # Sustituye las areas parseadas.
    foreach my $key (sort { $a cmp $b } (keys %areas)) {
        my $aux = $areas{$key};
        $key    =~ s/_\d+$//;
        $buffer =~ s/%%loop$key(\([^)]+?\))?%%(.*?)%%\/loop%%/$aux/s;
    }

    # Parseo de nifloop.
    foreach my $key (keys %area_check) {
        if ($areas{$key}) { # el area no esta vacia.
            $key =~ s/_\d+$//;
            $buffer =~ s/%%nifloop$key%%(.*?)%%\/nifloop%%//s;
        }
    }

    # parsea valores de seo
    if (scalar(keys %seo_data)) {
        foreach my $seoKey (keys %seo_data) {
            $buffer =~ s/%%$seoKey%%/$seo_data{$seoKey}/sg;
            $buffer = lib_prontus::parse_tag_prontus($seo_data{$seoKey}, $seoKey, $buffer);
        }

        # Procesar IFs y NIFs fuera de los loops de portada
        $buffer = lib_parser::procesa_condicional($buffer, \%seo_data);
    }

    # Borra todos los tags IFV que quedaron en la pagina.
    $buffer =~ s/%%ifvc?\(\d+\, *\d+\)%%//sg;
    $buffer =~ s/%%\/ifvc?%%//g;

    # Borra todos los tags NIFV que quedaron en la pagina.
    $buffer =~ s/%%nifv\(\d+\, *\d+\)%%//sg;
    $buffer =~ s/%%\/nifv%%//g;

    $buffer =~ s/$doblegato_string/##/g;

    $buffer = lib_prontus::parser_custom_function($buffer);

    $buffer = lib_prontus::parse_includes($dir_server, $buffer);

    $buffer =~ s/%%[^_].+?%%//g;
    $buffer =~ s/<!--POST_PROCESO=.+?-->//g;

    return $buffer;
}

# ---------------------------------------------------------------
# Verifica si la ed. consultada es o no la vigente, revisando si el nro
# de esta es el q figura en el archivo ed_vigente.txt
sub verifica_ed_vigente {
    my $nombre_ed = $_[0];
    my $pag = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/ed_vigente.txt");

    if ($pag =~ /$nombre_ed/) {
        return 'SI';
    } else {
        return 'NO';
    }
}

# -------------------------------------------------------------------------
sub generar_rows_xml_port {
    my ($control_fecha,$nomport) = @_;

    # Ordenar ascendentemente por area / orden / id articulo.
    my @arr_art = sort {$AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$a} <=> $AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$b}
                        || $PRIO{$prontus_varglb::PRONTUS_ID}{$nomport}{$a} <=> $PRIO{$prontus_varglb::PRONTUS_ID}{$nomport}{$b}
                        || $b <=> $a}
                        (keys %{$AREA{$prontus_varglb::PRONTUS_ID}{$nomport}});

    my ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($base)) {
        print STDERR "ERROR: $msg_err_bd\n";
        exit;
    }

    my $rowartics_xml = '';
    # plantilla para los rowartics.
    my $rowartics_tpl = "<rowartic>\n".
    "    <dir>%%dir_fecha%%</dir>\n".
    "    <file>%%key%%</file>\n".
    "    <area>%%area%%</area>\n".
    "    <ord>%%prio%%</ord>\n".
    "    <vb>%%vb%%</vb>\n".
    "    <pub>%%pub%%</pub>\n</rowartic>";
    # antes de escribir el nuevo xml, saca una copia de respaldo.
    foreach my $key (@arr_art) {
        my $stamp = $rowartics_tpl;
        my $prio      = $PRIO{$prontus_varglb::PRONTUS_ID}{$nomport}{$key};
        my $dir_fecha = lib_prontus::get_dirfecha_by_ts($key);
        my $area      = $AREA{$prontus_varglb::PRONTUS_ID}{$nomport}{$key};
        my $vb        = $VB{$prontus_varglb::PRONTUS_ID}{$nomport}{$key};

        my $pub = lib_prontus::get_status_pub($key, $control_fecha, $base);

        next if (!$area);
        $prio = 1 if ((!$prio) && ($area));

        $stamp =~ s/%%dir_fecha%%/$dir_fecha/g;
        $stamp =~ s/%%key%%/$key/g;
        $stamp =~ s/%%area%%/$area/g;
        $stamp =~ s/%%prio%%/$prio/g;
        $stamp =~ s/%%vb%%/$vb/g;
        $stamp =~ s/%%pub%%/$pub/g;

        if (($rowartics_xml !~ /$stamp/) && ($key ne 'filler')) {
            $rowartics_xml .= $stamp;
        }
    }

    return $rowartics_xml;
}

# -------------------------------------------------------------------------
# Escribe el xml de portada prontus
sub write_xml_port {
    # Validar quota antes de trabajar
    my $msg_err_quota = lib_quota::check_quota_suficiente();
    if ($msg_err_quota) {
        return "Error: $msg_err_quota";
    }
    my $dest_xml      = $_[0];    # Deduce del path completo de la portada, el del xml.
    my $rowartics_xml = $_[1];
    my $no_write      = $_[2];
    my $port_pub = 1;

    if ($no_write) {
        $port_pub = 0;
    } else {
        $port_pub = 1;
    }

    if ($dest_xml !~ /\.xml$/) {
        $dest_xml =~ s/\/port\/(\w+)(\.\w+)?$/\/xml\/$1\.xml/;
    }

    # plantilla para los xml.
    my $new_xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<port_data>\n$rowartics_xml\n<port_pub>$port_pub</port_pub>\n</port_data>\n";

    # antes de escribir el nuevo xml, saca una copia de respaldo.
    my $md5_newXml = Digest::MD5::md5_hex($new_xml);

    my $dobackup = 0;
    my ($nomxml, $dirxml) = ('', '');
    if ($dest_xml =~ /^(.*\/xml)\/(.*\.xml)$/) {
        $dirxml = $1;
        $nomxml = $2;
        glib_fildir_03::check_dir("$dirxml/bak");

        my $curXml     = glib_fildir_03::read_file($dest_xml);
        my $md5_curXml = Digest::MD5::md5_hex($curXml);
        if ($md5_newXml eq $md5_curXml) {
            # portada nueva y antigua son iguales, no hace falta sobre escribir el archivo
            return '';
        } else {
            $dobackup = 1;
        }

        # el respaldo se realiza sólo si los xml son distintos, es decir, sólo si lo que se guarda es distinto a lo guardado
        if ($dobackup) {
            unlink "$dirxml/bak/$nomxml.9" if (-f "$dirxml/bak/$nomxml.9");
            for (my $i = 9 ; $i > 0 ; $i--) {
                File::Copy::move("$dirxml/bak/$nomxml." . ($i - 1), "$dirxml/bak/$nomxml." . ($i));
            }
            File::Copy::move("$dirxml/bak/$nomxml", "$dirxml/bak/$nomxml.0");
        }
    }
    # escribe el nuevo xml
    glib_fildir_03::write_file($dest_xml, $new_xml);
    File::Copy::copy($dest_xml, "$dirxml/bak/$nomxml") if ($dobackup);
}

# -------------------------------------------------------------------------
# Escribe el xml del rss de portada prontus
sub write_rss_port {
    my ($dest_rss, $nom_edic, $buffer) = @_;
    $dest_rss =~ s/\/port(\-\w+)?\/(\w+)\.\w+?$/\/rss$1\/$2\.xml/;    # Deduce del path completo de la portada, el del rss.

    # si es una edicion normal, igual se escribe en el dir de la edic base
    # ya que los rss deben estar en una ubicacion fija
    $dest_rss =~ s/$nom_edic/base/ig;

    my $destdir_rss = $dest_rss;
    $destdir_rss =~ s/\/[\w\.]+$//;

    glib_fildir_03::check_dir($destdir_rss);
    glib_fildir_03::write_file($dest_rss, $buffer);
    lib_prontus::purge_cache($dest_rss);
}

# -------------------------------------------------------------------------#
# Genera un area de acuerdo al template suministrado.
# Retorna el buffer parseado y el nro. de articulos en el area
# %AREA = ();
# %PRIO = ();
sub parser_area {
    my ($area, $theloop, $prontus_id, $control_fecha, $ts_preview, $doblegato_string, $mv, $seo_data, $plt_file) = @_;

    # Obtiene lista de articulos de esta area.
    my $num_artics_in_area = 0;
    my (%articulos) = ();    # Articulos que pertenecen a esta area.
    foreach my $key (sort { $b cmp $a } (keys %{$AREA{$prontus_id}{$plt_file}})) {
        if ($AREA{$prontus_id}{$plt_file}{$key} eq $area) {
            $articulos{$key} = $AREA{$prontus_id}{$plt_file}{$key};
        }
    }

    # Ordena los art. de esta area por prioridad y timestamp
    my @keys_articulos = sort { $PRIO{$prontus_id}{$plt_file}{$a} <=> $PRIO{$prontus_id}{$plt_file}{$b} or $b <=> $a } keys %articulos;

    my (@filtered_keys, $ultimo, %hash_ifvs, $salida_html);
    my $totartics = 0;

    # Aca se calculan los articulos que efectivamente serán publicados
    my %array_artics;
    foreach my $ts_artic (@keys_articulos) {
        # Revisar VoBo de publicacion
        if ($VB{$prontus_id}{$plt_file}{$ts_artic} eq '0') {
            next;
        }

        if (!defined($lib_prontus::ARTICS{$prontus_id}{$ts_artic})) {
            $lib_prontus::ARTICS{$prontus_id}{$ts_artic} = Artic->new('prontus_id'         => $prontus_id,
                                   'public_server_name' => $prontus_varglb::CONFIG{$prontus_id}{'PUBLIC_SERVER_NAME'},
                                   'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_id}{'CPAN_SERVER_NAME'},
                                   'document_root'      => $prontus_varglb::DIR_SERVER,
                                   'ts'                 => $ts_artic,
                                   'no_check_dirs'      => 1) || die "Error inicializando objeto articulo: $Artic::ERR\n";
        }

        my %campos_xml = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_xml_content();

        # Filtro por fechap/horap y fechae/horae del artic
        if ($control_fecha eq 'SI') {
            if (!lib_utils::fechas_ok($ts_preview, $campos_xml{'_fechap'}, $campos_xml{'_horap'}, $campos_xml{'_fechae'}, $campos_xml{'_horae'})) {
                next;
            }
        }

        # Filtro por alta
        my $alta_key = $campos_xml{'_alta'};
        if (!$alta_key) {    # sin alta
            next;
        }

        # Se guarda el objeto para usarlo mas abajo
        $filtered_keys[$totartics] = $ts_artic;
        $array_artics{$ts_artic} = \%campos_xml;
        $totartics++;
    }

    # Aca se asume que todos los artículos serán parseados
    my $stop_loop   = 0;
    my $loopcounter = 0;
    while (($loopcounter <= $#filtered_keys) && ($stop_loop < 1)) {
        my $ts_artic = $filtered_keys[$loopcounter];

        if (!defined($lib_prontus::ARTICS{$prontus_id}{$ts_artic})) {
            $lib_prontus::ARTICS{$prontus_id}{$ts_artic} = Artic->new('prontus_id'         => $prontus_id,
                                        'public_server_name' => $prontus_varglb::CONFIG{$prontus_id}{'PUBLIC_SERVER_NAME'},
                                        'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_id}{'CPAN_SERVER_NAME'},
                                        'document_root'      => $prontus_varglb::DIR_SERVER,
                                        'ts'                 => $ts_artic,
                                        'no_check_dirs'      => 1) || die "Error inicializando objeto articulo: $Artic::ERR\n";
        }

        my %campos_xml;
        if (ref $array_artics{$ts_artic}) {
            %campos_xml = %{$array_artics{$ts_artic}};
        } else {
            %campos_xml = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_xml_content();
        }

        # agrega vista a datos de artículo
        $campos_xml{'_mv'} = $mv;

        $loopcounter++;
        # Copia sub-template para generar instancia.
        my $localbuf = $theloop;

        # Genera loop_artic
        $localbuf = lib_parser::procesa_loop_artic($localbuf, $doblegato_string);

        # ------------- IFVs.
        # Si el contador de articulos para este loop MOD div != res entonces borrar contenido
        my $buf_aux = $localbuf;
        my ($div, $res, $mod, $instancias);

        if (($buf_aux =~ /%%ifvc\((\d+)\, *(\d+)\)%%.+?%%\/ifvc%%/s) and ($buf_aux !~ /%%ifv\((\d+)\, *(\d+)\)%%.+?%%\/ifv%%/s)) {
            return 'Error en Loop: Sentencia IFVC debe ir acompañada de a lo menos un IFV', $loopcounter;
        }

        while ($buf_aux =~ /%%ifv(C?)\((\d+)\, *(\d+)\)%%.+?%%\/ifv\1%%/sg) {
            my $ifvc = $1;
            $div = $2;
            $res = $3;
            if (!$div) {
                $localbuf =~ s/%%(ifv$ifvc\($div\, *$res\))%%(.+?)%%\/(ifv$ifvc)%%/<h1>Error: división por 0<\/h1>/s;
                next;
            }
            $mod = $loopcounter % $div;
            #~ print STDERR "var[$loopcounter] % div[$div] = mod[$mod] y res[$res] y ifvc[$ifvc]\n"; # DEBUG
            # elimina los segmentos q no corresponden, dejando solo uno de acuerdo a la iteracion q toca.
            if ($mod == $res) {
                if ($ifvc) {
                    $ultimo = 'ifvc';
                } else {
                    $ultimo = 'ifv';
                }
                #~ print STDERR "CALZA EN [%%IFV$ifvc($div,$res)%%.+?%%/IFV%%] EN instancias_x_area[$loopcounter]\n";
                #~ print STDERR "ultimo[$ultimo]\n";
                $hash_ifvs{$loopcounter} = $ultimo;

                $localbuf =~ s/%%(ifv$ifvc\($div\, *$res\))%%(.+?)%%\/(ifv$ifvc)%%/##$1##$2##\/$3##/s;    # try again (enmascarar)
                last;
            }
            if (($ts_artic eq 'filler') && ($loopcounter == $div)) {
                #~ print STDERR "STOP LOOP - INSTANCIAS == DIV\n";
                $stop_loop = 1;
            }
        }  # while %%IFV...

        $localbuf =~ s/%%ifv(C?)\(\d+\, *\d+\)%%.+?%%\/ifv\1%%//sg;
        $localbuf =~ s/##/%%/g;                                        # try sian

        # elimino los tags del if que quedo.
        $localbuf =~ s/%%ifvc?\(\d+\, *\d+\)%%//sg;
        $localbuf =~ s/%%\/ifvc?%%//sg;
        # ------------- /IFVs.

        # ------------- NIFVs.
        # Si el contador de articulos para este loop MOD div == res entonces borrar contenido
        $buf_aux = $localbuf;
        ($div, $res, $mod) = '';
        while ($buf_aux =~ /%%nifv\((\d+)\, *(\d+)\)%%.+?%%\/nifv%%/sg) {
            $div = $1;
            $res = $2;
            if (!$div) {
                $localbuf =~ s/%%nifv\($div\, *$res\)%%.+?%%\/nifv%%/<h1>Error: división por 0<\/h1>/s;
                next;
            }
            $mod = $loopcounter % $div;
            # print "var[$loopcounter] % div[$div] = mod[$mod] y res[$res]<br>"; # DEBUG

            if ($mod == $res) {
                $localbuf =~ s/%%nifv\($div\, *$res\)%%.+?%%\/nifv%%//sg;
            }
        }
        # elimino los tags del if que quedo.
        $localbuf =~ s/%%nifv\(\d+\, *\d+\)%%//sg;
        $localbuf =~ s/%%\/nifv%%//sg;
        # ------------- /NIFVs.

        # Parsea datos del artic
        if ($ts_artic eq 'filler') {
            #~ print STDERR "Haciendo filler de la iteracion\n";
            my %nodata;
            my %fillerdata;
            $fillerdata{_ts} = $ts_artic;
            $localbuf = lib_parser::procesa_condicional($localbuf, \%fillerdata, \%nodata);
            $localbuf =~ s/%%.*?%%//sg;
        } elsif ($localbuf ne '') {
            # Recupera marcas externas (desde /xdata)
            my %claves_adicionales = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_xdata();
            # Procesa condicionales mas algunas variables adicionales
            $claves_adicionales{_ts}          = $ts_artic;
            $claves_adicionales{_loopcounter} = $loopcounter;
            $claves_adicionales{_totartics}   = $totartics;
            $claves_adicionales{_area}        = $area;

            foreach my $seoKey (keys %{$seo_data}) {
                $claves_adicionales{_area} = $seo_data->{$seoKey};
            }

            # Buscamos las eventuales fixed_urls en db.
            $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_fixed_urls(\%claves_adicionales);

            # procesar loops e if de multitag
            $localbuf = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->parsea_loop_multitag($localbuf, \%campos_xml, \%claves_adicionales);

            $localbuf = lib_parser::procesa_condicional($localbuf, \%campos_xml, \%claves_adicionales);

            my $fullpath_vista = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->get_fullpath_artic($mv, $campos_xml{'_plt'});
            $localbuf = $lib_prontus::ARTICS{$prontus_id}{$ts_artic}->parse_artic_data($fullpath_vista, $localbuf, \%campos_xml, \%claves_adicionales);
        }
        $salida_html .= $localbuf;

        if ($loopcounter >= $totartics) {
            #~ print STDERR "-- Instancias_x_area[$loopcounter] - FINALMENTE TOCO UN[$hash_ifvs{$loopcounter}]\n";
            if ($hash_ifvs{$loopcounter} eq 'ifvc') {
                push @filtered_keys, 'filler';
                if ($loopcounter > 10000) {
                    $stop_loop = 1;
                    #~ print STDERR "STOPLOOP forced\n";
                }
            } else {
                $stop_loop = 1;
                #~ print STDERR "STOPLOOP\n";
            }
        }
        #~ print STDERR "Fin de la iteracion $loopcounter / $totartics\n\n";
    }

    #~ print STDERR "Fin del area $area\n------------------ \n\n\n";
    return $salida_html;
}    # parserArea

# --------------------------------------------------------------------
# Lee el xml de una portada prontus y retorna su contenido en un hash
# indexado por el articulo (<ts>.<ext>)
# Los rowartics son de la forma:
#   <rowartic>
#    <dir>20070125</dir>
#    <file>20070125195234</file>
#    <area>2</area>
#    <ord>2</ord>
#    <pub>0</pub>
#  </rowartic>
sub get_rowartics {
    my $full_path_xmlport = $_[0];
    my $buffer_port_xml   = glib_fildir_03::read_file($full_path_xmlport);
    my %rowartics;
    while ($buffer_port_xml =~ /<rowartic>(.*?)<\/rowartic>/sg) {
        my $rowartic = $1;
        if ($rowartic =~ /<file>(\d{14})<\/file>/) {
            my $artic_file = $1;
            while ($rowartic =~ /<(\w+?)>(.*?)<\/\1>/sg) {
                my $nom_campo   = $1;
                my $valor_campo = $2;
                $rowartics{$artic_file}{$nom_campo} = $valor_campo;
            }
        }
    }
    return %rowartics;
}

return 1;
