#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# -------------------------------BEGIN LIBRERIA-------------
# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------
package lib_prontus;

use strict;
use utf8;

# Indica si hay Prontus Function disponibles
our $PF_ENABLED = 0;
eval {require lib_custom;};
if ($@) {
    $PF_ENABLED = 0;
} else {
    $PF_ENABLED = 1;
}

our $DO_PURGE = 0;
our $DISABLE_PURGE_CACHE = 0;

# cache de artículos, para no leerlos de nuevo
our %ARTICS = ();
# cache de multitag para no cargar de nuevo
our %MULTITAG = ();
# cache de tags para no cargar de nuevo
our %TAGS_HASH = ();

sub check_dirs {
    # TEMPLATES ARTICULOS
    # Dir. de tpl de las pags de los articulos
    my $dir = $prontus_varglb::DIR_SERVER .
           $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} .
           $prontus_varglb::DIR_ARTIC .
           $prontus_varglb::DIR_FECHA .
           $prontus_varglb::DIR_PAG;

    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de plantillas de articulos [$dir] no es válido";
    }

    if (!(&check_contenido_dir($dir))) {
        print STDERR "El directorio de plantillas de artículos está vacío, éste debe contener al menos una plantilla [$dir]\n";
        return "El directorio de plantillas de artículos está vacío, éste debe contener al menos una plantilla 1";
    }

    # DESTINO ARTICULOS -----------------------------
    # Dir a los articulos existentes hasta antes de la fecha
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_ARTIC;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio destino de artículos no es válido";
    }

    # Dir para respaldos
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_BKP;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio respaldos no es válido";
    }

    # Dir para semaforos mutex
    $dir = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_LOCK";
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio mutex no es válido";
    }

    # Dir Multimedia
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EXMEDIA;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio destino de multimedia no es válido";
    }

    # TEMPLATES TAXONOMIA
    if ($prontus_varglb::TAXONOMIA_NIVELES =~ /^[1-3]$/) {
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXONOMIA;

        if (!(glib_fildir_03::check_dir($dir))) {
            return "El directorio de plantillas de art. relacionados no es válido";
        }
        if (!(&check_contenido_dir($dir))) {
            return "El directorio de plantillas de art. relacionados está vacío. Debe haber a lo menos una plantilla.";
        }
    }

    # DESTINO TAXONOMIA
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} =~ /^[1-3]$/) {
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXONOMIA;

        if (!(glib_fildir_03::check_dir($dir))) {
            return "El directorio de destino de art. relacionados no es válido";
        }

        # TEMPLATE portada tipo tema
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_TAXPORT;

        if (!(glib_fildir_03::check_dir($dir))) {
            return "El directorio de plantillas de portadas taxonómicas no es válido";
        }

        # DESTINO portadas tipo tema
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_TAXPORT;

        if (!(glib_fildir_03::check_dir($dir))) {
            return "El directorio de destino de portadas taxonómicas no es válido";
        }
    }

    # Directorio de destino de los includes friendly v4
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '4') {
        $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_FRIENDLY;

        if (!(glib_fildir_03::check_dir($dir))) {
            return "El directorio de destino de urls friendly v4 no es válido";
        }
    }

    # Directorio de cache de multitag
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_MULTITAG;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de cache de multitag no es válido";
    }

    # Dir de los dbm
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}))) {
        return "El directorio de data de usuarios no es válido";
    }

    # dir de las keys para decriptar tokens openid connect
    if ($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} eq 'SI' && !(glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/oidc_keys/"))) {
        return "El directorio de claves para decriptación de tokens OpenID Connect no existe.";
    }

    # Dir de los logs
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/log'))) {
        return "El directorio de logs de Prontus no es válido";
    }

    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/procs'))) {
        return "El directorio de datos de procesos de Prontus no es válido";
    }

    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . $prontus_varglb::DIR_DATA4FIDS))) {
        return "El directorio de datos de para Cpan de Prontus no es válido";
    }

    # Dir cpan/fid
    if (!(glib_fildir_03::check_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/fid'))) {
        return "El directorio de FIDs no es válido";
    }
    if (!(&check_contenido_dir($prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/fid'))) {
        return "El directorio de FIDs está vacío, éste debe contener al menos un FID";
    }

    # TEMPLATES DE PORTADAS
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_PORT;

    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de plantillas de portadas no es válido";
    }

    if (!(&check_contenido_dir($dir))) {
        return "El directorio de plantillas de portadas está vacío, éste debe contener al menos un template";
    }

    # Dir de TEMPLATE de homepages.
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_NROEDIC . $prontus_varglb::DIR_HPAGES;

    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de plantilla de la home page de ediciones no es válido";
    }

    # Archivo Template de homepage de edición.
    my $nom_tpl_home = "$dir/$prontus_varglb::INDEX_EDIC";
    if (!-f $nom_tpl_home) {
        print STDERR "Plantilla de homepage de edición no existe[$nom_tpl_home]\n";
    }

    # DESTINO EDICIONES
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC;

    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de destino de ediciones no es válido";
    }

    # Dir. destino de Edicion unica
    $dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC;

    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de edición única no es válido";
    }

    $dir .= $prontus_varglb::DIR_PORT;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de destino de las portadas de la edición única no es válido";
    }

    # Dir destino de los xml de portada de la ed. unica
    $dir =~ s/$prontus_varglb::DIR_PORT$/\/xml/;
    # print STDERR "alfa[$dir]\n";
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de destino del xml de las portadas de la edición única no es válido";
    }

    # Dir destino de los rss de portada de la ed. unica
    $dir =~ s/\/xml$/\/rss/;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de destino de los rss de las portadas de la edición única no es válido";
    }

    return '';
}

# ---------------------------------------------------------------
# Chequea que dentro del dir pasado por parametro haya por lo menos 1 archivo.
sub check_contenido_dir {
    my $ruta_dir = $_[0];

    my @lisdir = glib_fildir_03::lee_dir($ruta_dir);
    @lisdir = grep !/^\./, @lisdir;    # Elimina directorios . y ..

    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            return 1;    # encuentra a lo menos un archivo valido dentro del directorio.
        }
    }
    return 0;
}

# ---------------------------------------------------------------
# Obtener el nombre del primer archivo que figura en el directorio.
# Param: 0) Path al Directorio
# Retorna : el nombre del archivo correspondiente  sin path y c/extension o '' en caso de que no se encuentre ningun archivo.
sub get_nomfile {
    my $dir = shift;
    my @entries = glib_fildir_03::lee_dir($dir);
    foreach my $entry (@entries) {
        if (($entry !~ /^\./g) and ($entry !~ /^preview/ig) and (-f "$dir/$entry")) {
            return $entry;
        }
    }
    return '';
}

# ---------------------------------------------------------------
# chequea la extension contra un string (separados por coma)
sub check_extension {
    my $nomfile   = shift;
    my $str2check = shift;

    my $file_extension = lc lib_utils::get_file_extension($nomfile);
    return 0 if ($str2check !~ /(^|,)$file_extension($|,)/);
    return 1;
}

# ---------------------------------------------------------------
sub get_dirfecha_by_ts {
    my $ts = shift;
    if ($ts =~ /^(\d{8})\d{6}/) {
        return $1;
    }
    return '';
}

# ---------------------------------------------------------------
# Chequea si el articulo tiene fotos
sub check_fotos_from_ts {
    my $ts = $_[0];
    my $buff_xml_data = $_[1];

    if (! defined($_[1])) {
        my $dirfecha = substr($ts, 0, 8);
        my $fullpath_xml = $prontus_varglb::DIR_SERVER .
                           $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} .
                           $prontus_varglb::DIR_ARTIC .
                           "/$dirfecha" .
                           "/xml" .
                           "/$ts.xml";
        $buff_xml_data = &get_xml_data($fullpath_xml);
    }
    if ($buff_xml_data =~ /<foto_(\d+)>(.*?)<\/foto_\1>/s) {
        my $num  = $1;
        my $buff = $2;
        if ($buff =~ /<_nomfoto_$num>(.+?)<\/_nomfoto_$num>/) {
            return 1;
        }
    }
    return 0;
}

# ---------------------------------------------------------------
# Borra el cache de artículos publicados en portadas para que se refresque automáticamente.
sub borrar_json_artic_pubs {
    my $pathdir_json = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'} . '/data/cache';
    glib_fildir_03::check_dir($pathdir_json);
    my $path_json = "$pathdir_json/json_artics_in_ports.json";

    if (-f $path_json) {
        unlink $path_json;
    }
}

# -------------------------------------------------------------------------#
# Carga la configuracion de prontus, mantiene compatibilidad con scripts
# que usan path_conf como parametro
# Param :
# 0) Path completo absoluto al archivo.
sub load_config {
    my $path_conf = $_[0];
    print STDERR "load_config: este método no esta disponible\n";
    print STDERR "Debe actualziar su script para ser compatible con Prontus 12.\n";
    return "Error: este método no esta disponible";
}

# -------------------------------------------------------------------------#
sub get_prontus_plt_list_by_id {
    my $prontus_id = shift;

    my $path_cfg = $prontus_varglb::DIR_SERVER. "/$prontus_id/cpan/$prontus_id-port.cfg";
    my $buffer =  glib_fildir_03::read_file($path_cfg);          # portadas

    my %templates = ();

    while ($buffer =~ m/PORT\_PLTS\s*=\s*["'](.+?)\(.*?\)\(.*?\)(?:\(.*?\))?["']/g) {
        next if (length($1) >64);
        my $plantilla    = $1;    # plantilla principal de la portada
        if ($plantilla !~ /^\w+\.\w+$/) {
            print STDERR "Configuración: Plantilla de Portada [$plantilla] no es válido, debe contener sólo caracteres alfanuméricos, incluido el underscore. Además, debe tener una extensión válida.\n";
            next;
        }
        $templates{'port'}{$plantilla} = 1;
    }

    $path_cfg = $prontus_varglb::DIR_SERVER. "/$prontus_id/cpan/$prontus_id-art.cfg";
    $buffer =  glib_fildir_03::read_file($path_cfg);          # portadas

    while ($buffer =~ m/FORM_PLTS\s*=\s*["'](.+?):(.+?)\((.+?)\)\(?(.*?)?\)?["']/g) {
        next if (length($1) >64);
        my @templates = split(/;/, $3);
        $templates{'art'}{$1} = 1;
    }

    return \%templates;
}

# -------------------------------------------------------------------------#
sub get_prontus_sso_dirs {
    my $dir = $prontus_varglb::DIR_SERVER;
    my @entries = glib_fildir_03::lee_dir($dir);
    my @core_dirs; # core dirs
    # se agrega el directorio del prontus master
    push(@core_dirs, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});

    # Recorre revisando todas las carpetas de la raiz buscando donde haya *-var.cfg y *-bd.cfg
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        if (&valida_prontus_dir($dir, $entry)) {
            my $buffer_conf = glib_fildir_03::read_file("$dir/$entry/cpan/$entry-usr.cfg");
            if ($buffer_conf  =~ /PRONTUS_SSO\s*=\s*["']SI["']/ && $buffer_conf =~ /PRONTUS_SSO_MASTER_ID\s*=\s*["']\Q$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'}\E["']/) {
                print STDERR "SSO Enabled [$entry]\n";
                push(@core_dirs, $entry);
            }
        }
    }
    return @core_dirs;
}

# -------------------------------------------------------------------------#
sub get_prontus_sso_masters {
    my $dir = $prontus_varglb::DIR_SERVER;
    my @entries = &glib_fildir_03::lee_dir($dir);
    my @core_dirs; # core dirs

    # Recorre revisando todas las carpetas de la raiz que sean un prontus valido
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        if (&valida_prontus_dir($dir, $entry)) {
            my $buffer_conf = glib_fildir_03::read_file("$dir/$entry/cpan/$entry-usr.cfg");
            if ($buffer_conf =~ /PRONTUS_SSO\s*=\s*["']SI["']/ && $buffer_conf =~ /PRONTUS_SSO_MASTER\s*=\s*["']SI["']/) {
                print STDERR "SSO Enabled [$entry]\n";
                push(@core_dirs, $entry);
            }
        }
    }
    return @core_dirs;
}
# -------------------------------------------------------------------------#
sub get_prontus_dirs {
    my $dir = $prontus_varglb::DIR_SERVER;
    my @entries = glib_fildir_03::lee_dir($dir);
    my @core_dirs; # core dirs

    # Recorre revisando todas las carpetas de la raiz buscando donde haya *-var.cfg y *-bd.cfg
    foreach my $entry (@entries) {
        next if ($entry eq 'prontus');
        if (&valida_prontus_dir($dir, $entry)) {
            push(@core_dirs, $entry);
        }
    }
    return @core_dirs;
}

# -------------------------------------------------------------------------#
sub valida_prontus_dir {
    my $dir = $_[0];
    my $entry = $_[1];
    return 0 if (substr($entry, 0, 1) eq '.');
    return 0 if ($entry eq 'prontus');
    return 0 if (!-d "$dir/$entry");
    if ((-f "$dir/$entry/cpan/$entry-var.cfg") && (-f "$dir/$entry/cpan/$entry-bd.cfg")) {
        return 1;
    }
    return 0;
}

# -------------------------------------------------------------------------#
# genera jsons con datos de prontus para ser usados via javascript
sub generaJsonProntus {
    # guardamos algunos datos para poder usarlos en el fid
    my %data = ();
    # guardamos algunos datos para poder usarlos en el frontend del sitio
    my %dataFrontend = ();
    $data{'public-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    $dataFrontend{'public-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    $data{'cpan-host'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    $data{'front-protocol'} = 'http';
    $dataFrontend{'front-protocol'} = 'http';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'} eq 'https') {
        $data{'front-protocol'} = 'https';
        $dataFrontend{'front-protocol'} = 'https';
    }
    $data{'cpan-protocol'} = 'http';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'} eq 'https') {
        $data{'cpan-protocol'} = 'https';
    }

    $data{'player-embed-video'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_VIDEO'};
    $data{'player-embed-audio'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_AUDIO'};

    # datos auth
    $data{'psw-max-length'} = prontus_auth::get_psw_max_length();
    $data{'psw-min-length'} = prontus_auth::get_psw_min_length();

    # datos para multitag
    $data{'multitag-incluir-prontus_id-url'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'INCLUIR_PRONTUS_ID_URL'} eq 'SI') {
        $data{'multitag-incluir-prontus_id-url'} = 1;
        $dataFrontend{'multitag-incluir-prontus_id-url'} = 1;
    }
    $data{'friendly-include-view-name'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
        $data{'friendly-include-view-name'} = 1;
    }

    $data{'friendly-v5-url-prefix'} = '';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V5_URL_PREFIX'} ne '') {
        $data{'friendly-v5-url-prefix'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V5_URL_PREFIX'};
    }

    $data{'oidc-enabled'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} eq 'SI') {
        $data{'oidc-enabled'} = 1;
    }

    $data{'seo-api-enabled'} = 0;
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_HABILITADO'} eq 'SI') {
        $data{'seo-api-enabled'} = 1;
    }

    @{$data{'mv'}} = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}};

    my $jsonString = JSON::encode_json(\%data);
    utf8::decode($jsonString);

    my $jsonStringFrontend = JSON::encode_json(\%dataFrontend);
    utf8::decode($jsonStringFrontend);

    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS");
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/prontus.js", "var ProntusConf = $jsonString");
    # escribimos a frontend lo mínimo indispensable. Creamos el dir [prontus]/site/js si no existe.
    glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js");
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js/prontus.js", "var ProntusConf = $jsonStringFrontend");

    my %data_ads = (
        "adEnable"   => ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_ENABLE'} eq 'SI')?1:0,
        "adInfoMsg"  => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_INFO_MSG'},
        "adTag" => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_TAG'});

    my $jsonStringAds = JSON::encode_json(\%data_ads);
    utf8::decode($jsonStringAds);

    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/js/playerConf.json", $jsonStringAds);

    return '';
}

# ---------------------------------------------------------------
# Carga plantilla de articulo incluyendo las macros que contenga.
sub carga_buffer_plt {
    my $fullpath_plt       = shift;    # path de la plantilla de la vista estandar
    my $pathdir_plt_macros = shift;
    my $mv                 = shift;

    $fullpath_plt =~ s/(.+)\/([^\/]+)$/$1-$mv\/$2/ if ($mv ne '');
    # warn "cargando plantilla[$fullpath_plt]";
    my $buffer = glib_fildir_03::read_file($fullpath_plt);
    if ($buffer !~ /\w/) {
        return '';
    }
    $buffer = lib_parser::ajusta_crlf($buffer);
    # agrega macros.
    $buffer = lib_parser::add_macros($buffer, $pathdir_plt_macros, '', '');
    $buffer =~ s/%25%25/%%/sg;
    return $buffer;
}

# ---------------------------------------------------------------
sub replace_hash_fields {
    my $buffer        = shift;
    my $vars2replace  = shift;
    my $prontus_style = shift;
    $prontus_style = 1 if ($prontus_style eq '');    # default

    foreach my $k (keys %$vars2replace) {
        my $marca = $k;
        $marca = '%%' . $k . '%%' if ($prontus_style);
        my $value = $$vars2replace{$k};
        $buffer =~ s/$marca/$value/sg;
    }

    return $buffer;
}

sub replace_mtime {
    my ($buf, $path_artic, $mtime) = @_;

    # Si viene la fecha de modificacion desde el xml del articulo, se usa. sino se sigue usando la fecha de
    # modificacion del archivo.
    if ($mtime) {
        my $ts = glib_hrfec_03::time2ts($mtime);
        my $fechamlong = glib_hrfec_03::expande_fecha($ts);
        my $fechamshrt = glib_hrfec_03::des_normaliza_fecha($ts);

        $ts =~ /(\d{8})(\d{2})(\d{2})(\d{2})/;
        my $fecham = $1;
        my $horam = "$2:$3:$4";

        $buf =~ s/%%_fechamlong%%/$fechamlong/g;
        $buf =~ s/%%_fechamshrt%%/$fechamshrt/g;
        $buf =~ s/%%_fecham%%/$fecham/g;
        $buf =~ s/%%_horam%%/$horam/g;
    } else {
        my $mtime = (stat $path_artic)[9];
        my $ts_lastmodif = glib_hrfec_03::time2ts($mtime);
        # warn("mtime[$mtime]ts_lastmodif[$ts_lastmodif]\n");

        my $fecham = substr($ts_lastmodif, 0, 8);
        my $horam  = substr($ts_lastmodif, 8, 6);
        $buf =~ s/%%_fecham%%/$fecham/g;

        my $flong = glib_hrfec_03::expande_fecha($fecham);
        $buf =~ s/%%_fechamlong%%/$flong/g;

        my $fshrt = glib_hrfec_03::des_normaliza_fecha($fecham);
        $buf =~ s/%%_fechamshrt%%/$fshrt/g;

        if ($horam =~ /(\d{2})(\d{2})(\d{2})$/) {
            $horam = $1 . ':' . $2 . ':' . $3;
        }
        $buf =~ s/%%_horam%%/$horam/g;
    }

    return $buf;
}

# ---------------------------------------------------------------
# Reemplaza TS, FECHAC, FECHACLONG, FECHACSHRT, a partir del TS
sub replace_tsdata {
    my $buf = $_[0];
    my $ts  = $_[1];
    my $prettyprint_ts = defined($_[2]) ? $_[2] : '';

    # Timestamp
    if ($ts =~ /^(\d{8})(\d{6})$/) {

        my $fechac = $1;
        my $horac  = $2;
        $buf =~ s/%%_ts%%/$ts/g;

        my $fechaclong = glib_hrfec_03::expande_fecha($fechac);
        $buf =~ s/%%_fechaclong%%/$fechaclong/g;

        my $fechacshrt = glib_hrfec_03::des_normaliza_fecha($fechac);
        $buf =~ s/%%_fechacshrt%%/$fechacshrt/g;

        $buf =~ s/%%_fechac%%/$fechac/g;

        if ($horac =~ /(\d{2})(\d{2})(\d{2})$/) {
            $horac = $1 . ':' . $2 . ':' . $3;
        }
        $buf =~ s/%%_horac%%/$horac/g;

    } else {
        $buf =~ s/%%_new_art%%/$prettyprint_ts/g;
        $buf =~ s/%%_ts%%//g;
        $buf =~ s/%%_fechaclong%%//g;
        $buf =~ s/%%_fechacshrt%%//g;
        $buf =~ s/%%_fechac%%//g;
        $buf =~ s/%%_horac%%//g;
    }
    return $buf;
}

# ---------------------------------------------------------------
# Parsea textos en localbuf de portada, ajustando nro. de caracteres.
sub parsea_texto {
    my $nomcampo = $_[0];
    # warn "nom_campo enpt[$nomcampo]\n";
    my $valcampo     = $_[1];
    my $localbuf     = $_[2];
    my $localbuf_aux = $localbuf;
    while ($localbuf_aux =~ /%%$nomcampo\(?\s*([0-9]*)[)]?%%/sg) {
        my $maxchars           = $1;
        my $val_campo_ajustado = $valcampo;
        # ajusta el nro de caracteres
        if (($valcampo ne '') && ($maxchars > 0)) {
            $val_campo_ajustado = &ajusta_nchars($valcampo, $maxchars);
        }

        $localbuf = &replace_in_port($val_campo_ajustado, $nomcampo, $localbuf, $maxchars);
    }
    return $localbuf;
}

# ---------------------------------------------------------------
# Ajusta el valor del campo de texto al max. nro. de caracteres
# $val_campo: string a ser recortada
# $nchars: cantidad de caracteres a conservar
sub ajusta_nchars {
    my ($val_campo, $nchars) = @_;
    if (! $nchars) { # si pasan 0 o nada, no acortamos.
        return $val_campo;
    }
    my $val_campo_con_tags = $val_campo;
    $val_campo =~ s/<[^>]+>//sg;       # elimino los tags para que no interfieran en el conteo de palabras.
    $val_campo =~ s/&nbsp\;/ /sg;    # elimina los &nbsp; insertados al formatear el texto (estos marcaban los saltos de linea.) # 1.18
    # Corta al nro. de caracteres especificado

    my $val_campo_original = $val_campo;
    if (length($val_campo_original) <= $nchars) {
        if ($val_campo_con_tags =~ /^\s*<p>.*<\/p>\s*$/s) {
            $val_campo_original = '<p>' . $val_campo_original . '</p>';
        }
        return $val_campo_original;
    }

    $val_campo = '';
    # Recorrer por palabras.
    my $string_ajustado;
    while ($val_campo_original =~ /([^\s]+)/g) {
        $string_ajustado = $val_campo;
        $val_campo .= ' ' if ($val_campo);
        $val_campo .= $1;

        # print STDERR "val_campo[$val_campo] largo[" . length($val_campo) . "][$string_ajustado]\n";
        if (length($val_campo) > ($nchars - 3)) {    # se restan 3 para que quede espacio para los puntos suspensivos
            if (length($string_ajustado) == 0) {     # si el largo de una palabra es mayor al maximo, se recorta dicha palabra.
                $string_ajustado = substr($val_campo, 0, ($nchars - 3));
            }
            last;
        }
    }

    $string_ajustado =~ s/\s*$//;                    # remueve espacio sobrante.
    $string_ajustado .= '...' if ($string_ajustado);
    if ($val_campo_con_tags =~ /^\s*<p>.*<\/p>\s*$/s) {
        $string_ajustado = '<p>' . $string_ajustado . '</p>';
    }
    return $string_ajustado;
}

# ---------------------------------------------------------------
# Parsea fechaplong y fechapshrt en base a fechap en el articulo.
# @param string $buffer el texto a procesar
# @param string $val_campo el campo con la fecha
# @param string $nom_campo el campo con el nombre de la marca.
# @return string el buffer procesado
sub parser_fechap {
    my ($buffer, $val_campo, $nom_campo) = @_;
    # %%FECHAPLONG%% = Fecha de publicación, en formato largo
    # %%FECHAPSHRT%% = Fecha de publicación, en formato corto
    # %%FECHAP%%     = Fecha de publicación, en formato ISO --> Calza con el 2o. tipo de sustitucion estandar por tanto no se hace nada especial.
    # funciona igual para _fechae
    my $marca_shrt = $nom_campo . 'shrt';
    my $marca_long = $nom_campo . 'long';

    if ($val_campo) {
        $buffer =~ s/%%$nom_campo%%/$val_campo/g;
        my $fechaplong = glib_hrfec_03::expande_fecha($val_campo);

        $buffer =~ s/%%$marca_long%%/$fechaplong/g;

        my $fechapshrt = glib_hrfec_03::des_normaliza_fecha($val_campo);
        $buffer =~ s/%%$marca_shrt%%/$fechapshrt/g;
    } else {
        $buffer =~ s/%%$marca_shrt%%//g;
        $buffer =~ s/%%$marca_long%%//g;
    }

    return $buffer;

}

# ---------------------------------------------------------------
# Obtiene status de publicacion del articulo.
sub get_status_pub {
    my $art                      = shift;
    my $control_fecha            = shift;
    my $base                     = shift;

    # Conectar a BD si es que no viene la conexion
    if (!ref($base)) {
        # print STDERR "connect a BD dentro\n";
        my $msg_err_bd;
        ($base, $msg_err_bd) = lib_bd::conectar_prontus_bd();
        if (!ref($base)) {
            print STDERR "ERROR: $msg_err_bd\n";
            exit;
        }
    }

    # Obtener datos necesarios del articulo de la bd
    my $id = $art;
    $id =~ s/\.\w*$//;    # saca extension
    my $sql = "select ART_ALTA, ART_INDEXAR, ART_FECHAP, CAST(ART_HORAP as CHAR) as ART_HORAP, ART_FECHAE, CAST(ART_HORAE as CHAR) as ART_HORAE from ART where ART_ID = ?";
    my ($alta, $indexar, $fechap, $horap, $fechae, $horae);
    my @params = ($id);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($alta, $indexar, $fechap, $horap, $fechae, $horae));
    $salida->fetch;
    $salida->finish;

    # Filtro por fechap/horap y fechae/horae del artic
    my $pub = 1;          # indicador de si el artic. quedo publicado en el html o no.
    if ($control_fecha eq 'SI') {
        if (!lib_utils::fechas_ok('', $fechap, $horap, $fechae, $horae)) {
            $pub = 0;
        }
    }
    # Filtro por alta
    if (!$alta) {
        $pub = 0;
    }

    return $pub;
}

# ----------------------------------------------------------------------------- #
    # Dado un texto, obtiene el valor del campo pasado como parametro, considerando
    # que esta almacenado de la forma <!--campo=valor--!>
sub get_valcampo_html {
    my $nom_campo  = $_[0];
    my $texto_html = $_[1];

    my ($texto, $delimitador_inicio, $delimitador_fin, $valor_campo);

    $texto = $texto_html;
    $texto =~ /<!--\s*$nom_campo\s*=\s*(.*?)\s*-->/;
    $valor_campo = $1;

    return $valor_campo;

}

# --------------------------------------------------------------------#
# Formatea el texto submitido, linea a linea.
sub parrafea_texto {
    my ($buffer) = ($_[0]);
    my ($salida, $i) = ('', 0);

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $buffer =~ s/$crlf/<p>/sg;

    # Extrae TODAS las lineas y las pone en el arreglo @LINEAS.
    my (@LINEAS) = split(/<P[^>]*?>/i, $buffer);

    # Formatea las lineas de acuerdo a su contexto.
    for ($i = 0 ; $i <= $#LINEAS ; $i++) {
        # print STDERR "LINEA: $LINEAS[$i]\n\n";
        if ($LINEAS[$i] =~ /^\s*$/) {
            # Inserta espacios para que aparezcan las lineas usadas como separador.
            $LINEAS[$i] = '&nbsp;';
        }
        # Agrega parrafo a la salida.
        $salida .= $LINEAS[$i] . "</p>\n<p>";
    }

    # Saca ult. ret.
    $salida =~ s/<\/p>\n<p>$/<\/p>/i;
    $salida = '<p>' . $salida if ($salida =~ /\w/);    # esto es para asegurarse que el texto quede rodeado de <p>...</p>

    return $salida;
}

# ---------------------------------------------------------------
# Cargar xml del articulo a partir del path a su html o bien del mismo xml.
sub get_xml_data {
    my $path_final_xml = $_[0];
    if ($path_final_xml !~ /\.xml$/) {
        $path_final_xml =~ s/\/pags\/(\w+)(\.\w+)?$/\/xml\/$1\.xml/;
    }

    my $xml = glib_fildir_03::read_file($path_final_xml);
    return '' if ($xml eq '');

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $xml =~ s/$crlf/\x0a/sg;

    my ($priv, $pub) = ('', '');
    if ($xml =~ /<_private>(.*?)<\/_private>/sg) {
        $priv = $1;
    }
    if ($xml =~ /<_public>(.*?)<\/_public>/sg) {
        $pub = $1;
    }
    return "$priv\n$pub";
}

# -------------------------------------------------------------------------
# Escapea lo minimo para que el texto no interfiera con los tags HTML.
sub escape_html {
    my ($toencode) = $_[0];

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_ENCODE_CHARS'} eq 'SI') {
        $toencode =~ s/&([^#][^0-9]+)/&amp;$1/g;    # Hace escaping de entities no numéricas.
    }

    $toencode =~ s/&quot;/&amp;quot;/g;
    $toencode =~ s/&lt;/&amp;lt;/g;
    $toencode =~ s/&gt;/&amp;gt;/g;
    $toencode =~ s/&#39;/&amp;#39;/g;

    $toencode =~ s/>/&gt;/g;                        # >
    $toencode =~ s/"/&quot;/g;                      # "
    $toencode =~ s/'/&#39;/g;                       # '
    $toencode =~ s/</&lt;/g;                        # <

    return $toencode;
}

# -------------------------------------------------------------------------#
# Detecta y pone en html los caracteres con tildes.
sub tildes2html {
    my ($to_encode) = $_[0];

    $to_encode =~ s/\x86/&Yacute;/g;
    $to_encode =~ s/\x87/&brvbar;/g;
    $to_encode =~ s/\x8B/&eth;/g;
    $to_encode =~ s/\x95/&middot;/g;
    $to_encode =~ s/\x96/&shy;/g;

    $to_encode =~ s/\xA6/&#166;/g;
    $to_encode =~ s/\xAF/&#175;/g;
    $to_encode =~ s/\xB2/&#178;/g;
    $to_encode =~ s/\xB3/&#179;/g;
    $to_encode =~ s/\xB9/&#185;/g;
    $to_encode =~ s/\xBC/&#188;/g;
    $to_encode =~ s/\xBD/&#189;/g;
    $to_encode =~ s/\xBE/&#190;/g;
    $to_encode =~ s/\xD7/&#215;/g;

    $to_encode =~ s/\xA1/&iexcl;/g;
    $to_encode =~ s/\xA2/&cent;/g;
    $to_encode =~ s/\xA3/&pound;/g;
    $to_encode =~ s/\xA4/&curren;/g;
    $to_encode =~ s/\xA5/&yen;/g;
    $to_encode =~ s/\xA7/&sect;/g;
    $to_encode =~ s/\xA8/&uml;/g;
    $to_encode =~ s/\xA9/&copy;/g;
    $to_encode =~ s/\xAA/&ordf;/g;
    $to_encode =~ s/\xAC/&not;/g;
    $to_encode =~ s/\xAD/&shy;/g;
    $to_encode =~ s/\xAE/&reg;/g;
    $to_encode =~ s/\xB0/&deg;/g;
    $to_encode =~ s/\xB1/&plusmn;/g;
    $to_encode =~ s/\xB4/&acute;/g;
    $to_encode =~ s/\xB5/&micro;/g;
    $to_encode =~ s/\xB6/&para;/g;
    $to_encode =~ s/\xB7/&middot;/g;
    $to_encode =~ s/\xB8/&cedil;/g;
    $to_encode =~ s/\xBA/&ordm;/g;
    $to_encode =~ s/\xBB/&raquo;/g;
    $to_encode =~ s/\xBF/&iquest;/g;
    $to_encode =~ s/\xC0/&Agrave;/g;
    $to_encode =~ s/\xC1/&Aacute;/g;
    $to_encode =~ s/\xC2/&Acirc;/g;
    $to_encode =~ s/\xC3/&Atilde;/g;
    $to_encode =~ s/\xC4/&Auml;/g;
    $to_encode =~ s/\xC5/&Aring;/g;
    $to_encode =~ s/\xC6/&AElig;/g;
    $to_encode =~ s/\xC7/&Ccedil;/g;
    $to_encode =~ s/\xC8/&Egrave;/g;
    $to_encode =~ s/\xC9/&Eacute;/g;
    $to_encode =~ s/\xCA/&Ecirc;/g;
    $to_encode =~ s/\xCB/&Euml;/g;
    $to_encode =~ s/\xCC/&Igrave;/g;
    $to_encode =~ s/\xCD/&Iacute;/g;
    $to_encode =~ s/\xCE/&Icirc;/g;
    $to_encode =~ s/\xCF/&Iuml;/g;
    $to_encode =~ s/\xD1/&Ntilde;/g;
    $to_encode =~ s/\xD2/&Ograve;/g;
    $to_encode =~ s/\xD3/&Oacute;/g;
    $to_encode =~ s/\xD4/&Ocirc;/g;
    $to_encode =~ s/\xD5/&Otilde;/g;
    $to_encode =~ s/\xD6/&Ouml;/g;

    $to_encode =~ s/\xD8/&Oslash;/g;
    $to_encode =~ s/\xD9/&Ugrave;/g;
    $to_encode =~ s/\xDA/&Uacute;/g;
    $to_encode =~ s/\xDB/&Ucirc;/g;
    $to_encode =~ s/\xDC/&Uuml;/g;
    $to_encode =~ s/\xDD/&Yacute;/g;
    $to_encode =~ s/\xDE/&THORN;/g;
    $to_encode =~ s/\xDF/&szlig;/g;
    $to_encode =~ s/\xE0/&agrave;/g;
    $to_encode =~ s/\xE1/&aacute;/g;
    $to_encode =~ s/\xE2/&acirc;/g;
    $to_encode =~ s/\xE3/&atilde;/g;
    $to_encode =~ s/\xE4/&auml;/g;
    $to_encode =~ s/\xE5/&aring;/g;
    $to_encode =~ s/\xE6/&aelig;/g;
    $to_encode =~ s/\xE7/&ccedil;/g;
    $to_encode =~ s/\xE8/&egrave;/g;
    $to_encode =~ s/\xE9/&eacute;/g;
    $to_encode =~ s/\xEA/&ecirc;/g;
    $to_encode =~ s/\xEB/&euml;/g;
    $to_encode =~ s/\xEC/&igrave;/g;
    $to_encode =~ s/\xED/&iacute;/g;
    $to_encode =~ s/\xEE/&icirc;/g;
    $to_encode =~ s/\xEF/&iuml;/g;
    $to_encode =~ s/\xF0/&eth;/g;
    $to_encode =~ s/\xF1/&ntilde;/g;
    $to_encode =~ s/\xF2/&ograve;/g;
    $to_encode =~ s/\xF3/&oacute;/g;
    $to_encode =~ s/\xF4/&ocirc;/g;
    $to_encode =~ s/\xF5/&otilde;/g;
    $to_encode =~ s/\xF6/&ouml;/g;
    $to_encode =~ s/\xF7/&divide;/g;
    $to_encode =~ s/\xF8/&oslash;/g;
    $to_encode =~ s/\xF9/&ugrave;/g;
    $to_encode =~ s/\xFA/&uacute;/g;
    $to_encode =~ s/\xFB/&ucirc;/g;
    $to_encode =~ s/\xFC/&uuml;/g;
    $to_encode =~ s/\xFD/&brvbar;/g;
    $to_encode =~ s/\xFE/&thorn;/g;
    $to_encode =~ s/\xFF/&yuml;/g;

    $to_encode =~ s/\x83/&#131;/g;

    $to_encode =~ s/\x85/&#133;/g;

    $to_encode =~ s/\x88/&#136;/g;
    $to_encode =~ s/\x89/&#137;/g;
    $to_encode =~ s/\x8A/&#138;/g;

    $to_encode =~ s/\x8C/&#140;/g;

    $to_encode =~ s/\x8E/&#142;/g;

    $to_encode =~ s/\x91/&#145;/g;
    $to_encode =~ s/\x92/&#146;/g;

    $to_encode =~ s/\x97/&#151;/g;

    $to_encode =~ s/\x99/&#153;/g;
    $to_encode =~ s/\x9A/&#154;/g;

    $to_encode =~ s/\x9C/&#156;/g;

    $to_encode =~ s/\x9E/&#158;/g;
    $to_encode =~ s/\x9F/&#159;/g;
    $to_encode =~ s/\x8242/&#8242;/g;
    $to_encode =~ s/\x8243/&#8243;/g;

    return $to_encode;
}

# Transforma comillas tipográficas, como en “Quotes1” «Quotes2» ‘Quotes3’ a &quot; para evitar problemas en el encoding
# parametros:
# - el texto a corregir
# retorna:
# - el texto corregido
sub convert_typography_quotes {
    my $to_clean = shift;
    $to_clean =~ s/[\p{Pi}\p{Pf}]/&quot;/g;
    $to_clean =~ s/\x{2013}/-/g;
    return $to_clean;
}

# -------------------------------------------------------------------------#
    # Realiza escapeos minimos para poder guardar el campo tipo txt dentro de un cdata
    # al interior de un XML.
sub escape_cdata {
    my $valor = $_[0];
    $valor =~ s/]]>/]] >/sg;    # para q no interfiera con cdata
    # No realiza ningun escapeo adicional para efectos de compatibilidad HTML

    return $valor;
}

# -------------------------------------------------------------------------#
sub replace_in_port {
    my $valor     = $_[0];
    my $nom_campo = $_[1];
    my $buffer    = $_[2];
    my $maxchars  = $_[3];      # \d

    if ($maxchars) {
        $buffer =~ s/%%$nom_campo[(]$maxchars[)]%%/$valor/sg;
    } else {
        $buffer =~ s/%%$nom_campo%%/$valor/g;
    }

    return $buffer;
}

# -------------------------------------------------------------------------#
sub parse_tag_prontus {
    my $valor_campo = shift;
    my $nom_campo   = shift;
    my $buffer      = shift;
    my $noescape    = shift;
    # print STDERR "nom_campo[$nom_campo] - valor_campo[$valor_campo]\n";

    $valor_campo =~ s/< *\?/< &#63;/g unless ($noescape);    # elimina secuencias <?
    $valor_campo =~ s/%%/&#37;&#37;/sg;                      # para poder poner nombres de marcas

    # Cambia rets carro por <p>
    $valor_campo = &parrafea_texto($valor_campo) if ($nom_campo =~ /^_?txt_/);

    # sustitucion en la pagina
    $buffer =~ s/%%$nom_campo%%/$valor_campo/sg;

    # parsea marcas con ajuste de chars
    if ($nom_campo !~ /^asocfile_|^multimedia_|^fotofija_|^_hfoto|^_wfoto|^chk_cuadrar_fotofija|^_nomfoto_|^foto_\d+|^_seccion|^_tema|^_subtema|^_fecha[ep]|^_hora[ep]|^_?v?txt_/) {
        $buffer = &parse_maxchars($nom_campo, $valor_campo, $buffer);
        # parsea otras salidas con restriccion de largo
        # parse codetext
        # parse jsontext
        # parse rss text (es un minitext, pero escapeado para xml)
        # parse strip.
        # parse urlencode
        $buffer = &parse_maxchars_special($nom_campo, $valor_campo, $buffer);
    } elsif ($nom_campo =~ /^(_)?v?txt_(\w+?)$/) {
        # proceso marcas que contengan código HTML
        my $nom_minitext = $1 . $2;

        # parse minitext
        my $minitext = &get_minitext_value($valor_campo);
        $buffer =~ s/%%$nom_minitext%%/$minitext/g;
        # parse _?txt_marca.raw o vtxt_marca.raw
        $buffer =~ s/%%$nom_campo.raw%%/$minitext/g;

        # parse minitext con max chars
        $buffer = &parse_maxchars($nom_minitext, $minitext, $buffer);
        # parsea la marca empezando con _TXT o _VTXT
        $buffer = &parse_maxchars($nom_campo, $valor_campo, $buffer);

        # parsea otras salidas con restriccion de largo
        # parse xml
        # parse codetext
        # parse jsontext
        # parse strip
        # parse urlencode
        $buffer = &parse_maxchars_special($nom_minitext, $minitext, $buffer);
        # parsea la marca empezando con _TXT o _VTXT
        $buffer = &parse_maxchars_special($nom_campo, $valor_campo, $buffer);
    }

    return $buffer;
}

# -------------------------------------------------------------------------#
sub urlencode_text {
    my $toencode = shift;
    $toencode =~ s/\r\n/ /sg;
    $toencode =~ s/\n/ /sg;
    $toencode =~ s/\r/ /sg;
    $toencode =~ s/<.*?>/ /sg;
    $toencode =~ s/ {2,}/ /sg;
    $toencode =~ s/ $//sg;
    $toencode =~ s/^ //sg;
    #~ $toencode =~ s/([^\-A-Za-z0-9_.!~*() ])/sprintf("%%%02X", ord($1))/eg;
    #~ $toencode =~ s/ /%20/sg;
    $toencode = URI::Escape::uri_escape_utf8($toencode);
    $toencode =~ s/'/%27/g;
    $toencode =~ s/"/%22/g;
    return $toencode;
}

# -------------------------------------------------------------------------#
sub strip_text {
    my $toencode = $_[0];

    $toencode = &saca_tags_rets($toencode);
    $toencode = &notildes($toencode);
    $toencode = lc $toencode;
    $toencode =~ s/[^a-z0-9_ ]//sg;
    $toencode =~ s/ {2,}//sg;
    $toencode =~ s/\s/_/sg;

    return $toencode;
}

# -------------------------------------------------------------------------#
sub get_minitext_value {
    my ($toencode) = $_[0];

    $toencode =~ s/\r\n/ /sg;
    $toencode =~ s/\n/ /sg;
    $toencode =~ s/\r/ /sg;
    $toencode =~ s/<.*?>/ /sg;
    $toencode =~ s/ {2,}/ /sg;
    $toencode =~ s/ $//sg;
    $toencode =~ s/^ //sg;
    return $toencode;
}

# -------------------------------------------------------------------------#
sub get_codetext_value {
    my ($toencode) = $_[0];

    # caracteres imprimibles, del 33(dec) en adealnte, exceptuando caracteres no imprimibles.
    $toencode =~ s/[^\x20-\x7e\x80\x82-\x9c\x9e-\xff]//sg;
    $toencode =~ s/\\/\\\\/sg;
    $toencode =~ s/"/\\"/sg;
    $toencode =~ s/&quot;/\\"/sg;
    $toencode =~ s/'/\\'/sg;
    $toencode =~ s/&#39;/\\'/sg;

    return $toencode;
}

# -------------------------------------------------------------------------#
# recibe una referencia a hash o array, o un escalar perl, devuelve una string json encodeada, rodeada en comillas dobles.
sub get_jsontext_value {
    my $toencode = $_[0];
    my $json = JSON->new->allow_nonref;
    $toencode = $json->encode($toencode);
    $toencode =~ s/^"(.*)"$/$1/gs;
    return $toencode;
}

# -------------------------------------------------------------------------#
sub parse_maxchars_special {
    my $nom_campo   = shift;
    my $valor_campo = shift;
    my $buffer      = shift;
    my $buffer_aux = $buffer;
    my %already_parsed = ();

    # se hace utf8::encode para aumentar la velocidad de procesamiento de las regexp
    # ya que no estamos buscando caracteres extendidos
    utf8::encode($buffer_aux);
    while ($buffer_aux =~ /%%$nom_campo((?:\((\d+)\))?\.(\w+)(?:\.(\w+))?)%%/sg) {
        my $marca_extension = $1;
        my $maxchars        = $2;
        my $type            = lc($3);
        my $type2            = lc($4);
        my $temp_match = lc("$nom_campo$marca_extension");
        # print STDERR "[$temp_match] - [$nom_campo][$marca_extension][$maxchars][$type][$type2]\n";

        # las marcas se parsean todas de una vez, si se vuelve a encontrar
        # la misma marca en el html, ya se proceso y es innecesario continuar
        if (defined($already_parsed{"$temp_match"})) {
            next;
        }

        my $valor_real = $valor_campo;
        if (($maxchars > 0) && ($valor_campo)) {
            $valor_real = &ajusta_nchars($valor_campo, $maxchars);
        }
        if ($type eq "code") {
            $valor_real = &get_codetext_value($valor_real);
        } elsif ($type eq "json") {
            $valor_real = &get_jsontext_value($valor_real);
        } elsif ($type eq "xml" ) {
            $valor_real = lib_utils::escape_xml($valor_real);
        } elsif ($type eq "strip" ) {
            $valor_real = &strip_text($valor_real);
        } elsif ($type eq "urlencode" ) {
            $valor_real = &urlencode_text($valor_real);
        } elsif ($type eq "raw") {
            my $minitext = &get_minitext_value($valor_real);
            if ($type2 eq "code") {
                $valor_real = &get_codetext_value($minitext);
            } elsif ($type2 eq "json") {
                $valor_real = &get_jsontext_value($minitext);
            } elsif ($type2 eq "xml" ) {
                $valor_real = lib_utils::escape_xml($minitext);
            } elsif ($type2 eq "strip" ) {
                $valor_real = &strip_text($minitext);
            } elsif ($type2 eq "urlencode" ) {
                $valor_real = &urlencode_text($minitext);
            } else {
                $valor_real = $minitext;
            }
        }

        $buffer =~ s/%%$nom_campo\Q$marca_extension\E%%/$valor_real/g;
        $already_parsed{"$temp_match"} = 1;
    }
    return $buffer;
}

# -------------------------------------------------------------------------#
sub parse_maxchars {
    my $nom_campo   = shift;
    my $valor_campo = shift;
    my $buffer      = shift;
    my $buffer_aux = $buffer;
    my %already_parsed = ();

    while ($buffer_aux =~ /%%$nom_campo(\(([0-9]*)\))%%/sg) {
        my $marca_maxchars = $1;
        my $maxchars       = $2;
        my $temp_match = lc("$nom_campo$marca_maxchars");

        # las marcas se parsean todas de una vez, si se vuelve a encontrar
        # la misma marca en el html, ya se proceso y es innecesario continuar
        if (defined($already_parsed{"$temp_match"})) {
            next;
        }

        if (($maxchars > 0) && ($valor_campo)) {
            $valor_campo = &ajusta_nchars($valor_campo, $maxchars);
        }
        $buffer =~ s/%%$nom_campo\Q$marca_maxchars\E%%/$valor_campo/g;
        $already_parsed{"$temp_match"} = 1;
    }
    return $buffer;
}

# -------------------------------------------------------------------------#
sub replace_in_xml {
    # tal vez esto debiera estar en Artic
    my ($xml_base, $nom_campo, $valor, $parse_as_cdata) = @_;

    # esto es necesario en html, no en xml
    # $valor =~ s/< *\?/< &#63;/g; # elimina secuencias <?

    if ($parse_as_cdata) {
        $valor = &escape_cdata($valor);
    } else {
        $valor = lib_utils::escape_xml($valor);
    }

    if ($nom_campo =~ /^fotofija_/) {
        # quita servers name de fotos para que sean relativas al sitio
        $valor =~ s/^https?\:\/\/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'}//;
        $valor =~ s/^https?\:\/\/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}//;
    }

    if ($parse_as_cdata) {
        if (!($xml_base =~ s/<$nom_campo>.*?<\/$nom_campo>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>/s)) {
            # campo empieza con _, es privado
            if (substr($nom_campo, 0, 1) eq '_') {
                $xml_base =~ s/<\/_private>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>\n<\/_private>/;
            } else {
                $xml_base =~ s/<\/_public>/<$nom_campo>\n<!\[CDATA\[$valor\]\]>\n<\/$nom_campo>\n<\/_public>/;
            }
        }
    } else {
        if (!($xml_base =~ s/<$nom_campo>.*?<\/$nom_campo>/<$nom_campo>$valor<\/$nom_campo>/s)) {
            # campo empieza con _, es privado
            if (substr($nom_campo, 0, 1) eq '_') {
                $xml_base =~ s/<\/_private>/<$nom_campo>$valor<\/$nom_campo>\n<\/_private>/;
            } else {
                $xml_base =~ s/<\/_public>/<$nom_campo>$valor<\/$nom_campo>\n<\/_public>/;
            }
        }
    }

    return $xml_base;
}

# -------------------------------------------------------------------------#
# Elimina tags y ret. de carro.
sub saca_tags_rets {

    my ($toencode) = $_[0];
    $toencode =~ s/[\r\n]+/ /sg;
    $toencode =~ s/<[^>]+>/ /sg;
    $toencode =~ s/ {2,}/ /sg;
    $toencode =~ s/ $//sg;
    $toencode =~ s/^ //sg;
    $toencode =~ s/"/'/g;
    $toencode =~ s/&quot;/'/g;
    my $inclinada = "´";
    $toencode =~ s/'/$inclinada/sg;        # cambia comilla vertical por comilla inclinada, para compatib. JS
    $toencode =~ s/&#39;/$inclinada/sg;    # cambia comilla vertical por comilla inclinada, para compatib. JS

    return $toencode;
}

# -------------------------------------------------------------------------
# Retorna el nombre del directorio de ediciones mas nuevo, el que corresp. al nombre de
# la edicion mas nueva.
# Param: 0) Path absoluto al dir de ediciones
sub get_last_edic {
    my $dir_hpage = $_[0];

    my @entries = glib_fildir_03::lee_dir($dir_hpage);
    #  Utiliza un hash auxiliar con las fecha de las ediciones en iso + el correlativo para ordenar.
    my %aux_sort;
    foreach my $entry (@entries) {
        $entry =~ /^(\d\d\d\d)\_(\d\d)\_(\d\d)\_(\d+)$/;
        my $aaaammddnro = $1 . $2 . $3 . $4;
        $aux_sort{$entry} = $aaaammddnro;
    }

    # Ordena numericamente de mayor a menor.
    @entries = sort { $aux_sort{$b} <=> $aux_sort{$a} } @entries;

    # Ordenar descendentemente.
    foreach my $entry (@entries) {
        if ($entry !~ /^\./g) {
            # Si es un dir valido
            if (-d "$dir_hpage/$entry") {
                return $entry;
            }
        }
    }
}

# ---------------------------------------------------------------
# Chequea existencia de dirs. de trabajo de la edicion seleccionada.
sub check_dirs_edic {
    my ($dir_dest_edic) = $_[0];
    my ($dir);

    # Dir de la edicion.
    if (!(glib_fildir_03::check_dir($dir_dest_edic))) {
        return "El directorio de la edición no es válido";
    }

    # Dir de portadas de la edicion.
    $dir = $dir_dest_edic . $prontus_varglb::DIR_PORT;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de portadas de la edición no es válido";
    }

    # destino del xml de las portadas de la edicion
    $dir =~ s/$prontus_varglb::DIR_PORT$/\/xml/;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio del xml de portadas de la edición no es válido";
    }

    # Dir de hpage de la edicion
    $dir = $dir_dest_edic . $prontus_varglb::DIR_HPAGES;
    if (!(glib_fildir_03::check_dir($dir))) {
        return "El directorio de homepage de la edición no es válido";
    }

    return '';
}

# ---------------------------------------------------------------
sub dev_tam_img {
    my $file = $_[0];
    my ($ancho, $alto, $msg, $orientation) = (0, 0, '', 0);

    if (!-e $file) {
        return ('Archivo no existe', 0, 0);
    } else {
        if ($file =~ /\.(jpg|jpe|jpeg)$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_jpg($file);
            # print STDERR "FILE[$file] ancho[$ancho] alto[$alto] MSG[$msg]\n";
            my $img_info = Image::ExifTool::ImageInfo($file, 'EXIF:Orientation#');
            $orientation = $img_info->{'Orientation #'}?$img_info->{'Orientation #'}:0;
        } elsif ($file =~ /\.gif$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_gif($file);
        } elsif ($file =~ /\.png$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_png($file);
        } elsif ($file =~ /\.svg$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_svg($file);
        } elsif ($file =~ /\.webp$/i) {
            ($msg, $ancho, $alto) = &ancho_alto_webp($file);
        } else {
            return ('Archivo [' . $file . '] no tiene extensión JPG/JPE/JPEG/PNG/GIF/BMP/SVG/WEBP', 0, 0);
        }

        return ($msg, $ancho, $alto, $orientation);
    }

    return ('Archivo no existe', 0, 0, $orientation);
}    # dev_tam_img.

# ---------------------------------------------------------------
sub ancho_alto_webp {
    my $file = $_[0];

    my $img_info = Image::ExifTool::ImageInfo($file, 'ImageWidth', 'ImageHeight');
    my ($width, $height, $msg) = ('', '', '');

    if ($img_info) {
        $width = $img_info->{'ImageWidth'};
        $height = $img_info->{'ImageHeight'};
    } else {
        $msg = "Error leyendo información de la imagen.";
    }

    return ($msg, $width, $height);
}

# ---------------------------------------------------------------
# (\d+)[\d.]*, rescarta el primer(os) numero(s) y descarta . y decimales si existen
sub ancho_alto_svg {
    my ($file) = $_[0];
    my $svg = glib_fildir_03::read_file($file);
    my ($w, $h, $msg) = '';

    if ($svg =~ /<svg([^>]*)/) {
        my $svg_props = $1;

        if ($svg_props =~ /width=["|'](\d+)[\d.]*["|']/) {
            $w = $1;
        }

        if ($svg_props =~ /height=["|'](\d+)[\d.]*["|']/) {
            $h = $1;
        }

        if (!$w || !$h) {
            if ($svg_props =~ /viewBox="(\d+)[\d.]* (\d+)[\d.]* (\d+)[\d.]* (\d+)[\d.]*"/) {
                $w = $3;
                $h = $4;
            }
        }
    }

    $msg = 'error leyendo dimenciones de svg.' if (!$w || !$h);

    return ($msg, $w, $h);
};

# ---------------------------------------------------------------
sub ancho_alto_png {
    # Adaptacion de http://www.la-grange.net/2000/05/04-png.html
    my ($file) = $_[0];
    my ($head) = "";

    (open my $png, '<', $file) || return ("No se puede abrir archivo PNG!", 0, 0);
    binmode $png;

    my ($a, $b, $c, $d, $e, $f, $g, $h) = 0;

    if (   defined($png)
        && read($png, $head, 8) == 8
        && $head eq "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a"
        && read($png, $head, 4) == 4
        && read($png, $head, 4) == 4
        && $head eq "IHDR"
        && read($png, $head, 8) == 8)
    {
        ($a, $b, $c, $d, $e, $f, $g, $h) = unpack("C" x 8, $head);
        close $png;
        return ('', $a << 24 | $b << 16 | $c << 8 | $d, $e << 24 | $f << 16 | $g << 8 | $h);
    }
    close $png;
    return ('error reading png', 0, 0);
}

# ---------------------------------------------------------------
sub ancho_alto_jpg {
    my $file = $_[0];
    my ($c1, $c2, $ch, $s, $junk, $a, $b, $c, $d, $length);
    (open my $jpeg, '<', $file) || return ("No se puede abrir archivo JPEG!", 0, 0);
    binmode $jpeg;
    read($jpeg, $c1, 1);
    read($jpeg, $c2, 1);
    ((ord($c1) == 0xFF) && (ord($c2) == 0xD8)) || return ("Este archivo no es un JPEG!", 0, 0);

    my $done = 0;
    my $dummy;

    while (ord($ch) != 0xDA) {
        # Find next marker (JPEG markers begin with 0xFF)
        while (ord($ch) != 0xFF) {
            return ('error reading jpg', 0, 0) unless read($jpeg, $ch, 1);
        }
        # JPEG markers can be padded with unlimited 0xFF's
        while (ord($ch) == 0xFF) {
            return ('error reading jpg', 0, 0) unless read($jpeg, $ch, 1);
        }
        # Now, $ch contains the value of the marker.
        if ((ord($ch) >= 0xC0) && (ord($ch) <= 0xC3)) {
            return ('Error de formato en jpg', 0, 0) unless read($jpeg, $dummy, 3);
            return ('Error de formato en jpg', 0, 0) unless read($jpeg, $s,     4);
            ($a, $b, $c, $d) = unpack("C" x 4, $s);
            return ('', $c << 8 | $d, $a << 8 | $b);

        } else {
            # We **MUST** skip variables, since FF's within variable
            # names are NOT valid JPEG markers
            return ('error reading jpg', 0, 0) unless read($jpeg, $s, 2);
            ($c1, $c2) = unpack("C" x 2, $s);
            $length = $c1 << 8 | $c2;
            if (!defined($length) || $length < 2) {
                close($jpeg);
                return ("Longitud del marcador JPEG errónea!", 0, 0);
            }

            read($jpeg, $dummy, $length - 2);
        }
    }
}    # ancho_alto_jpg.

# ---------------------------------------------------------------
sub ancho_alto_gif {
    my ($file) = $_[0];
    my ($a, $b, $c, $d, $s, $type);
    (open my $gif, '<', $file) || return ("No se puede abrir archivo GIF!", 0, 0);
    binmode $gif;
    read($gif, $type, 6);
    $type =~ /GIF8[7,9]a/ || return ("Archivo GIF inválido!", 0, 0);
    read($gif, $s, 4) == 4 || return ("Archivo GIF corrupto!", 0, 0);
    close($gif);
    ($a, $b, $c, $d) = unpack("C" x 4, $s);
    return ('', ($b << 8 | $a), ($d << 8 | $c));
}    # ancho_alto_gif.

# ---------------------------------------------------------------
sub ancho_alto_bmp {
    my ($file) = $_[0];
    my ($a, $b, $c, $d, $e, $f, $g, $h, $s, $type);
    my $bmp;
    (open $bmp, '<', $file) || return ("No se puede abrir archivo BMP!", 0, 0);
    binmode $bmp;
    read($bmp, $type, 2);
    $type =~ /BM/ || return ("Archivo BMP inválido!", 0, 0);
    read($bmp, $type, 12);
    read($bmp, $s, 4) == 4 || return ("Archivo BMP corrupto!", 0, 0);
    $a = unpack("C" x 1, $s);

    if ($a - 4 == 12) {
        read($bmp, $s, 4) == 4 || return ("Archivo BMP corrupto!", 0, 0);
        ($a, $b, $c, $d) = unpack("C" x 4, $s);
        close($bmp);
        return ('', ($b << 8 | $a), ($d << 8 | $c));
    } elsif ($a - 4 > 12) {
        read($bmp, $s, 8) == 8 || return ("Archivo BMP corrupto!", 0, 0);
        ($a, $b, $c, $d, $e, $f, $g, $h) = unpack("C" x 8, $s);
        close($bmp);
        return ('', ($b << 8 | $a | $d << 8 | $c), ($f << 8 | $e | $h << 8 | $g));
    } else {
        close($bmp);
        return ("Archivo BMP inválido!", 0, 0);
    }
}    # ancho_alto_bmp.

sub can_edit_img {
    my $img_type = $_[0];

    # Si no existe en el hash, se puede manipular.
    if (!defined $prontus_varglb::IMG_TYPE_NOEDIT{$img_type}) {
        return 1;
    }

    return 0;
}

# ---------------------------------------------------------------
# Borra caches de portadas taxonomicas asociadas al art.
# para forzar que se vuelvan a generar.
sub borra_taxport {
    my ($secc, $tem, $stem, $vista) = @_;
    # elimina portadas taxonomicas
    if ($secc) {
        my $dir_taxport = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
        if ($vista) {    # nombre de la vista.
            $dir_taxport = $dir_taxport . '-' . $vista;
        }

        my $taxport = $dir_taxport . "/$secc" . '_' . $tem . '_' . $stem . '_';
        unlink glob($taxport . '*');
    }
}

# ---------------------------------------------------------------
# Devuelve la string $buffer con las marcas _FILEURL* reemplazadas por las URL friendly correctas.
# 20180720 - AVP - el ultimo parametro es un hash con los datos del articulo necesarios para la f2 f3 y f4
sub parse_filef {
    my ($buffer, $titular, $ts, $prontus_id, $relpath_artic) = @_;

    my %artic_data = ();
    if (defined($_[5]) && ref($_[5]) eq ref {}) {
        %artic_data  = %{$_[5]};
    }

    return $buffer if (!$ts || !$prontus_id || !$titular);
    return $buffer unless($buffer =~ /%%_fileurl(\(\w+\))?%%/);

    # Ajusta titular para usar en url
    $titular = &ajusta_titular_friendly($titular);
    my $ext = '';
    $ext = $1 if ($relpath_artic =~ /\.(\w+)$/);
    my $fecha4friendly = '';
    my $fecha = '';
    my $hora = '';

    if ($ts =~ /^(\d{4})(\d{2})(\d{2})(\d{6})/) {
        $fecha4friendly = $1 . '-' . $2 . '-' . $3;
        $fecha          = "$1$2$3";
        $hora           = $4;
    } else {
        return $buffer;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq 'N') {
        # Links normal, no friendly
        my $file = "/$prontus_id/site/artic/$fecha/pags/$ts.$ext";
        $buffer =~ s/%%_fileurl%%/$file/g;
        $buffer =~ s/%%_fileurl\(\w+\)%%/$file/g;
        $buffer =~ s/%%_fileurl_canon%%/$file/g;
    } else {
        my $prontus_proto = '';
        my $fileurl = '';
        my $fileurl_proto = '';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '1') {
            $fileurl = "/$titular/$prontus_id/$fecha4friendly/$hora.$ext";
        } else {
            my $tax = '';
            if (defined($artic_data{'nom_seccion'}) && $artic_data{'nom_seccion'} ne '') {
                $artic_data{'nom_seccion'} = &despulgar_texto_friendly($artic_data{'nom_seccion'});
                $tax = "/$artic_data{'nom_seccion'}";
                # no hay tema sin seccion
                if (defined($artic_data{'nom_tema'}) && $artic_data{'nom_tema'} ne '') {
                    $artic_data{'nom_tema'} = &despulgar_texto_friendly($artic_data{'nom_tema'});
                    $tax .= "/$artic_data{'nom_tema'}";
                    # no hay subtema sin tema
                    if (defined($artic_data{'nom_subtema'}) && $artic_data{'nom_subtema'} ne '') {
                        $artic_data{'nom_subtema'} = &despulgar_texto_friendly($artic_data{'nom_subtema'});
                        $tax .= "/$artic_data{'nom_subtema'}";
                    }
                }
            }

            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '2') {
                # Formato: /prontus/seccion/tema/subtema/titular/aaaa-mm-dd/hhnnss.extension
                $fileurl = "/$prontus_id$tax/$titular/$fecha4friendly/$hora.$ext";

            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '3') {
                # Formato: www.<prontus_id>.cl/seccion/tema/subtema/titular/aaaa-mm-dd/hhnnss.extension
                $fileurl = "$tax/$titular/$fecha4friendly/$hora.$ext";

            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '4') {
                # Formato: /prontus/seccion/tema/subtema/titular.extension
                # si el fid esta excluido de friendly 4, se genera url directa
                if (defined($artic_data{'fid'}) && !exists $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$artic_data{'fid'}}) {
                    # si se debe usar url custom
                    if (defined($artic_data{'custom_slug'}) && $artic_data{'custom_slug'} eq 'SI') {
                        # rescatamos el slug, si no existe usamos el titular de forma normal
                        if (defined($artic_data{'slug'}) && $artic_data{'slug'} ne '') {
                            $titular = &ajusta_titular_friendly($artic_data{'slug'});
                        }
                    }

                    my $vista = '';
                    # obtenemos la vista para la url
                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
                        if ($relpath_artic =~ /\/pags\-(\w+)\// ) {
                            $vista = "/$1";
                        }
                    }

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} eq 'SI') {
                        $prontus_proto = "/$prontus_id";
                    }

                    # evita casos en que la ultima taxonomia y el titular son iguales
                    if ($tax ne '' && $tax =~ /\/$titular$/) {
                        $fileurl = "$prontus_proto$vista$tax";
                        $fileurl_proto = $tax;
                    } else {
                        $fileurl = "$prontus_proto$vista$tax/$titular";
                        $fileurl_proto = "$tax/$titular";
                    }

                } else {
                    $fileurl = "/$prontus_id/site/artic/$fecha/pags/$ts.$ext";
                }
            } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
                my $vista;

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
                    if ($relpath_artic =~ /\/pags\-(\w+)\// ) {
                        $vista = "/$1";
                    }
                }

                if (defined($artic_data{'fid'}) && !exists $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$artic_data{'fid'}}) {
                    # Si no existe la url se muestra la ruta directa al archivo.
                    if (defined($artic_data{'_url'})) {
                        $fileurl = $vista . $artic_data{'_url'};
                        $fileurl_proto = $artic_data{'_url'};
                    } else {
                        $fileurl = "/$prontus_id/site/artic/$fecha/pags/$ts.$ext";
                        $fileurl_proto = $fileurl;
                    }
                } else {
                    $fileurl = "/$prontus_id/site/artic/$fecha/pags/$ts.$ext";
                    $fileurl_proto = $fileurl;
                }
            }
        }
        #print STDERR "fileurl[$fileurl] prontus_proto[$prontus_proto] fileurl_proto[$fileurl_proto]\n";

        $buffer =~ s/%%_fileurl%%/$fileurl/g;    # Links friendly
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '4' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
            $buffer =~ s/%%_fileurl\((\w+)\)%%/$prontus_proto\/$1$fileurl_proto/g; # Links friendly con vista sin prontus
            $buffer =~ s/%%_fileurl_canon%%/$prontus_proto$fileurl_proto/g;    # link de url canonica, vista principal
        } elsif ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI') {
            $buffer =~ s/%%_fileurl\((\w+)\)%%/\/$1$fileurl_proto/g; # Links friendly con vista sin prontus
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        } else {
            $buffer =~ s/%%_fileurl\(\w+\)%%/$fileurl/g; # Links friendly
            $buffer =~ s/%%_fileurl_canon%%/$fileurl/g;    # Links friendly
        }
    }

    return $buffer;
}

# ---------------------------------------------------------------
# Funcion para ajustar el titular para friendly url
sub ajusta_titular_friendly {
    my $titular = $_[0];
    # recorta largo del titular
    $titular = &ajusta_nchars($titular, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'});
    # elimina todos los puntos al final del titular
    $titular =~ s/\.+$//sg;

    return &despulgar_texto_friendly($titular);
}

# ---------------------------------------------------------------
sub despulgar_texto_friendly {
    my $texto = $_[0];
    $texto = &saca_tags_rets($texto);
    $texto = &notildes($texto);
    $texto = lc $texto;
    $texto =~ s/[^a-z0-9]+/-/sg;
    $texto =~ s/-{2,}/-/g;
    # elimina todos los guiones al principio
    $texto =~ s/^-+//sg;
    # elimina todos los guiones al final
    $texto =~ s/-+$//sg;

    return $texto;
}

#------------------------------------------------------------------------------#
sub bytes2kb {
    my $bytes = $_[0];
    $bytes = $bytes / 1024;
    $bytes = 0 if ($bytes < 0);
    $bytes = &format_miles_dec($bytes, 0);
    $bytes .= ' KiB';
    return $bytes;
}

# ------------------------------------------------------------------------------------- #
sub format_bin_bytes {
    my ($espacio_final) = @_;
    my $unidad        = ' KiB';
    $espacio_final = $espacio_final / 1024;

    if ($espacio_final >= 1024) {
    # Conversion a Megas.
        $espacio_final = $espacio_final / 1024;
        $unidad        = ' MiB';
        if ($espacio_final >= 1024) {
            $espacio_final = $espacio_final / 1024;
            $unidad        = ' GiB';
            if ($espacio_final >= 1024) {
                $espacio_final = $espacio_final / 1024;
                $unidad        = ' TiB';
            }
        }
    }

    $espacio_final = &format_number($espacio_final);
    if ($espacio_final) {
        return $espacio_final . $unidad;
    } else {
        return '0 B';
    }
}

# ------------------------------------------------------------------------------------- #
sub format_number {
    my ($number) =  @_;
    $number =~ s/[^\d\.\,]//g;    # se limpia el numero

    # Verifica si tiene decimales, si tiene mas que dos elimina el resto.
    if ($number =~ /^(\d+)\.(\d+)$/) {
        my $entero    = $1;
        my $decimales = $2;
        if (length($decimales) > 1) {
            if (length($entero) < 2) {
                $decimales = substr($decimales, 0, 2);
            } else {
                $decimales = substr($decimales, 0, 1);
            }
            if ($decimales) {
                $number = $entero . ',' . $decimales;
            } else {
                $number = $entero;
            }
        }
    }
    return $number;
}
#------------------------------------------------------------------------------#
# Recibe numero real y lo formatea con separadores de miles y cant. de decimales deseada.
sub format_miles_dec {
    my ($valor1, $valor2, $dec) = ($_[0], $_[0], $_[1]);
    # print "valor1 s/f:[$valor1]"; # debug

    my $signo = '';
    if ($valor1 < 0) {
        $signo = '-';
    }

    $valor1 = $valor2 = abs $valor1;
    # Si el valor hay que devolverlo como entero sin decimales.
    # $valor1 += 1 if ($valor2 - int($valor2) and $dec == 0) >= 0.5;  # 03.2

    # 03.2
    if ($dec == 0) {
        if ($valor1 =~ /\.(\d)/) {
            my $d = $1;
            $valor1 += 1 if ($d >= 5);
        }
    }

    # Si el valor hay que devolverlo como entero con decimales.
    $valor1 += '0.' . '0' x ($dec - 1) . (1 - ($valor2 - int($valor2))) if ($valor2 - int($valor2) and $dec > 0) >= 0.5;
    $valor2 = sprintf("%.${dec}f", $valor2);
    $valor2 =~ s/^\d+\.//g;

    $valor1 =~ s/\..*//g;

    $valor1 = glib_str_03::format_miles($valor1);    # 03.2

    if ($dec > 0) {
        $valor1 .= ',' . $valor2;
    }

    if ($signo eq '-') {
        $valor1 = '-' . $valor1;
    }

    # print "valor1 c/f:[$valor1]<br>"; # debug
    return $valor1;
}

# ---------------------------------------------------------------
# aaaammddhhmmss  -> Mon, 24 Apr 2006 16:52:32 GMT
sub get_dtime_rfc822 {
    my ($aaaammddhhmmss) = shift;
    return '' if ($aaaammddhhmmss =~ /^9999/);
    if ($aaaammddhhmmss =~ /^(\d{8})(\d{2})(\d{2})(\d{2})/) {
        my ($aaaammdd, $hh, $mm, $ss) = ($1, $2, $3, $4);
        my $dtime_rfc822 = &fecha2rfc822($aaaammdd, $hh, $mm, $ss);
        return $dtime_rfc822;
    } else {
        return '';
    }
}

# ---------------------------------------------------------------
# Obtiene hhmmss a partir de hh:mm
sub get_hora_iso {
    my $horap = shift;
    my $horap_iso;
    if ($horap =~ /^(\d\d):(\d\d)$/) {
        $horap_iso = $1 . $2 . '00';
    } else {
        $horap_iso = '0' x 6;
    }
    return $horap_iso;
}

# ---------------------------------------------------------------
# Toma una fecha en iso y la pone como "Mon, 24 Apr 2006 16:52:32 GMT"
# Param : string con la fecha ISO.
sub fecha2rfc822 {
    my ($fecha, $hh, $mm, $ss) = @_;

    my (@dias) = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
    my (@meses) = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
    $fecha =~ /(\d\d\d\d)(\d\d)(\d\d)/;
    my ($dia, $mes, $ano) = ($3, $2, $1);
    my ($tiempo, $tiempo2);

    $tiempo = Time::Local::timelocal_nocheck($ss, $mm, $hh, $dia, ($mes - 1), ($ano - 1900)) || return '';
    $tiempo2 = Time::Local::timegm_nocheck($ss, $mm, $hh, $dia, ($mes - 1), ($ano - 1900)) || return '';

    $tiempo = $tiempo + ($tiempo - $tiempo2);

    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($tiempo);
    $year = $year + 1900;

    $sec  = glib_str_03::format_n($sec,  2);
    $min  = glib_str_03::format_n($min,  2);
    $hour = glib_str_03::format_n($hour, 2);

    return $dias[$wday] . ", $mday " . $meses[$mon] . " $year $hour:$min:$sec GMT";
}

# -------------------------------------------------------------------------
# Detecta y pone en ISO los caracteres escapeados en html, pero solo para las letras con tildes.
sub mini_unescape_html {
    my ($toencode) = $_[0];

    $toencode =~ s/&Agrave;/\xC0/g;
    $toencode =~ s/&Aacute;/\xC1/g;
    $toencode =~ s/&Acirc;/\xC2/g;
    $toencode =~ s/&Atilde;/\xC3/g;
    $toencode =~ s/&Auml;/\xC4/g;
    $toencode =~ s/&Aring;/\xC5/g;
    $toencode =~ s/&AElig;/\xC6/g;
    $toencode =~ s/&Ccedil;/\xC7/g;
    $toencode =~ s/&Egrave;/\xC8/g;
    $toencode =~ s/&Eacute;/\xC9/g;
    $toencode =~ s/&Ecirc;/\xCA/g;
    $toencode =~ s/&Euml;/\xCB/g;
    $toencode =~ s/&Igrave;/\xCC/g;
    $toencode =~ s/&Iacute;/\xCD/g;
    $toencode =~ s/&Icirc;/\xCE/g;
    $toencode =~ s/&Iuml;/\xCF/g;
    $toencode =~ s/&Ntilde;/\xD1/g;
    $toencode =~ s/&Ograve;/\xD2/g;
    $toencode =~ s/&Oacute;/\xD3/g;
    $toencode =~ s/&Ocirc;/\xD4/g;
    $toencode =~ s/&Otilde;/\xD5/g;
    $toencode =~ s/&Ouml;/\xD6/g;

    $toencode =~ s/&Oslash;/\xD8/g;
    $toencode =~ s/&Ugrave;/\xD9/g;
    $toencode =~ s/&Uacute;/\xDA/g;
    $toencode =~ s/&Ucirc;/\xDB/g;
    $toencode =~ s/&Uuml;/\xDC/g;
    $toencode =~ s/&Yacute;/\xDD/g;
    $toencode =~ s/&szlig;/\xDF/g;
    $toencode =~ s/&agrave;/\xE0/g;
    $toencode =~ s/&aacute;/\xE1/g;
    $toencode =~ s/&acirc;/\xE2/g;
    $toencode =~ s/&atilde;/\xE3/g;
    $toencode =~ s/&auml;/\xE4/g;
    $toencode =~ s/&aring;/\xE5/g;
    $toencode =~ s/&aelig;/\xE6/g;
    $toencode =~ s/&ccedil;/\xE7/g;
    $toencode =~ s/&egrave;/\xE8/g;
    $toencode =~ s/&eacute;/\xE9/g;
    $toencode =~ s/&ecirc;/\xEA/g;
    $toencode =~ s/&euml;/\xEB/g;
    $toencode =~ s/&igrave;/\xEC/g;
    $toencode =~ s/&iacute;/\xED/g;
    $toencode =~ s/&icirc;/\xEE/g;
    $toencode =~ s/&iuml;/\xEF/g;
    $toencode =~ s/&eth;/\xF0/g;
    $toencode =~ s/&ntilde;/\xF1/g;
    $toencode =~ s/&ograve;/\xF2/g;
    $toencode =~ s/&oacute;/\xF3/g;
    $toencode =~ s/&ocirc;/\xF4/g;
    $toencode =~ s/&otilde/\xF5;/g;
    $toencode =~ s/&ouml;/\xF6/g;
    $toencode =~ s/&oslash;/\xF8/g;
    $toencode =~ s/&ugrave/\xF9;/g;
    $toencode =~ s/&uacute;/\xFA/g;
    $toencode =~ s/&ucirc;/\xFB/g;
    $toencode =~ s/&uuml;/\xFC/g;
    $toencode =~ s/&brvbar;/\xFD/g;
    $toencode =~ s/&yuml;/\xFF/g;

    return $toencode;
}

# --------------------------------------------------------------------
# parsea invocacion a funciones definidas en lib_custom.pm
sub parser_custom_function {
    my $buffer       = $_[0];

    if (!$PF_ENABLED) {
        # Avisa del error solo en caso de que las p functions se esten utilizando.
        if ($buffer =~ /%%_pf_\w+\(.*?\)%%/s) {
            print STDERR "Error cargando lib_custom.pm\n";
            $buffer =~ s/%%_pf_\w+\(.*?\)%%/<h2>Error cargando Prontus Functions<h2>/sg;
        }
    } else {
        while ($buffer =~ /%%(_pf_(\w+)\((.*?)\))%%/sg) {
            my $match = $1;
            my $function = $2;
            my $params   = $3;

            my @args = ();
            # Para obtener los parametros de entrada
            if ($params ne '') {
                $params =~ s/^\s*["']//;
                $params =~ s/["']\s*$//;

                my @arrparams = split(/"\s*,\s*"|'\s*,\s*'/, $params);
                foreach my $keys (@arrparams) {
                    # borra marcas prontus
                    $keys =~ s/%%(.*?)%%//g;
                    push(@args, $keys);
                }
            }

            # devuelve CODE, pero si no existe se cae
            # print STDERR ref(\&{lib_custom->${\$function}}) ."\n";
            # devuelve CODE(0x337ca78), direccion de memoria, no se cae si no existe
            # y retorna undef sirve para probar con defined
            # no estamos usando objetos pero es alternativa para probar
            # print STDERR lib_custom->can($function) ."\n";

            # esta forma permite invocar como funcion sin romper el funcionamiento anterior
            my $action = "lib_custom::$function";
            if (defined(&{$action} )) {
                # print STDERR "Ejecutar lib_custom::$function\n";
                my $result = &{\&{"lib_custom::$function"}}(@args);

                if (!($buffer =~ s/%%\Q$match\E%%/$result/s)) {
                    print STDERR "[FAIL] reemp[%%$match%%] x result[$result]\n";
                }
            } else {
                print STDERR "Error: no existe lib_custom::$function\n";
            }
        }
    }
    return $buffer;
}

# ---------------------------------------------------------------
sub despulga_item_tax {                         # rotulos tax
    my $str = $_[0];
    $str =~ s/["']//g;
    $str =~ s/\t|\r|\n/ /sig;
    $str =~ s/\\/ /sg;
    $str =~ s/ +/ /g;
    $str =~ s/^ +//g;
    $str =~ s/ +$//g;
    return $str;
}

sub despulga_item_tag {                         # rotulos tag
    my $str = $_[0];
    $str =~ s/["']//g;
    $str =~ s/[\t\r\n\,\|\>\<]/ /sg;            # la [,|<>] se estripean por req. de la gestion de tags en el FID
    $str =~ s/\\/ /sg;
    $str =~ s/ +/ /g;
    $str =~ s/^ +//g;
    $str =~ s/ +$//g;
    return $str;
}

# ---------------------------------------------------------------
sub get_nomtax_envista {
    my $mv         = shift;
    my $nom4vistas = shift;
    my $nom_default = shift;

    $nom_default = '' if (!defined($nom_default));

    if ($nom4vistas =~ /(^|\n)$mv\t(.*?)($|\n)/s) {
        # necesita ser escapeada html en destino antes de usar.
        return $2;
    } else {
        # necesita ser escapeada html en destino antes de usar.
        return $nom_default;
    }
}

# ---------------------------------------------------------------
sub get_nom4vistas {
    my ($mv, $id_s, $id_t, $id_st) = @_;

    my ($nom_s, $nom_t, $nom_st) = ('', '', '');
    my ($secc_nom4vistas, $temas_nom4vistas, $subtemas_nom4vistas) = ('', '', '');

    $mv = 'default' if (!$mv);

    # Conectar a BD si es que no hay conexion
    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        print STDERR "ERROR: $msg_err_bd\n";
    }

    my $salida;
    my @result;

    # nombres en la vista dada
    if ($id_s) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv})) {
            my @params = ($id_s);
            my $sql = "select SECC_NOM, SECC_NOM4VISTAS from SECC where SECC_ID = ?";
            $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_s, $secc_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_s);
            glib_str_03::correct_encode_utf8_string($secc_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{'default'} = $nom_s;

            my @vistas = split(/\n/, $secc_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'seccion'}{$id_s}{$mv});
    } else {
        push (@result, undef);
    }

    if ($id_t) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv})) {
            my @params = ($id_t);
            my $sql = "select TEMAS_NOM, TEMAS_NOM4VISTAS from TEMAS where TEMAS_ID = ?";
            $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_t, $temas_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_t);
            glib_str_03::correct_encode_utf8_string($temas_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{'default'} = $nom_t;

            my @vistas = split(/\n/, $temas_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'tema'}{$id_t}{$mv});
    } else {
        push (@result, undef);
    }

    if ($id_st) {
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv})) {
            my @params = ($id_st);
            my $sql = "select SUBTEMAS_NOM, SUBTEMAS_NOM4VISTAS from SUBTEMAS where SUBTEMAS_ID = ?";
            my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom_st, $subtemas_nom4vistas));
            $salida->fetch;
            $salida->finish;
            glib_str_03::correct_encode_utf8_string($nom_st);
            glib_str_03::correct_encode_utf8_string($subtemas_nom4vistas);

            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{'default'} = $nom_st;

            my @vistas = split(/\n/, $subtemas_nom4vistas);
            foreach my $vista (@vistas) {
                my @temp = split(/\t/, $vista);
                $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$temp[0]} = $temp[1];
            }
        }
        # si sigue vacia usa nombre de la vista principal
        if (!defined($prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv})) {
            $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv} = $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{'default'};
        }
        push (@result, $prontus_varglb::cache_nom4vista{$prontus_varglb::PRONTUS_ID}{'subtema'}{$id_st}{$mv});
    } else {
        push (@result, undef);
    }

    return @result;
}

# ---------------------------------------------------------------
# Valida xml bien formado.
sub valid_xml {                   # rotulos tax
    my ($buffer) = $_[0];
    my $p = new XML::Parser();
    eval { $p->parse($buffer) };
    if ($@) {
        $@ =~ s/ at \/.*$//;
        $@ =~ s/^(\n|\r)*//;
        return $@;
    }
    return '';                    # xml esta ok
}

# ---------------------------------------------------------------
# Valida xml bien formado para la importacion
sub valid_xml_import {
    my ($buffer) = $_[0];
    $buffer =~ s/&/&amp;/g;
    my $p = new XML::Parser();
    eval { $p->parse($buffer) };
    if ($@) {
        $@ =~ s/ at \/.*$//;
        $@ =~ s/^(\n|\r)*//;
        return $@;
    }
    return '';    # xml esta ok

}

# ---------------------------------------------------------------
# Asigna valores por defecto a la fechap y fechae cuando estos vienen vacios desde el fid.
# Para fechap vacio:
#    Si hay control fecha => poner 99999999
#    Si no hay control fecha => poner fecha de hoy en iso
# Para fechae vacio:
#    Siempre poner 99999999 (independiente del ctrl fecha)
sub asigna_fecha_default {
    my ($nom_campo) = lc $_[0];
    my ($fecha_default);

    if ($nom_campo eq '_fechap') {
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $fecha_default = '9' x 8;
        } else {
            $fecha_default = glib_hrfec_03::get_date_pack4();
        }
    }

    if ($nom_campo eq '_fechae') {
        $fecha_default = '9' x 8;
    }
    return $fecha_default;
}

# ---------------------------------------------------------------
sub artic_parser_fechas {
    my ($buffer, $fechap, $fechae) = @_;

    my %fechas;
    $fechas{'_fechap'} = $fechap;
    $fechas{'_fechae'} = $fechae;
    my ($iso_fechap, $iso_fechae);

    foreach my $nom_campo (keys %fechas) {
        my $val_campo = $fechas{$nom_campo};
        $val_campo = &asigna_fecha_default($nom_campo) if ($val_campo eq '');

        # sustitucion en la pagina
        $buffer =~ s/%%$nom_campo%%/$val_campo/g;

        # Realizar sustituciones especiales.
        $buffer = &parser_fechap($buffer, $val_campo, $nom_campo);

        # Parsear _NOM_EDIC en base a la fechap
        if ($nom_campo eq '_fechap') {
            if ($val_campo =~ /^(\d\d\d\d)(\d\d)(\d\d)/) {
                my $nomedic_mula = $1 . '_' . $2 . '_' . $3 . '_1';
                $buffer = &parse_tag_prontus($nomedic_mula, '_nom_edic', $buffer);
            }
        }

    };    # foreach fechas

    return $buffer;
}
# ---------------------------------------------------------------
# Revisa si el reg. ya existe en la BD.
sub existe_registro {
    my $sql     = $_[0];
    my $basedat = $_[1];
    my @data_array = $_[2];
    my $campo;

    my $salida;
    if (defined($_[2])) {
        $salida = glib_dbi_03::ejecutar_sql_bind_param($basedat, $sql, \@data_array, \($campo));
    } else {
        $salida = glib_dbi_03::ejecutar_sql_bind($basedat, $sql, \($campo));
    }
    $salida->fetch;
    $salida->finish;
    if ($campo ne '') {
        glib_str_03::correct_encode_utf8_string($campo);
        return $campo;
    } else {
        return 0;
    }
}

# --------------------------------------------------------------------
# Descripcion:
# Retorna un hash con pares (<nombre_del_campo>, <valor_del_campo>)
# en base a un buffer xml dado.
#
# Parametros de entrada:
# $xmlData : buffer xml, obligatorio
# $campos  : lista de campos a cargar, separados por comas, optativo
#            Si no viene, se cargan todos los campos en el hash.
#
# Retorna:
# %camposHash : hash con los campos cargados.
sub getCamposXml {
    my ($xmlData, $campos) = @_;
    my %camposHash;
    if ($campos) {
        my @camposLista = split(/, */, $campos);
        foreach my $nombreCampo (@camposLista) {
            if ($xmlData =~ /<$nombreCampo>(.+?)<\/$nombreCampo>/is) {
                my $campoValor = $1;
                if ($campoValor =~ /<!\[CDATA\[(.*?)\]\]>/is) {
                    $campoValor = $1;
                }
                # campos reservados pueden tener valor vacío
                $camposHash{lc $nombreCampo} = $campoValor if ($campoValor || substr($nombreCampo, 0, 1) eq '_');
            }
        }
    } else {
        while ($xmlData =~ /<([\w\.]+?)>(.*?)<\/\1>/sg) {
            my $nombreCampo = lc($1);
            my $campoValor  = $2;

            if ($nombreCampo =~ /^fotofija_\w+$/) {
                #<fotofija_art200>
                #<_wfotofija_art200>468</_wfotofija_art200>
                #<_hfotofija_art200>60</_hfotofija_art200>
                #<![cdata[http://es.lipsum.com/images/black_468x60.gif]]>
                #</fotofija_art200>

                if ($campoValor =~ /<_w$nombreCampo>(.+?)<\/_w$nombreCampo>/i) {
                    $camposHash{"_w$nombreCampo"} = $1;
                }
                if ($campoValor =~ /<_h$nombreCampo>(.+?)<\/_h$nombreCampo>/i) {
                    $camposHash{"_h$nombreCampo"} = $1;
                }
            }

            if ($campoValor =~ /<!\[CDATA\[(.*?)\]\]>/s) {
                $campoValor = $1;
            }

            if ($nombreCampo =~ /^foto_\w+$/) {
                if ($campoValor) {
                    $camposHash{$nombreCampo} = 1;
                    #<foto_00000249>
                    #    <_nomfoto_00000249>foto_0000024920200807223546_IMG_20160708_145344637.jpg</_nomfoto_00000249>
                    #    <_wfoto_00000249>4608</_wfoto_00000249>
                    #    <_hfoto_00000249>2592</_hfoto_00000249>
                    #    <_bfoto_00000249>1684270</_bfoto_00000249>
                    #    <_mfoto_00000249>1602887775</_mfoto_00000249>
                    #    <_ofoto_00000249>1</_ofoto_00000249>
                    #</foto_00000249>

                    if ($campoValor =~ /<_nom$nombreCampo>(.+?)<\/_nom$nombreCampo>/i) {
                        $camposHash{"_nom$nombreCampo"} = $1;
                    }
                    if ($campoValor =~ /<_w$nombreCampo>(.+?)<\/_w$nombreCampo>/i) {
                        $camposHash{"_w$nombreCampo"} = $1;
                    }
                    if ($campoValor =~ /<_h$nombreCampo>(.+?)<\/_h$nombreCampo>/i) {
                        $camposHash{"_h$nombreCampo"} = $1;
                    }
                    if ($campoValor =~ /<_b$nombreCampo>(.+?)<\/_b$nombreCampo>/i) {
                        $camposHash{"_b$nombreCampo"} = $1;
                    }
                    if ($campoValor =~ /<_m$nombreCampo>(.+?)<\/_m$nombreCampo>/i) {
                        $camposHash{"_m$nombreCampo"} = $1;
                    }
                    if ($campoValor =~ /<_o$nombreCampo>(.+?)<\/_o$nombreCampo>/i) {
                        $camposHash{"_o$nombreCampo"} = $1;
                    }
                }
            } else {
                # campos reservados pueden tener valor vacío
                $camposHash{$nombreCampo} = $campoValor if ($campoValor ne '' || substr($nombreCampo, 0, 1) eq '_');
            }
        }
    }

    return %camposHash;
}

# -------------------------------------------------------------------------#
# Reemplaza tildes por letras normales.
sub notildes {
    my $toencode = $_[0];

    $toencode =~ tr/áéíóúÁÉÍÓÚüÜñÑ/aeiouaeiouuunn/;    # Destilda ISO.

    $toencode =~ s/&(.)acute;/$1/g;
    $toencode =~ s/&(.)tilde;/$1/g;
    $toencode =~ s/&(.)uml;/$1/g;
    $toencode =~ s/&(.)circ;/$1/g;
    $toencode =~ s/&(.)grave;/$1/g;

    $toencode =~ s/&[^;]+;//g;  # Elimina toda otra entidad.

    $toencode =~ s/\xC0/A/g;
    $toencode =~ s/\xC1/A/g;
    $toencode =~ s/\xC2/A/g;
    $toencode =~ s/\xC3/A/g;
    $toencode =~ s/\xC4/A/g;
    $toencode =~ s/\xC5/A/g;
    $toencode =~ s/\xC6/A/g;
    $toencode =~ s/\xC7/C/g;
    $toencode =~ s/\xC8/E/g;
    $toencode =~ s/\xC9/E/g;
    $toencode =~ s/\xCA/E/g;
    $toencode =~ s/\xCB/E/g;
    $toencode =~ s/\xCC/I/g;
    $toencode =~ s/\xCD/I/g;
    $toencode =~ s/\xCE/I/g;
    $toencode =~ s/\xCF/I/g;
    $toencode =~ s/\xD1/N/g;
    $toencode =~ s/\xD2/O/g;
    $toencode =~ s/\xD3/O/g;
    $toencode =~ s/\xD4/O/g;
    $toencode =~ s/\xD5/O/g;
    $toencode =~ s/\xD6/O/g;

    $toencode =~ s/\xD8/O/g;
    $toencode =~ s/\xD9/U/g;
    $toencode =~ s/\xDA/U/g;
    $toencode =~ s/\xDB/U/g;
    $toencode =~ s/\xDC/U/g;
    $toencode =~ s/\xDD/Y/g;
    $toencode =~ s/\xE0/a/g;
    $toencode =~ s/\xE1/a/g;
    $toencode =~ s/\xE2/a/g;
    $toencode =~ s/\xE3/a/g;
    $toencode =~ s/\xE4/a/g;
    $toencode =~ s/\xE5/a/g;
    $toencode =~ s/\xE6/a/g;
    $toencode =~ s/\xE7/c/g;
    $toencode =~ s/\xE8/e/g;
    $toencode =~ s/\xE9/e/g;
    $toencode =~ s/\xEA/e/g;
    $toencode =~ s/\xEB/e/g;
    $toencode =~ s/\xEC/i/g;
    $toencode =~ s/\xED/i/g;
    $toencode =~ s/\xEE/i/g;
    $toencode =~ s/\xEF/i/g;
    $toencode =~ s/\xF1/n/g;
    $toencode =~ s/\xF2/o/g;
    $toencode =~ s/\xF3/o/g;
    $toencode =~ s/\xF4/o/g;
    $toencode =~ s/\xF5/o/g;
    $toencode =~ s/\xF6/o/g;
    $toencode =~ s/\xF8/o/g;
    $toencode =~ s/\xF9/u/g;
    $toencode =~ s/\xFA/u/g;
    $toencode =~ s/\xFB/u/g;
    $toencode =~ s/\xFC/u/g;
    $toencode =~ s/\xFF/y/g;

    return $toencode;
}    # notildes

# ---------------------------------------------------------------
# Lee un archivo por completo para efectos de parsearlo como ssi simulada en los FIDs
# Param. de entrada :
# 0) Path real del archivo.
# Retorna : El texto leido | '' en caso que el archivo no exista.
sub read_file4include {
    my $archivo    = shift;
    my $dir_server = shift;
    $archivo =~ s/\.+[\/\\]+//g;

    my $buffer = '';

    if (($archivo !~ /^$dir_server\//) || ($archivo =~ /\.\./)) {
        return '';
    }

    if (-f $archivo) {
        my ($size) = (-s $archivo);
        open(my $ARCHIVO, '<', $archivo)
          || die "Fail Open file $archivo \n $!\n";
        binmode $ARCHIVO;
        read $ARCHIVO, $buffer, $size;
        close $ARCHIVO;
    }

    return $buffer;
}

# ---------------------------------------------------------------
sub parse_includes {
    my $dir_server = shift;
    my $buffer     = shift;

    $buffer =~ s/%%include\((.+?)\)%%/&read_file4include($dir_server . $1, $dir_server)/ge;
    return $buffer;
}

# ---------------------------------------------------------------
sub parse_userinfo {
    my $buffer = $_[0];
    my $users_id = $_[1];

    $buffer =~ s/%%_users_name%%/$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$users_id}{'USERS_NOM'}/g;
    $buffer =~ s/%%_users_usr%%/$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$users_id}{'USERS_USR'}/g;

    return $buffer;
}

# ---------------------------------------------------------------
# Imprime error al STDERR y retorna un msg amistoso para el usuario, o bien, hace un exit sin mas tramite
sub handle_internal_error {
    my ($msg_internal, $msg_external) = @_;
    print STDERR $msg_external . " - " . $msg_internal;
    return $msg_external;
}

# ---------------------------------------------------------------
# lee el archivo "plugin.json" para el path especificado, dentro del directorio de plugins de Prontus
# devuelve una referencia a un hash con todos los datos del plugin.
sub get_plugin_info {
    my $plugin_name = shift;
    # evitamos que intenten cargar archivos con . o ..
    $plugin_name =~ s/\.\.+/./sg;
    my $path_plugin = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/prontus_plugins/$plugin_name/plugin.json";
    return {} if (! -f $path_plugin);

    my $json_text = glib_fildir_03::read_file_bin($path_plugin);
    my $data = JSON::decode_json($json_text);

    # evitamos que intenten cargar archivos con . o ..
    $data->{'plugin'} =~ s/\.\.+/./sg;
    next unless $data->{'nombre'};

    return $data;
}

# ---------------------------------------------------------------
sub get_url {
    my ($url)         = $_[0];
    my ($espera_segs) = $_[1];
    my ($type)        = $_[2];

    $espera_segs = 30    if ($espera_segs eq '');
    $type        = 'GET' if ($type        eq '');
    return '' if (($url eq '') or ($url !~ /^https?/i));
    # https://stackoverflow.com/a/9401362 intenta hacer que realmente respete el timeout.
    local $SIG{ALRM} = sub { die "SERVER timeout\n" };
    my ($ua) = new LWP::UserAgent;
    # segs # default de este módulo es 180, 3 minutos.
    $ua->timeout($espera_segs);
    # default es "libwww-perl/#.##"
    $ua->agent('Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36');
    my $request = new HTTP::Request($type, $url) || return ('', $!);
    my $response;
    eval {
        alarm($espera_segs);
        $response = $ua->request($request) || return ('', $!);
    }; # este ; debe ir ya que termina la linea de eval, sino la linea queda abierta
    # detiene la alarma anterior si no se gatilló
    alarm(0);
    if ($@) {
        if ($@ =~ /SERVER timeout/) {
            return('', 'Request Timeout.');
        } else {
            return ('', "Error en la request: $@");
        }
    } else {
        if ($response->is_success) {
            return ($response->content, '');
        } else {
            return ('', $response->status_line);
        }
    }
}    # get_url

# ---------------------------------------------------------------
# Obtiene el tamano del archivo segun el valor de la cabecera HTTP Content-length.
sub get_http_content_length {
    my $url = $_[0];
    my ($ua, $req, $res);

    $ua = new LWP::UserAgent;
    $ua->agent('Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36');
    $req = new HTTP::Request 'HEAD' => $url;
    $req->header('Accept' => 'text/html');
    $res = $ua->request($req);
    if ($res->is_success) {
        my $headers = $res->headers;
        return $headers->content_length;
    }
    return 0;
}

# ---------------------------------------------------------------
sub get_formatos_multimedia {
    my $marca = $_[0];
    my %formatos;
    my $file_formatos = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::XCODING_DATA_PATH$prontus_varglb::XCODE_CFG";

    if ($marca ne '') {
        if (-f $file_formatos) {
            my $buffer_formatos = glib_fildir_03::read_file($file_formatos);
            while ($buffer_formatos =~ /\s*($marca\.\w+)\.(\w+)\s*=\s*'(.*?)'/g) {
                $formatos{$1}{$2} = $3;
            }
        }
    }

    return %formatos;
}

# ---------------------------------------------------------------
sub dropbox_backup {
    my $recurso       = $_[0];
    my $dir_dropbox   = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA/dropbox";
    my $pid           = $$;
    my $dir_semaf     = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA/semaforos";
    my $semaf_dropbox = "$dir_semaf/dropbox_backup.lck";
    my $file          = "$dir_dropbox/$^T_$pid.txt";

    glib_fildir_03::check_dir($dir_dropbox);
    glib_fildir_03::check_dir($dir_semaf);

    open(my $FILE, '>>', $file);
    print $FILE $recurso . "\n";
    close $FILE;

    if (-f $semaf_dropbox) {
        my $mtime = (stat($semaf_dropbox))[9];
        my $now   = time;
        my $diff  = $now - $mtime;
        if ($diff > 7200) {    # 2 hrs.
            # muy antiguo, eliminar.
            unlink $semaf_dropbox;
        }
    }

    if (!-f $semaf_dropbox && -f $file) {
        my $cmd = "/usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_dropbox_backup.pl $prontus_varglb::PRONTUS_ID $file >/dev/null 2>&1 &";
        glib_fildir_03::write_file($semaf_dropbox, "0");
        print STDERR "dropbox[$cmd]\n";
        system $cmd;
    }

}

# ---------------------------------------------------------------
# Escribe al archivo PURGEFILE definido en prontus_purge_cache.pl las líneas a purgar.
sub purge_cache {
    my $path_file = $_[0];

    return if ($DISABLE_PURGE_CACHE);

    my $servers   = (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}});

    return if (!$servers && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} ne 'SI');

    my $relpath = lib_utils::remove_front_string($path_file, $prontus_varglb::DIR_SERVER);

    if ($relpath =~ m|/site/tax/port/|i && $relpath !~ /_1\.\w+$/) {
        return;
    }

    my $dir_pend = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/purgepend";
    glib_fildir_03::check_dir($dir_pend);
    my $pid = $$;
    open(my $PURGEFILE, '>>', "$dir_pend/$^T_$pid.txt");
    print $PURGEFILE $relpath . "\n";
    close $PURGEFILE;
    $DO_PURGE = 1;
}

# ---------------------------------------------------------------
sub set_exclude_port_table {
    my $path_xml      = $_[0];
    my $exclude_areas = $_[1];    # puede ser mas de una, separadas por coma.
    my $base          = $_[2];

    my $nom_table = glib_str_03::random_string(8);
    $nom_table = "EXCLUDE_PORT_$nom_table";

    $exclude_areas =~ s/\s*//sg;
    $exclude_areas =~ s/\t//sg;
    $exclude_areas =~ s/\n//sg;

    my @numAreas = split(/,/, $exclude_areas);
    my %areas;
    @areas{@numAreas} = undef;

    if (-f $path_xml) {
        my $buffer = glib_fildir_03::read_file($path_xml);
        lib_parser::ajusta_crlf($buffer);

        # Crear tabla temporal.
        my $sql_create = "CREATE TEMPORARY TABLE $nom_table (EXCLUDE_ART_ID bigint unsigned not null default 0);";

        glib_dbi_03::ejecutar_sql($base, $sql_create);
        if ($base->errstr) {
            my $error = "Error creando tabla temporal";
            print STDERR "$error [$sql_create][$!][".$base->errstr."]\n";
            return '';
        }

        while ($buffer =~ /(<rowartic>\s*<dir>(\d+)<\/dir>\s*<file>(.*?)<\/file>\s*<area>(.*?)<\/area>.*?<\/rowartic>)/sg) {
            my $rowartic = $1;
            my $dirfecha = $2;
            my $art      = $3;
            my $area     = $4;

            #print STDERR "art[$art] area[$area]\n";

            my $sql_insert = '';
            if ($exclude_areas) {
                if (exists $areas{$area}) {
                    $sql_insert = "INSERT INTO $nom_table SET EXCLUDE_ART_ID = ?";
                    my @params = ($art);
                    glib_dbi_03::ejecutar_sql_param($base, $sql_insert, \@params);
                    #print STDERR "exclude[$art] area[$area]\n";
                    if ($base->errstr) {
                        my $error = "Error insertando en tabla temporal [$art]";
                        print STDERR "$error [$sql_insert][$!][".$base->errstr."]\n";
                        return '';
                    }
                }
            } else {
                $sql_insert = "INSERT INTO $nom_table SET EXCLUDE_ART_ID = ?";
                my @params = ($art);
                glib_dbi_03::ejecutar_sql_param($base, $sql_insert, \@params);
                #print STDERR "exclude[$art]\n";
                if ($base->errstr) {
                    my $error = "Error insertando en tabla temporal [$art]";
                    print STDERR "$error [$sql_insert][$!][".$base->errstr."]\n";
                    return '';
                }
            }
        }

        return $nom_table;
    }
    return '';
}

# ---------------------------------------------------------------
sub call_purge_proc {
    return if ($DISABLE_PURGE_CACHE);
    return if (!$DO_PURGE);

    my $cmd = "/usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_purge_cache.pl $prontus_varglb::PRONTUS_ID >/dev/null 2>&1 &";
    print STDERR "purge[$cmd]\n";
    system $cmd;
    $DO_PURGE = 0;
}

# ---------------------------------------------------------------
sub get_html_ediciones {
    my $dir_edics = $_[0];
    my $form_edic = $_[1];
    # Generar cod. html correspondiente a la combo de ediciones
    my $html_ediciones = &get_edic_list($dir_edics, $form_edic);
    my $qty_base_ports = @{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}};
    # warn "qty_base_ports[$qty_base_ports]";
    if (($qty_base_ports <= 0) && ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI')) {
        $html_ediciones =~ s/<option value="base".*?<\/option>//s;    # eliminar de la combo la edic. base.
    }
    return $html_ediciones;
}

# ---------------------------------------------------------------
sub get_edic_list {
    my $path_dir       = $_[0];
    my $valor_clave    = $_[1];

    # Abre directorio.
    opendir(DIR, $path_dir) || die "Can't opendir" . $path_dir . $!;
    my @entries = readdir(DIR);
    closedir DIR;

    @entries = sort { $b cmp $a } (@entries);
    my $lista = '';
    my $nro_elem = 0;
    foreach my $entry (@entries) {
        if ($entry !~ /^\./g) {
            if ((-d ("$path_dir/$entry"))) {
                my $nom_arch = $entry;

                my $seleccionado = '';
                if ($nom_arch eq $valor_clave) {
                    $seleccionado = 'selected="selected"';
                }
                $lista    = $lista . '<option value="' . $nom_arch . '" ' . $seleccionado . '>';
                $lista    = $lista . $nom_arch . '</option>';                                      # 01.i
                $nro_elem = $nro_elem + 1;
            }
        }
    }

    return $lista;
}

# ---------------------------------------------------------------
# recibe el nombre de una portada. Deduce el path al archivo XML correspondiente, y calcula el md5sum del archivo.
sub get_md5sum_xml_port {
    my $nombre_port = $_[0];

    # Deduce del nombre de la portada, el del xml.
    my $path_xml = $nombre_port;
    $path_xml =~ s/(\w+?)\.\w*$/\/xml\/$1\.xml/;
    $path_xml = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic/base$path_xml";

    # print STDERR "nombre_port: $nombre_port, path_xml: $path_xml.\n";
    if (-f $path_xml) {
        my $xml_port = glib_fildir_03::read_file($path_xml);
        return Digest::MD5::md5_hex($xml_port);
    }

    return 0;
}

# ---------------------------------------------------------------
# recibe una string con un tamaño de archivo posiblemente terminado en letras, devuelve el valor en bytes.
# convierte hasta terabytes.
sub convierte_human_to_bytes {
    my $max_upload_size = shift;
    my $kilo = 1024;
    my $mega = $kilo * $kilo;
    my $giga = $mega * $kilo;
    my $tera = $giga * $kilo;

    if ($max_upload_size =~ /^\d+$/) {
        # es un valor en bytes, lo devuelvo tal cual
        return $max_upload_size;
    } else {
        # convertimos a bytes.
        my %factores = (
            'B'     => 1,
            'K'     => $kilo,
            'M'     => $mega,
            'G'     => $giga,
            'T'     => $tera,
        );
        my ($amount, $mag) = ($max_upload_size =~ /^(\d+)\s*([BKMGT])B?$/i); # un número, seguido de posibles espacios, luego las letras B, K, M, G, o T seguidas opcionalmente por la letra B. E.g. 200 M, 300GB.
        if ($factores{$mag}) {
            return $amount * $factores{$mag};
        } else {
            print STDERR "ERROR: lib_prontus::convierte_human_to_bytes, se recibió un formato desconocido: '$max_upload_size'\n";
            return $amount;
        }
    }

}

# ---------------------------------------------------------------
sub carga_multitag_json {
    my $num_elem = keys(%{$MULTITAG{$prontus_varglb::PRONTUS_ID}});
    if ($num_elem) {
        return;
    }
    my $src_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/multitag.json";
    my $str_json = glib_fildir_03::read_file_bin($src_file);

    if ($str_json =~ /^\{.*\}$/) {
        %{$MULTITAG{$prontus_varglb::PRONTUS_ID}} = %{JSON::decode_json($str_json)};
    }
}

# ---------------------------------------------------------------
sub updateSeoPort {
    my ($file, $url_seo) = @_;

    # Parsear datos de SEO.
    my $port_seo_dir = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . $prontus_varglb::DIR_BASEEDIC. "/seo";
    my ($nom, $ext) = lib_utils::split_nom_y_extension($file);
    my $port_seo_file = "$port_seo_dir/$nom.json";

    my %jsonSeo;
    if (-f $port_seo_file) {
        my $jsonSeoBuffer = glib_fildir_03::read_file($port_seo_file);
        my $jsonSeoRef = JSON::from_json($jsonSeoBuffer);
        %jsonSeo = %{$jsonSeoRef};

    }
    $jsonSeo{'_seo_url'} = $url_seo;

    glib_fildir_03::check_dir($port_seo_dir);
    glib_fildir_03::write_file($port_seo_file, JSON::to_json(\%jsonSeo));
}

# redirecciona a la "home" del cpan, a la que todos los usuarios tienen acceso
sub redirect_home {
    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_dashboard.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
    exit;
}

sub cache_active {
    if (defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'}) {
        return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'};
    } else {
        my $servers = (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}});
        if ($servers || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} eq 'SI') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'} = 'SI'
        } else {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'} = 'NO'
        }
    }
    return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_ACTIVE'};
}

sub end_tasks {
    %ARTICS = ();
    %MULTITAG = ();

    &call_purge_proc();
}
# ---------------------------------------------------------------
END {
    &end_tasks();
}

$SIG{INT} = \&end_tasks;

1;
