#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Funciones centralizadas para el sistema de tags
#---------------------------------------------------------------
# Requiere
# use lib_prontus;
# use glib_dbi_03;
# use glib_fildir_03;

package lib_tags;
use strict;

our $MAX_ARTICS_PER_SEARCH  = 20;
our $MAX_TAGS_SEARCH_RESULT = 20;
our $MAX_LIMIT = 50; # Limite maximo permitido para la generación de relacionados.
our $RELDIR_ARTIC = '';
our $RELDIR_ARTIC_RELAC = '';
our $NUM_RELAC_DEFAULT = '';

our $RELDIR_DST_ARTIC_RELAC = '';
# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------

sub get_max_tags_search_result {
    return $MAX_TAGS_SEARCH_RESULT
}

# configura las variables globales.
sub set_vars {
    my ($dir_contenido, $dir_artic, $dir_pag, $dir_temp, $dir_tags, $num_relac_default) = @_;
    $RELDIR_ARTIC           = $dir_contenido . $dir_artic . '/%%dirfecha%%' . $dir_pag;
    $RELDIR_ARTIC_RELAC     = $dir_temp . $dir_tags;
    $RELDIR_DST_ARTIC_RELAC = $dir_contenido . $dir_tags;
    $NUM_RELAC_DEFAULT      = $num_relac_default;
}

# Verifica que el nombre no exista en la tabla.
sub check_nom {
    my ($db, $id, $nombre) = ($_[0], $_[1], $_[2]);
    my @params;
    my $sql;

    if ($id) {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?) and TAGS_ID <> ?";
        push @params, $nombre;
        push @params, $id;
    } else {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?)";
        push @params, $nombre;
    }

    my $cant;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($db, $sql, \@params, \$cant);

    $salida->fetch;
    $salida->finish;

    if ($cant > 0) {
        return 'Nombre de ítem ya existe.';
    }

    return '';
}

# ---------------------------------------------------------------
sub get_tags_for_fid {
    my $base = $_[0];
    my $ts   = $_[1];
    my $tags_order = $_[2];
    my @params;

    # tags asignados
    my $sql = "select TAGSART_IDTAGS,TAGS_TAG from TAGSART join TAGS on TAGS_ID=TAGSART_IDTAGS where TAGSART_IDART = ?";
    push @params, $ts;

    if ($tags_order) {
        $sql .= " order by FIELD(TAGS_ID, ?);";
        push @params, $tags_order;
    } else {
        $sql .= " order by TAGS_TAG asc;";
    }

    my $tags_query = glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    my ($idtag_asignado, $tag_name);

    $tags_query->bind_columns(undef, \($idtag_asignado, $tag_name));

    # Se leen los tags de la BD
    my $tags_asignados = '';

    while ($tags_query->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_name);
        $tag_name = lib_utils::escape_xml($tag_name);
        $tags_asignados = $tags_asignados . "$idtag_asignado|$tag_name,";
    }

    $tags_asignados =~ s/,$//;
    $tags_query->finish;

    # warn("tags_asignados[$tags_asignados]");
    return $tags_asignados;
}

# ---------------------------------------------------------------
sub get_all_tags {
    my $base = $_[0];
    my %hash_tags;

    my $sql = "select TAGS_ID, TAGS_TAG from TAGS where TAGS_MOSTRAR = 1 order by TAGS_TAG asc";
    my $tab_output = glib_dbi_03::ejecutar_sql($base, $sql);

    my ($idtag, $tag_name);
    $tab_output->bind_columns(undef, \($idtag, $tag_name));

    while ($tab_output->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_name);
        $hash_tags{$idtag} = $tag_name;
    }
    return %hash_tags;
}

# ---------------------------------------------------------------
sub get_tag_noms {
    my $base   = shift;
    my $tag_id = shift;
    my %tag_noms;
    my @params;
    my $sql = "select TAGS_TAG, TAGS_NOM4VISTAS from TAGS where TAGS_ID = ?";

    push @params, $tag_id;

    my $tag;
    my $nom4vistas;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($tag, $nom4vistas));

    while ($salida->fetch) {
        glib_str_03::correct_encode_utf8_string($tag);
        glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tag_noms{''} = lib_prontus::escape_html($tag);    # nombre en la vista default o ppal

        while ($nom4vistas =~ /(^|\n)(\w+)\t(.*?)($|\n)/sg) {
            my $vista = $2;
            my $nom   = $3;

            $tag_noms{$vista} = lib_prontus::escape_html($nom);    # nombre en cada vista
        }
    }

    $salida->finish;

    return %tag_noms;
}

sub get_tags_noms_hash {
    my $base = $_[0];
    my $tags_id = $_[1];
    my @tags_id_arr = split("," , $tags_id);
    my %tags_noms;

    if (!$tags_id) {
        return %tags_noms;
    }

    my $placeholders = ', ?' x scalar @tags_id_arr;
    $placeholders = substr $placeholders, 2;
    my $sql = "select TAGS_ID, TAGS_TAG, TAGS_NOM4VISTAS from TAGS where TAGS_ID IN ($placeholders)";
    my $tag;
    my $tag_id;
    my $nom4vistas;
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@tags_id_arr, \($tag_id, $tag, $nom4vistas));

    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag);
        &glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tags_noms{$tag_id}{''} = lib_prontus::escape_html($tag);

        while ($nom4vistas =~ /(^|\n)(\w+)\t(.*?)($|\n)/sg) {
            my $vista = $2;
            my $nom   = $3;

            $tags_noms{$tag_id}{$vista} = lib_prontus::escape_html($nom);
        }
    }

    $salida->finish;

    return %tags_noms;
}

# Dada una id de tag, devuelve la URL custom, si existe en db.
# De otro modo devuelve la URL estándar de tagport
# Intenta leer el archivo json de tags.
# @param int $tag_id
# @return string
sub get_tag_url {
    my $tag_id = $_[0];
    my $url = '';

    my @params = ($tag_id);
    my $sql = "SELECT TAGS_PORT";
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        $sql .= ", URLF5_URL FROM TAGS LEFT JOIN URLF5 ON TAGS_ID=URLF5_SUBTYPEID";
    } else {
        $sql .= " FROM TAGS";
    }
    $sql .= " WHERE TAGS_ID = ?";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        $sql .= " AND URLF5_TYPE = ?";
        push(@params, 'tag')
    }

    my $salida = &glib_dbi_03::ejecutar_sql_param($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \@params);
    my $row = $salida->fetchrow_hashref;
    if ($row->{'URLF5_URL'}) {
        $url = $row->{'URLF5_URL'};
    } elsif ($row->{'TAGS_PORT'}) {
        $url = $row->{'TAGS_PORT'};
    } else {
        $url = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/tag/port/all/tagport_".$tag_id."_1.html";
    }

    $salida->finish;
    return $url;
}

#---------------------------------------------------------------
# genera un listado de tags en formato json.
sub generar_json {
    my $bd = shift;
    my ($sql, $salida, $tag_id, $tag_nom);

    my %tags_url = ();
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        my ($salida2, $urlf5_subtypeid, $urlf5_file, $urlf5_url);
        my $sql2 = "SELECT URLF5_SUBTYPEID, URLF5_FILE, URLF5_URL FROM URLF5 WHERE URLF5_TYPE = ?";
        my @params = ('tag');
        $salida2 = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql2, \@params, \($urlf5_subtypeid, $urlf5_file, $urlf5_url));

        while ($salida2->fetch) {
            $urlf5_file =~ s/\/$prontus_varglb::PRONTUS_ID//;
            $tags_url{$urlf5_subtypeid}{$urlf5_file} = $urlf5_url;
        }
        $salida2->finish;
    }

    $sql = "SELECT TAGS_ID, TAGS_TAG FROM TAGS WHERE TAGS_MOSTRAR = 1";
    $salida = glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($tag_id, $tag_nom));
    my @tags;
    while ($salida->fetch) {
        glib_str_03::correct_encode_utf8_string($tag_nom);
        my %urls = ();
        if (defined($tags_url{$tag_id})) {
            %urls = %{$tags_url{$tag_id}};
        }
        push @tags, {'id' => $tag_id, 'nom' => $tag_nom, 'urls' => \%urls};
    }

    $salida->finish;

    my $str_json = JSON::encode_json(\@tags);
    utf8::decode($str_json);

    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    glib_fildir_03::check_dir($dst_path);
    glib_fildir_03::write_file("$dst_path/tags.json", $str_json);
}

# ---------------------------------------------------------------
# subrutinas de generación de relacionados
# ---------------------------------------------------------------

# Genera un archivo con los relacionados por tags.
sub generar_relacionados_tags {
    my ($tag_list_str, $dst_dir, $ts, $base, $mv) = @_;
    # borra extension, en caso de que venga con ella
    $ts =~ s/\..+$//;

    my $ruta_dir = $prontus_varglb::DIR_SERVER . $RELDIR_ARTIC_RELAC;
    $ruta_dir =~ s/\/tags\/pags/\/tags\/pags-$mv/ if ($mv);

    $dst_dir =~ s/(\d{8})\/pags/$1\/pags-$mv/ if ($mv);

    my @lisdir;
    @lisdir = glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);

    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..
    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            my $pagina = glib_fildir_03::read_file("$ruta_dir/$k");

            my $loop = '';
            if ($pagina =~ /%%loop%%(.*?)%%\/loop%%/s) {
                $loop = $1;
            }

            # Generar listas con los articulos especificados manualmente
            my ($lista) = &make_lista_tag($ts, $tag_list_str, $loop, $base, $mv);
            # print STDERR "lista[$lista]\n";

            $pagina =~ s/%%_num_relac%%//g;
            # Parsear y escribir paginas para incluir en articulos.
            &parse_and_write_tag($lista, $pagina, $dst_dir, $ts, $k);
        }
    }
}

# ---------------------------------------------------------------
sub parse_and_write_tag {
    my ($lista, $pagina, $dst_dir, $ts, $nomfile) = @_;
    $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;

    $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;

    $pagina = lib_parser::parse_globals($pagina, '');

    my ($crlf) = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

    $pagina =~ s/$crlf/\x0a/sg;

    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html y marcas prontus sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    glib_fildir_03::check_dir($dst_dir);
    glib_fildir_03::write_file("$dst_dir/" . $ts . '_tags_' . $nomfile , $pagina);
}

# ---------------------------------------------------------------
# genera la query que buscará los tags para relacionados y devuelve los resultados.
sub make_lista_tag {
    my ($art_ts, $tag_ids_str, $loop, $base, $mv) = @_;

    my $dthr_system = glib_hrfec_03::get_dtime_pack4();
    $dthr_system =~ /^(\d{8})(\d\d\d\d)/;
    my $hhmm_system = $2;
    my $dt_system = $1;

    my ($filtros, $params_ref) = &genera_filtros_tag($art_ts, $tag_ids_str, $dt_system, $hhmm_system);
    my $sql = "SELECT TAGSART.TAGSART_IDART, TAGSART.TAGSART_IDTAGS, ART.ART_FECHAPHORAP, ART.ART_EXTENSION FROM TAGSART INNER JOIN ART ON TAGSART.TAGSART_IDART=ART.ART_ID %%FILTROS%% group by TAGSART.TAGSART_IDART order by ART_FECHAPHORAP desc LIMIT ?";
    my @params = @{$params_ref};

    $sql =~ s/%%FILTROS%%/WHERE $filtros/;

    my $tot_artics = &get_tot_artics($filtros, $params_ref, $base, $MAX_LIMIT);

    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $MAX_LIMIT;

    my ($art_extension, $id_tag, $art_fechap);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_ts, $id_tag, $art_fechap, $art_extension));
    my $nro_filas = 0;
    my $filas;
    my $loopcounter = 0;

    while ($salida->fetch) {
        $loopcounter++;
        my ($una_fila, $filler) = lib_parser::generar_fila($RELDIR_ARTIC, $art_ts, $art_extension, $loop, $loopcounter, $tot_artics, $mv);
        $filas .= $una_fila;
        $nro_filas++;
    }

    $salida->finish;

    return $filas;
}

# Genera segmento variable del sql para encontrar los articulos.
sub genera_filtros_tag {
    my ($art_ts, $tag_ids, $dt_system, $hhmm_system) = @_;
    my @params;
    my $filtros = ' 1 = 1 ';
    if ($tag_ids) {
        $filtros .= " AND (";
        my $i = 0;
        foreach my $id_tag (split(/,\s*/ , $tag_ids)){
            if ($i == 0) {
                $filtros = $filtros." TAGSART.TAGSART_IDTAGS = ?";
            } else {
                $filtros = $filtros." OR TAGSART.TAGSART_IDTAGS = ?";
            }
            push @params, $id_tag;
            $i++;
        }
        $filtros .= ")";

        $filtros = $filtros." AND TAGSART.TAGSART_IDART <> ?";
        push @params, $art_ts;

        $filtros .= " and (ART_FECHAPHORAP <= ?) ";
        push @params, "$dt_system$hhmm_system";
        $filtros .= " and (ART_ALTA = '1') ";

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $filtros .= " and ( (ART_FECHAEHORAE >= ?) OR ( (ART_FECHAEHORAE < ?) AND (ART_SOLOPORTADAS = '1') ) )";

            push @params, "$dt_system$hhmm_system";
            push @params, "$dt_system$hhmm_system";
        }

        # procesamos solamente los artículos que generan vistas.
        my %fids_hash = lib_helper::get_fids_with_view();
        my @fids_con_vista;
        foreach my $fid (keys %fids_hash) {
            push @fids_con_vista, "'$fid'";
        }
        my $fids_string = join(',', @fids_con_vista);
        $filtros .= " and ART_TIPOFICHA in ($fids_string) " if $fids_string;
    }
    return ($filtros, \@params);

}

sub get_tot_artics {
    my $filtros = shift;
    my $params_ref = shift;
    my $base = shift;
    my $maxartics = shift;
    my ($sql, $salida, $tot);
    my ($count_art);
    my @params = @{$params_ref};

    $sql = "SELECT COUNT(ART_ID) FROM ART INNER JOIN TAGSART ON ART.ART_ID = TAGSART.TAGSART_IDART %%FILTRO%%";

    if ($filtros ne '') {
        $sql =~ s/%%FILTRO%%/WHERE $filtros/;
    } else {
        $sql =~ s/%%FILTRO%%//;
    }

    $salida = glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_art));
    $salida->fetch;
    $salida->finish;

    $count_art = '0' if $count_art eq '';
    $count_art = $maxartics if ($count_art > $maxartics);

    return $count_art;
}

sub make_mapa_tags {
    my $mv = $_[0];
    my $bd = $_[1];
    my $dir_plt = 'pags';
    my @postprocesos;

    $dir_plt .= "-$mv" if ($mv);

    my $ruta_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}/extra/tags/$dir_plt";
    my @lisdir;
    if (-d $ruta_dir) {
        @lisdir = glib_fildir_03::lee_dir($ruta_dir);
        # Elimina directorios . y ..
        @lisdir = grep !/^\./, @lisdir;
    }

    foreach my $plt_file (@lisdir) {
        next if (! -f "$ruta_dir/$plt_file");

        my $mapa_plt = glib_fildir_03::read_file("$ruta_dir/$plt_file");
        $mapa_plt = lib_parser::ajusta_crlf($mapa_plt);

        my $loop_tags = '';
        if ($mapa_plt =~ /%%loop_tags%%(.*?)%%\/loop_tags%%/s) {
            $loop_tags = $1;
        }

        my %config = &get_config_mapa_tags($mapa_plt);
        my $output = &get_arbol_mapa_tags($loop_tags, $bd, $mv, \%config);

        $mapa_plt =~ s/%%loop_tags%%.*?%%\/loop_tags%%/$output/s;

        $mapa_plt = lib_prontus::parser_custom_function($mapa_plt);

        $mapa_plt =~ s/%%_server_name%%/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'}/g;
        $mapa_plt =~ s/%%_prontus_id%%/$prontus_varglb::PRONTUS_ID/g;

        my ($_plt_nom, $_plt_ext) = lib_utils::split_nom_y_extension($plt_file);
        $mapa_plt =~ s/%%_plt_nom%%/$_plt_nom/g;
        $mapa_plt =~ s/%%_plt_ext%%/$_plt_ext/g;
        $mapa_plt =~ s/%%_vista%%/$mv/g;

        # rescata el post_proceso
        if ($mv eq '') {
            while ($mapa_plt =~ /<!--POST_PROCESO=(.+?)-->/isg) {
                push(@postprocesos, $1);
            }
        }
        $mapa_plt =~ s/<!--POST_PROCESO=.+?-->//isg;

        # Borra marcas no sustituidas
        $mapa_plt =~ s/%%.+?%%//g;

        glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/tags/$dir_plt");
        my $dst_mapa = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/extra/tags/$dir_plt/$plt_file";
        glib_fildir_03::write_file($dst_mapa, $mapa_plt);
        lib_prontus::purge_cache($dst_mapa) if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_MAPA'} eq 'SI');
    }

    # ejecutamos los postprocesos asociados
    my $rutaScript = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN";

    foreach my $pproc (@postprocesos) {
        $pproc =~ s/[#;&<>|~]+//g;
        my $cmd = "$rutaScript/$pproc $prontus_varglb::PRONTUS_ID $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} >/dev/null 2>&1 &";
        print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "][mapa_tags][PPROC][$cmd]\n";
        system $cmd;
    }
}

sub get_config_mapa_tags {
    my $plt = $_[0];
    my %config = (
        'order' => 'TAGS_ID',
        'solo_con_artics' => 0
    );

    while ($plt =~ /<!--\s*CONFIG\s*(\w+)\s*=\s*(.*?)\s*-->/sg) {
        my $name = $1;
        my $value = $2;

        if ($name eq 'ORDERBY') {
            if ($value eq 'name') {
                $config{'order'} = 'TAGS_TAG';
            }

            if ($value eq 'id') {
                $config{'order'} = 'TAGS_ID';
            }
        }

        if ($name eq 'SOLO_CON_ARTICS') {
            if ($value =~ /^\d+$/) {
                $config{'solo_con_artics'} = $value;
            }
        }
    }

    return %config;
}

sub get_arbol_mapa_tags {
    my $loop =$_[0];
    my $bd = $_[1];
    my $mv = $_[2];
    my %config = %{$_[3]};

    my $sql = "SELECT TAGS_ID, TAGS_TAG, TAGS_NOM4VISTAS, TAGS_MOSTRAR, TAGS_COUNT, TAGS_PORT FROM TAGS";

    if ($config{'solo_con_artics'} ) {
        $sql .= " WHERE TAGS_COUNT > 0";
    }

    $sql .= " ORDER BY ?";
    my @params = ($config{'order'});

    my ($tags_id, $tags_tag, $tags_nom4vistas, $tags_mostrar, $tags_count, $tags_port);

    my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($tags_id, $tags_tag, $tags_nom4vistas, $tags_mostrar, $tags_count, $tags_port));
    my $mapa_tags;
    my $output;

    while ($salida->fetch) {
        $mapa_tags = $loop;

        $tags_tag = lib_prontus::get_nomtax_envista($mv, $tags_nom4vistas, $tags_tag) if ($mv);
        $tags_tag = lib_prontus::escape_html($tags_tag);
        $tags_port = lib_prontus::escape_html($tags_port);

        $mapa_tags =~ s/%%_id%%/$tags_id/g;
        $mapa_tags =~ s/%%_nom%%/$tags_tag/g;
        $mapa_tags =~ s/%%_mostrar%%/$tags_mostrar/g;
        my $tags_fixedurl = &get_tag_link($tags_port, $mv);
        $mapa_tags =~ s/%%_fixed_url%%/$tags_fixedurl/g;

        my %claves = ('_mostrar'      => $tags_mostrar);
        my %nodata;

        $mapa_tags = lib_parser::procesa_condicional($mapa_tags, \%claves, \%nodata);

        $output .= $mapa_tags;
    }

    $salida->finish;

    return $output;
}

# ---------------------------------------------------------------
# entrega link a portada prontus o url para un tag.
sub get_tag_link {
    # mv: nombre de la vista
    my ($nom, $mv)  = @_;

    if ($nom) {
        # url
        if ($nom =~ /^(https?:\/\/|\/)/i) {
            return $nom;
        } else {
            # nombre de una port => linkear a port de la edic. vigente o bien de la base.
            my $edic;
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} eq 'SI') {
                $edic = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/ed_vigente.txt");
            } else {
                $edic = 'base';
            }
            my $dir_port = $prontus_varglb::DIR_PORT;
            # nombre de la vista.
            if ($mv) {
                $dir_port = $dir_port . '-' . $mv;
            }
            return $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::DIR_EDIC . "/$edic" . $dir_port . "/$nom";
        }
    } else {
        return '';
    }
}

return 1;
