#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# funciones para manejo de usuarios
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use prontus_varglb;
# use glib_dbi_03;
# use lib_bd;
# use glib_str_03; # para load_all_user_data

use strict;
use utf8;

package lib_users;

# ---------------------------------------------------------------
# Chequea que el username pasado por parametro exista en el hash de USERS y retorna el USERS_ID y el USERS_PERFIL.
# Si no se logra autenticar, retorna el id vacio y en el perfil retorna el mensaje de error.
sub check_user_session {
    my $redir = $_[0];    # redireccion en caso de que no se detecte una sesion activa.

    my $sess_obj = Session->new('prontus_id'    => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'},
                                'document_root' => $prontus_varglb::DIR_SERVER
                               ) || die("Error inicializando objeto Session: $Session::ERR\n");

    if ($sess_obj->{id_session} eq '' || $sess_obj->{username} eq '' || $sess_obj->{psw_hash} eq '') {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\r\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\r\n\r\n";
            exit;
        } else {
            # print STDERR "No se detectó una sesión activa\n";
            return ('', 'No se detectó una sesión activa', $sess_obj);
        }
    }

    # solo aca se inicializa esta variable
    $prontus_varglb::USERS_USR = $sess_obj->{username};

    my ($res, $err_msg) = &load_user_data($sess_obj->{username});
    if (!$res) {
        return ('', "Error al cargar datos de usuarios [$err_msg]", $sess_obj);
    }

    my $login_result = &prontus_auth::check_valid_hash_psw($sess_obj->{username}, $sess_obj->{psw_hash});

    # caso mas comun, usuario logeado correctamente
    if ($login_result == 1) {
        # Regenera la sesion para evitar que sea borrada por el gc.
        $sess_obj->rejuvenece_sesion();

        # valida que el usuario tenga acceso al prontus que intenta acceder
        if ($prontus_auth::USERS_PERFIL != 1 &&
            !defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$prontus_auth::USERS_USR_ID}{$prontus_varglb::PRONTUS_ID})) {
            return ('', "No hay permiso", $sess_obj);
        }

        # carga los permisos de portadas y fids del usuario
        ($res, $err_msg) = &load_user_template_access($sess_obj->{username});
        if (!$res) {
            return ('', "Error al cargar permisos de usuarios [$err_msg]", $sess_obj);
        }
        lib_profile::load_profiles($prontus_auth::USERS_PERFIL); # cargamos la info de perfiles en varglb{PROFILES}
        return ($prontus_auth::USERS_USR_ID, $prontus_auth::USERS_PERFIL, $sess_obj);
    }
    # contraseña expirada
    if ($login_result == 3) {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
            exit;
        } else {
            print STDERR "Tu contraseña ha expirado. Inicia sesión nuevamente y cámbiala: ".$sess_obj->{username}."\n";
            return ('', 'Tu contraseña ha expirado. Inicia sesión nuevamente y cámbiala.', $sess_obj);
        }
    }
    # contraseña invalida
    if ($login_result == 0) {
        if ($redir) {
            # redirecciona al login.
            print "Status: 303 See Other\n";
            print "Location: /$prontus_varglb::DIR_CGI_CPAN/prontus_index.cgi?_prontus_id=$prontus_varglb::PRONTUS_ID\n\n";
            exit;
        } else {
            print STDERR "Usuario o Contraseña no válida: ".$sess_obj->{username}."\n";
            return ('', 'Usuario o Contraseña no válida.', $sess_obj)
        }
    }
}

# ---------------------------------------------------------------
# cierra session del usuario en prontus
sub cerrar_sesion {
    my $sess_obj = Session->new('prontus_id'    => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'},
                                'document_root' => $prontus_varglb::DIR_SERVER
                               ) || die("Error inicializando objeto Session: $Session::ERR\n");
    $sess_obj->end_session();

    return ($sess_obj->{username}, $sess_obj->{id_session});
}

# Verifica si un usuario existe en Prontus según un parámetro pasado.
# Implementada para sso/prontus_oidc_callback.cgi
# Parámetros:
#   field: el campo por el cual buscar. Por defecto es el nombre de usuario, puede ser
#   user, email, id.
#   all: puede ser vacío u "all". All implica considerar también a los usuarios desactivados.
# Retorna:
#   array (USERS_ID, USERS_USR, USERS_PERFIL), o (0, error_msg, '').
sub check_user_simple {
    my ($field, $value, $all) = @_;

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sub_query = " USERS_USR = ? ";
    if ($field eq 'email') {
        $sub_query = " USERS_EMAIL = ? ";
    } elsif ($field eq 'id') {
        $sub_query = " USERS_ID = ? ";
    }

    my $exclude_deleted = ' AND USERS_DELETED = 0 ';
    $exclude_deleted = '' if $all eq 'all';
    my $sql = "SELECT * FROM USERS WHERE $sub_query $exclude_deleted LIMIT 1";
    my @params = $value;
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);

    if (my $row = $salida->fetchrow_hashref) {
        if ($row->{'USERS_ID'}) {
            lib_profile::load_profiles($prontus_auth::USERS_PERFIL); # cargamos la info de perfiles en varglb{PROFILES}
            return ($row->{'USERS_ID'}, $row->{'USERS_USR'}, $row->{'USERS_PERFIL'});
        }
    }
    return (0, "No se encontró el usuario", '');
}

# ---------------------------------------------------------------
# lista las passwords antiguas para un usuario determinado
sub list_old_passwords {
    my $user_id  =  $_[0];

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql = 'SELECT USERS_PSW FROM OLD_PSW WHERE USERS_ID = ?';
    my $psw;
    my @params = ($user_id);
    my $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($psw));

    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    my @pws_list;
    while ($salida->fetch) {
        push(@pws_list, $psw);
    }
    $salida->finish;

    return @pws_list;
}
# ---------------------------------------------------------------
# guarda las passwords antiguas
# se asume que esta password no existe, por lo tanto es distinta a todas las anteriores
sub store_old_passwords {
    if ($prontus_auth::PSW_TO_REMEMBER == 0) {
        return;
    }
    my $user_id  =  $_[0];
    my $old_pass =  $_[1];

    if ($old_pass eq '') {
        return (0, "Password vacía");
    }

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql = "INSERT INTO OLD_PSW values(?,?,?,?)";
    my @params;
    push(@params, undef);
    push(@params, $user_id);
    push(@params, $old_pass);
    push(@params, time);

    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    $sql = 'SELECT COUNT(*) FROM OLD_PSW WHERE USERS_ID = ?';
    my $total_count;
    @params = ($user_id);
    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($total_count));
    $salida->fetch;
    $salida->finish;
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    $prontus_auth::PSW_TO_REMEMBER--;
    # debemos borrar las contraseñas mas antiguas
    while ($total_count > $prontus_auth::PSW_TO_REMEMBER) {
        # buscamos la mas antigua
        $sql = 'SELECT OLD_PSW_ID FROM OLD_PSW WHERE USERS_ID = ? ORDER BY PSW_EPOCH LIMIT 1';
        my $old_id;
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($old_id));
        $salida->fetch;
        $salida->finish;
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        @params = ($old_id);
        $sql = "DELETE FROM OLD_PSW WHERE OLD_PSW_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # revisamos si hay que eliminar mas
        $sql = 'SELECT COUNT(*) FROM OLD_PSW WHERE USERS_ID = ?';
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($total_count));
        $salida->fetch;
        $salida->finish;
    }

    return (1, "");
}

sub update_last_login {
    my $user_id = shift;
    return (0, "No se recibió un ID de usuario válido") if (!$user_id);

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }
    my $sql = "UPDATE USERS SET USERS_FEC_LAST_LOGIN = CURRENT_TIMESTAMP WHERE USERS_ID = ?";
    my @params;
    push(@params, $user_id);

    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    $salida->finish;
    return (1, "");
}

# guarda los datos de un usuario en bd
sub save_user_data {
    my $user_id = $_[0];

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql;
    my $idvalue = 0;
    if ($user_id eq 'new_user') {
        $sql = "INSERT INTO USERS (USERS_ID, USERS_NOM, USERS_USR, USERS_PSW, USERS_PROFILE_ID, USERS_EMAIL, USERS_EXP_DAYS, USERS_FEC_EXP, USERS_RENEW_PSW, USERS_2FA, USERS_OIDC) values(?,?,?,?,?,?,?,?,?,?,?)";
        my @params;
        push(@params, undef);
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_NOM'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EMAIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EXP_DAYS'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_FEC_EXP'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_RENEW_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_2FA'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_OIDC'});

        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $sql = 'SELECT last_insert_id()';
        $salida = glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($idvalue));
        $salida->fetch;
        $salida->finish;

        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # pasamos de vuelta el id del nuevo usuario.
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'}} = $idvalue;
    } else {
        $sql = "UPDATE USERS SET USERS_NOM = ?, USERS_USR = ?, USERS_PSW = ?, USERS_PROFILE_ID = ?, USERS_EMAIL = ?, USERS_EXP_DAYS = ?, USERS_FEC_EXP = ?, USERS_RENEW_PSW = ?, USERS_2FA = ?, USERS_OIDC = ? WHERE USERS_ID = ?";
        my @params;
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_NOM'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_USR'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EMAIL'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_EXP_DAYS'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_FEC_EXP'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_RENEW_PSW'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_2FA'});
        push(@params, $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_OIDC'});
        push(@params, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $idvalue = $user_id;
    }
    return ($idvalue, '');
}

# guarda los permisos de acceso a portadas y articulos de los usuarios
sub save_user_permissions {
    my ($user_id, $idvalue) = @_;

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    if (!$idvalue) {
        $idvalue = $user_id;
    }

    # idvalue es el valor del usuario que se insertara en bd
    # los datos a insertar estan asociados a "user_id" que puede ser "new_user"
    # usuario administrador o perfil administrador no guarda asociaciones de articulos y portadas
    if ($idvalue && $idvalue > 1) {
        my @params = ($idvalue);
        my $sql = "DELETE FROM ART_USERS WHERE ART_USER_ID = ?";

        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $sql = "DELETE FROM PORT_USERS WHERE PORT_USER_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $sql = "DELETE FROM PRONTUS_USERS WHERE PRONTUS_USER_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        if ($prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$user_id}{'USERS_PERFIL'} > 1) {
            foreach my $prontus_id (keys %prontus_varglb::CONFIG) {
                foreach my $art (keys %{$prontus_varglb::CONFIG{$prontus_id}{'ARTUSERS'}{$user_id}}) {
                    $sql = "INSERT INTO ART_USERS values(?, ?, ?)";
                    my @insert_data = ($idvalue, $art, $prontus_id);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }

                foreach my $port (keys %{$prontus_varglb::CONFIG{$prontus_id}{'PORTUSERS'}{$user_id}}) {
                    $sql = "INSERT INTO PORT_USERS values(?, ?, ?)";
                    my @insert_data = ($idvalue, $port, $prontus_id);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }

                foreach my $prontus (keys %{$prontus_varglb::CONFIG{$prontus_id}{'PRONTUS_USERS'}{$user_id}}) {
                    $sql = "INSERT INTO PRONTUS_USERS values(?, ?)";
                    my @insert_data = ($idvalue, $prontus);
                    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@insert_data);
                    if ($bd->errstr) {
                        return (0, $bd->errstr);
                    }
                }
            }
        }
    }
    return (1, "");
}

# ---------------------------------------------------------------
# borra un usuario en bd
sub delete_user {
    my $user_id = int($_[0]);

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql;
    my $idvalue;
    # usuario admin #1 no se puede borrar
    if ($user_id > 1 ) {
        $sql = "UPDATE USERS SET USERS_PSW = ?, USERS_EMAIL = ?, USERS_EXP_DAYS = ?, USERS_FEC_EXP = ?, USERS_RENEW_PSW = ?, USERS_2FA = ?, USERS_OIDC = ?, USERS_DELETED = ? WHERE USERS_ID = ?";
        my @params;
        push(@params, '');
        push(@params, '');
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 1);
        push(@params, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # borra asociaciones de articulos y portadas
        @params = ($user_id);
        $sql = "DELETE FROM ART_USERS WHERE ART_USER_ID = ?";

        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
        $sql = "DELETE FROM PORT_USERS WHERE PORT_USER_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        $sql = "DELETE FROM PRONTUS_USERS WHERE PRONTUS_USER_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }

        # borramos claves antiguas almacenadas
        @params = ($user_id);
        $sql = "DELETE FROM OLD_PSW WHERE USERS_ID = ?";
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
    } else {
        return (0, "El usuario no se puede borrar, id: [$user_id]");

    }

    return (1, "");
}

# ---------------------------------------------------------------
# recupera un usuario en bd
sub undelete_user {
    my $user_id = int($_[0]);

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql;
    my $idvalue;
    # usuario admin #1 no se puede borrar
    if ($user_id > 1 ) {
        $sql = "UPDATE USERS SET USERS_PSW = ?, USERS_EMAIL = ?, USERS_EXP_DAYS = ?, USERS_FEC_EXP = ?, USERS_RENEW_PSW = ?, USERS_2FA = ?, USERS_OIDC = ?, USERS_DELETED = ? WHERE USERS_ID = ?";
        my @params;
        push(@params, '');
        push(@params, '');
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, 0);
        push(@params, $user_id);
        my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        if ($bd->errstr) {
            return (0, $bd->errstr);
        }
    } else {
        return (0, "El usuario no se puede recuperar, id: [$user_id]");

    }

    return (1, "");
}

# ---------------------------------------------------------------
# carga los datos de todos los usaarios desde bd
# requiere glib_str_03
sub load_all_user_data {
    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    my $sql = "SELECT * FROM USERS";
    my $salida = glib_dbi_03::ejecutar_sql($bd, $sql);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }

    # reinicia valores de variables
    # Para retrocompatibilidad.
    %prontus_varglb::USERS = ();
    %prontus_varglb::USR_ID_REL= ();

    # nuevo hash para prontus 12.0.11+
    $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID} = ();
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}= ();

    # TODO debiese quedar cacheado?
    while (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);

        # Para retrocompatibilidad.
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_NOM'} = $row->{'USERS_NOM'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_USR'} = $row->{'USERS_USR'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_PSW'} = $row->{'USERS_PSW'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_PERFIL'} = $row->{'USERS_PROFILE_ID'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_EMAIL'} = $row->{'USERS_EMAIL'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_EXP_DAYS'} = $row->{'USERS_EXP_DAYS'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_EXP'} = $row->{'USERS_FEC_EXP'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_CREACION'} = $row->{'USERS_FEC_CREACION'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'USERS_FEC_LAST_LOGIN'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_RENEW_PSW'} = $row->{'USERS_RENEW_PSW'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_2FA'} = $row->{'USERS_2FA'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_OIDC'} = $row->{'USERS_OIDC'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_DELETED'} = $row->{'USERS_DELETED'};
        $prontus_varglb::USR_ID_REL{$row->{'USERS_USR'}} = $row->{'USERS_ID'};

        # nuevo hash para prontus 12.0.11+
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_NOM'} = $row->{'USERS_NOM'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_USR'} = $row->{'USERS_USR'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_PSW'} = $row->{'USERS_PSW'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_PERFIL'} = $row->{'USERS_PROFILE_ID'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_EMAIL'} = $row->{'USERS_EMAIL'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_EXP_DAYS'} = $row->{'USERS_EXP_DAYS'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_EXP'} = $row->{'USERS_FEC_EXP'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_CREACION'} = $row->{'USERS_FEC_CREACION'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'USERS_FEC_LAST_LOGIN'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_RENEW_PSW'} = $row->{'USERS_RENEW_PSW'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_2FA'} = $row->{'USERS_2FA'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_OIDC'} = $row->{'USERS_OIDC'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_DELETED'} = $row->{'USERS_DELETED'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$row->{'USERS_USR'}} = $row->{'USERS_ID'};
    }

    return (1, "");
}

# ---------------------------------------------------------------
sub load_user_template_access {
    my $user = $_[0];

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$user} eq '1') {
        return (1, "");
    }

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos de usuarios [$msg_err_bd]");
    }

    my $user_id = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$user};
    foreach my $prontus_id (keys %prontus_varglb::CONFIG) {
        $prontus_varglb::CONFIG{$prontus_id}{'ARTUSERS'}{$user_id} = undef;
        $prontus_varglb::CONFIG{$prontus_id}{'PORTUSERS'}{$user_id} = undef;
    }

    $prontus_varglb::ARTUSERS{$user_id} = undef;
    $prontus_varglb::PORTUSERS{$user_id} = undef;

    my @params = ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$user});
    my $sql = "SELECT * FROM ART_USERS WHERE ART_USER_ID = ?";
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    while (my $row = $salida->fetchrow_hashref) {
        $prontus_varglb::CONFIG{$row->{'ART_PRONTUS_ID'}}{'ARTUSERS'}{$row->{'ART_USER_ID'}}{$row->{'ART_TEMPLATE'}} = 1;
        $prontus_varglb::ARTUSERS{$row->{'ART_USER_ID'}}{$row->{'ART_TEMPLATE'}} = 1;
    }

    $sql = "SELECT * FROM PORT_USERS WHERE PORT_USER_ID = ?";
    $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
    if ($bd->errstr) {
        return (0, $bd->errstr);
    }
    while (my $row = $salida->fetchrow_hashref) {
        $prontus_varglb::CONFIG{$row->{'PORT_PRONTUS_ID'}}{'PORTUSERS'}{$row->{'PORT_USER_ID'}}{$row->{'PORT_TEMPLATE'}} = 1;
        $prontus_varglb::PORTUSERS{$row->{'PORT_USER_ID'}}{$row->{'PORT_TEMPLATE'}} = 1;
    }

    return (1, "");
}

# carga los datos de un usuario desde bd
sub load_user_data {
    my $user = $_[0];
    my $is_id = $_[1];

    my ($bd, $msg_err_bd) = &conectar_user_bd();
    if (!ref($bd)) {
        return (0, "No fue posible conectar a la base de datos [$msg_err_bd]");
    }

    $user =~ s/[^a-z\_\-\.0-9@]//;
    my $sql = "SELECT * FROM USERS WHERE USERS_USR = ? LIMIT 1";
    if (defined($is_id)) {
        $sql = "SELECT * FROM USERS WHERE USERS_ID = ? LIMIT 1";
    }
    my @params = ($user);
    my $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);

    if (my $row = $salida->fetchrow_hashref) {
        glib_str_03::correct_encode_utf8($row);
        # retrocompatiblidad
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_NOM'} = $row->{'USERS_NOM'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_USR'} = $row->{'USERS_USR'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_PSW'} = $row->{'USERS_PSW'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_PERFIL'} = $row->{'USERS_PROFILE_ID'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_EMAIL'} = $row->{'USERS_EMAIL'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_EXP_DAYS'} = $row->{'USERS_EXP_DAYS'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_EXP'} = $row->{'USERS_FEC_EXP'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_CREACION'} = $row->{'USERS_FEC_CREACION'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'USERS_FEC_LAST_LOGIN'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_RENEW_PSW'} = $row->{'USERS_RENEW_PSW'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_2FA'} = $row->{'USERS_2FA'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_OIDC'} = $row->{'USERS_OIDC'};
        $prontus_varglb::USERS{$row->{'USERS_ID'}}{'USERS_DELETED'} = $row->{'USERS_DELETED'};
        $prontus_varglb::USR_ID_REL{$row->{'USERS_USR'}} = $row->{'USERS_ID'};

        # nuevo hash para prontus 12.0.11+
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_NOM'} = $row->{'USERS_NOM'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_USR'} = $row->{'USERS_USR'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_PSW'} = $row->{'USERS_PSW'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_PERFIL'} = $row->{'USERS_PROFILE_ID'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_EMAIL'} = $row->{'USERS_EMAIL'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_EXP_DAYS'} = $row->{'USERS_EXP_DAYS'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_EXP'} = $row->{'USERS_FEC_EXP'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_CREACION'} = $row->{'USERS_FEC_CREACION'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_FEC_LAST_LOGIN'} = $row->{'USERS_FEC_LAST_LOGIN'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_RENEW_PSW'} = $row->{'USERS_RENEW_PSW'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_2FA'} = $row->{'USERS_2FA'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_OIDC'} = $row->{'USERS_OIDC'};
        $prontus_varglb::USER_DATA{$prontus_varglb::PRONTUS_ID}{$row->{'USERS_ID'}}{'USERS_DELETED'} = $row->{'USERS_DELETED'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USR_ID_REL'}{$row->{'USERS_USR'}} = $row->{'USERS_ID'};

        my $user_id = $row->{'USERS_ID'};
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$user_id} = undef;
        $sql = "SELECT PRONTUS_ID FROM PRONTUS_USERS WHERE PRONTUS_USER_ID = ?";
        @params = ($user_id);
        $salida = glib_dbi_03::ejecutar_sql_param($bd, $sql, \@params);
        while ($row = $salida->fetchrow_hashref) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_USERS'}{$user_id}{$row->{'PRONTUS_ID'}} = 1;
        }
        $salida->finish;

    } else {
        return (0, "No se encontraron datos del usuario [$user][$is_id]");
    }
    return (1, "");
}

# ---------------------------------------------------------------
# si esta activado el SSO y el manager no es este prontus, se debe conectar a la BD
# del prontus que administra los usuarios
sub conectar_user_bd {
    my $msg_err_bd = '';
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} eq 'SI' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} ne $prontus_varglb::PRONTUS_ID) {
        my %config = &get_user_bd_config_by_id($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'});
        # si ya existe la conexion a bd se reutiliza
        if (ref($prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID}) && $prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID}->ping == 1) {
            return ($prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd);
        }
        # no existe, se crea nueva conexion
        ($prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd) = lib_bd::conectar_prontus_bd($config{'NOM_BD'}, $config{'SERVER_BD'}, $config{'USER_BD'}, $config{'PWD_BD'});
        if (!ref($prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID})) {
            return ('', $msg_err_bd);
        }
        return ($prontus_varglb::BD_USER_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd);
    }
    # en caso contrario se conecta a la bd propia
    else {
        # Conectar a BD si es que no viene la conexion
        ($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd) = lib_bd::conectar_prontus_bd();
        if (!ref($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID})) {
            return ('', $msg_err_bd);
        }
        return ($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $msg_err_bd);
    }
}


# -------------------------------------------------------------------------#
sub get_user_bd_config_by_id {
    my $prontus_id = shift;
    # base de datos
    my $path_cfg = $prontus_varglb::DIR_SERVER. "/$prontus_id/cpan/$prontus_id-bd.cfg";
    my $buffer = glib_fildir_03::read_file($path_cfg);

    if (!-f $path_cfg) {
        print STDERR "No se pudo localizar la Configuración PRONTUS [$path_cfg]\n";
        return "Error: No se pudo localizar la Configuración PRONTUS";
    }

    my %config;
    # Parametros de conexion a la BD MySQL
    $config{'SERVER_BD'} = '';
    if ($buffer =~ m/SERVER_BD\s*=\s*["']([^'"]+)["']/) {
        $config{'SERVER_BD'} = $1;
    }
    $config{'NOM_BD'} = '';
    if ($buffer =~ m/NOM_BD\s*=\s*["']([^'"]+)["']/) {
        $config{'NOM_BD'} = $1;
    }
    $config{'USER_BD'} = '';
    if ($buffer =~ m/USER_BD\s*=\s*["']([^'"]+)["']/) {
        $config{'USER_BD'} = $1;
    }
    $config{'PWD_BD'} = '';
    if ($buffer =~ m/PWD_BD\s*=\s*["']([^'"]+)["']/) {
        $config{'PWD_BD'} = $1;
    }
    if ($config{'SERVER_BD'} eq '' || $config{'NOM_BD'} eq '' || $config{'USER_BD'} eq '' || $config{'PWD_BD'} eq '' ) {
        my $db_conn_error = "Configuración: Faltan parametros de conexion a la base de datos MySQL. Los parámetros son: SERVER_BD, NOM_BD, USER_BD y PWD_BD";
        print STDERR "$db_conn_error\n";
        return "Error: $db_conn_error";
    }
    return %config;
}

1;
