#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Utilidades generales para prontus
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use glib_fildir_03;
# use prontus_varglb;
# use glib_hrfec_03; para write_log y call_system_and_location
# use lib_waitlock; para write_log

use strict;
use utf8;

package lib_utils;

# Setea utf8 para stdout y stderr
sub setUtf8 {
    binmode STDERR, ':encoding(UTF-8)';
    binmode STDOUT, ':encoding(UTF-8)';
}
# -------------------------------------------------------------------------#
sub valida_prontus {
    my $prontus_id = $_[0];

    if (defined($prontus_varglb::CONFIG{$prontus_id})) {
        return 1;
    }

    if ($prontus_id !~ /^[\w\_\-]+$/) {
        print STDERR "Error valida_prontus: Prontus ID no valida REGEX\n";
        return 0;
    }

    # chequeamos que exista el prontus
    if (!-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id-bd.cfg") {
        print STDERR "Error valida_prontus: No hay conf de BD para este Prontus ID\n";
        return 0;
    }
    return 1;
}

# -------------------------------------------------------------------------#
# Carga la configuracion de prontus usando el id proporcionado
# Retorna los valores de c/u.
# ADVERTENCIA: NO usar greedy matching /g, en los if individuales, produce errores de matcheo
# solo usar en while
sub load_config_by_id {
    if (defined($_[0]) && $_[0] ne '' && $_[0] =~ /^[\w\_\-]+$/){
        $prontus_varglb::PRONTUS_ID = shift;
    } else {
        $prontus_varglb::PRONTUS_ID = 'prontus';
        print STDERR "No se pudo localizar la Configuración PRONTUS: Identificador no válido\n";
        return "Error: No se pudo localizar la Configuración PRONTUS";
    }

    if (defined($Prontus::modperl) && $prontus_varglb::CONFIG_LOADED{$prontus_varglb::PRONTUS_ID} > 0) {
        return '';
    }

    my $path_cfg = $prontus_varglb::DIR_SERVER. "/$prontus_varglb::PRONTUS_ID/cpan/$prontus_varglb::PRONTUS_ID";

    # Lee archivos de configuración.
    # configuración básica para que prontus funcione
    my $errcfg;
    $errcfg = "[$path_cfg-art.cfg]"  if (!-f $path_cfg . '-art.cfg');
    $errcfg = "[$path_cfg-port.cfg]" if (!-f $path_cfg . '-port.cfg');
    $errcfg = "[$path_cfg-bd.cfg]"   if (!-f $path_cfg . '-bd.cfg');
    $errcfg = "[$path_cfg-var.cfg]"  if (!-f $path_cfg . '-var.cfg');

    if ($errcfg) {
        print STDERR "No se pudo localizar la Configuración PRONTUS $errcfg\n";
        return "Error: No se pudo localizar la Configuración PRONTUS";
    }

    my $buffer = glib_fildir_03::read_file($path_cfg . '-art.cfg') . "\n"  # articulos
      . glib_fildir_03::read_file($path_cfg . '-port.cfg', 1) . "\n"          # portadas
      . glib_fildir_03::read_file($path_cfg . '-bd.cfg', 1) . "\n"            # base de datos
      . glib_fildir_03::read_file($path_cfg . '-tax.cfg', 1) . "\n"           # tax y taxports
      . glib_fildir_03::read_file($path_cfg . '-tag.cfg', 1) . "\n"           # tagports y tags
      . glib_fildir_03::read_file($path_cfg . '-list.cfg', 1) . "\n"          # portadas lista
      . glib_fildir_03::read_file($path_cfg . '-usr.cfg', 1) . "\n"           # permisos de usuarios
      . glib_fildir_03::read_file($path_cfg . '-var.cfg', 1) . "\n"           # variables generales
      . glib_fildir_03::read_file($path_cfg . '-xcoding.cfg', 1) . "\n"       # transcodificacion
      . glib_fildir_03::read_file($path_cfg . '-dropbox.cfg', 1) . "\n"       # dropbox
      . glib_fildir_03::read_file($path_cfg . '-cloudflare.cfg', 1) . "\n"    # cloudflare
      . glib_fildir_03::read_file($path_cfg . '-cache.cfg', 1) . "\n"         # cache
      . glib_fildir_03::read_file($path_cfg . '-sitemap.cfg', 1) . "\n"       # sitemap
      . glib_fildir_03::read_file($path_cfg . '-pplayer.cfg', 1) . "\n"       # configuracion prontus player
      . glib_fildir_03::read_file($path_cfg . '-redes.cfg', 1) . "\n"         # variables redes sociales
      . glib_fildir_03::read_file($path_cfg . '-oidc.cfg', 1) . "\n"          # variables openid connect.
      . glib_fildir_03::read_file($path_cfg . '-snp.cfg', 1) . "\n"           # notificaciones push.
      . glib_fildir_03::read_file($path_cfg . '-elasticsearch.cfg', 1) . "\n" # config elasticsearch
      . glib_fildir_03::read_file($path_cfg . '-seo.cfg', 1) . "\n";          # config SEO


    # elimina retornos de carro de lineas
    $buffer =~ s/\r/\n/g;
    $buffer =~ s/\n{2,}/\n/g;
    # print STDERR "buffer[$buffer]";

    if ($buffer !~ /\w/) {
        print STDERR "Archivos de Configuración PRONTUS están vacíos.\n";
        return "Error: Archivos de Configuración PRONTUS están vacíos.";
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} = '1';
    if ($buffer =~ m/FRIENDLY_URLS_VERSION\s*=\s*["']([N12345])["']/) {    # Es un char valor N, 1, 2, 3, 4, 5
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'} = 75;
    if ($buffer =~ m/FRIENDLY_URLS_LARGO_TITULAR\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V5_URL_PREFIX'} = '';
    if ($buffer =~ m/FRIENDLY_V5_URL_PREFIX\s*=\s*["']([\w\-\_]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V5_URL_PREFIX'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} = 'NO';
    if ($buffer =~ m/FRIENDLY_INCLUDE_VIEW_NAME\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} = 'SI';
    if ($buffer =~ m/FRIENDLY_V4_INCLUDE_PRONTUS_ID\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} = $1;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}} = ();
    while ($buffer =~ m/FRIENDLY_EXCLUDE_FID\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_EXCLUDE_FID'}{$1} = 1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CAPTCHA_OBLIGATORIO'} = 'SI';
    if ($buffer =~ m/CAPTCHA_OBLIGATORIO\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CAPTCHA_OBLIGATORIO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_API_URL'} = 'https://www.google.com/recaptcha/api/siteverify';
    if ($buffer =~ m/RECAPTCHA_API_URL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_API_URL'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_SECRET_CODE'} = '';
    if ($buffer =~ m/RECAPTCHA_SECRET_CODE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_SECRET_CODE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_SITEKEY'} = '';
    if ($buffer =~ m/RECAPTCHA_SITEKEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_SITEKEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_V3_SCORE'} = '0.5';
    if ($buffer =~ m/RECAPTCHA_V3_SCORE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RECAPTCHA_V3_SCORE'} = $1;
    }

    # Dropbox.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} = 'NO';
    if ($buffer =~ m/DROPBOX\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE'} = '';
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE_LIST'}} = ();
    if ($buffer =~ m/DROPBOX_FILEXT_EXCLUDE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE'} = $1;
        my @filext_exclude_list = split(/,/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE'});
        foreach my $ext (@filext_exclude_list) {
            $ext =~ s/\s//sg;
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_FILEXT_EXCLUDE_LIST'}{$ext} = 1;
        }
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CUSTOM_DIR'}} = ();
    while ($buffer =~ m/DROPBOX_CUSTOM_DIR\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CUSTOM_DIR'}{$1} = 1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_APPNAME'} = '';
    if ($buffer =~ m/DROPBOX_APPNAME\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_APPNAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_ID'} = '';
    if ($buffer =~ m/DROPBOX_CLIENT_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_SECRET'} = '';
    if ($buffer =~ m/DROPBOX_CLIENT_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_SECRET'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_AUTHORIZATION_CODE'} = '';
    if ($buffer =~ m/DROPBOX_AUTHORIZATION_CODE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_AUTHORIZATION_CODE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'} = '';
    if ($buffer =~ m/DROPBOX_REFRESH_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'} = $1;
    }

    # CloudFlare.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} = 'NO';
    if ($buffer =~ m/CLOUDFLARE\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'} = '';
    if ($buffer =~ m/CLOUDFLARE_API_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'} = '';
    if ($buffer =~ m/CLOUDFLARE_ZONE\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'} = '';
    if ($buffer =~ m/CLOUDFLARE_GLOBAL_PURGE\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'} = $1;
    }

    # Cache
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAXPORT'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_TAXPORT\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAXPORT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAGPORT'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_TAGPORT\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAGPORT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAXPORT_MV'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_TAXPORT_MV\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAXPORT_MV'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAGPORT_MV'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_TAGPORT_MV\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_TAGPORT_MV'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_MAPA'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_MAPA\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_MAPA'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_ART_RELAC'} = 'SI';
    if ($buffer =~ m/CACHE_PURGE_ART_RELAC\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_ART_RELAC'} = $1;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_EXCLUDE_FID'}} = ();
    while ($buffer =~ m/CACHE_PURGE_EXCLUDE_FID\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_EXCLUDE_FID'}{$1} = 1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTIVAR_FIDS_SIN_VISTA'} = 'NO';
    if ($buffer =~ m/ACTIVAR_FIDS_SIN_VISTA\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTIVAR_FIDS_SIN_VISTA'} = $1;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}} = ();
    if ($buffer =~ m/FIDS_CONTENIDO\s*=\s*["']([^'"]+)["']/s) {
        %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CONTENIDO'}} = map { $_ => 1 } split(',', $1);
    }

   %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}} = ();
    if ($buffer =~ m/FIDS_CALENDAR\s*=\s*["']([^'"]+)["']/s) {
        %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FIDS_CALENDAR'}} = map { $_ => 1 } split(',', $1);
    }

    # Transcodificacion - XCODING
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'N_THREADS'} = 0;
    if ($buffer =~ m/N_THREADS\s*=\s*["'](\d+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'N_THREADS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} = '';
    if ($buffer =~ m/XCODING_PPROC\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODING_PPROC'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'GEN_HLS'} = 'NO';
    if ($buffer =~ m/GEN_HLS\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'GEN_HLS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRECISION_HLS'} = 'NO';
    if ($buffer =~ m/PRECISION_HLS\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRECISION_HLS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIMIT_BITRATE'} = 'NO';
    if ($buffer =~ m/LIMIT_BITRATE\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIMIT_BITRATE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MODO_PARALELO'} = 'NO';
    if ($buffer =~ m/MODO_PARALELO\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MODO_PARALELO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_VIDEO_BITRATE'} = 2400;
    if ($buffer =~ m/MAX_VIDEO_BITRATE\s*=\s*["'](\d+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_VIDEO_BITRATE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'} = 128;
    if ($buffer =~ m/MAX_AUDIO_BITRATE\s*=\s*["'](\d+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_AUDIO_BITRATE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'} = '';
    if ($buffer =~ m/RUTA_TEMPORAL_XCODING\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'} = $1;
        if (substr($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'}, -1) ne '/') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'RUTA_TEMPORAL_XCODING'} .= '/';
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PIXEL'} = 1280;
    if ($buffer =~ m/XCODE_MAX_PIXEL\s*=\s*["'](\d+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PIXEL'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PARALELO'} = 3;
    if ($buffer =~ m/XCODE_MAX_PARALELO\s*=\s*["'](\d+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODE_MAX_PARALELO'} = $1;
    }

    # configuracion de transcodificacion externa
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_XCODER_EXTERNO'} = 'NO';
    if ($buffer =~ m/USAR_XCODER_EXTERNO\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_XCODER_EXTERNO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_HOST'} = '';
    if ($buffer =~ m/XCODER_HOST\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_HOST'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_PORT'} = '';
    if ($buffer =~ m/XCODER_PORT\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'XCODER_PORT'} = $1;
    }
    # /XCODING

    # extensiones permitidas para uploads
    if ($buffer =~ m/UPLOADS_PERMITIDOS\s*=\s*["'](\w+(,\w+)*)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPLOADS_PERMITIDOS'} = $1;
    } else {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPLOADS_PERMITIDOS'} = $prontus_varglb::UPLOADS_PERMITIDOS_ORIG;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}} = ();
    while ($buffer =~ m/MULTIVISTA\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}{$1} = 1;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}} = ();
    while ($buffer =~ m/FORM_PLTS\s*=\s*["'](.+?):(.+?)\((.*?)\)\((.*?)?\)["']/g) {
        next if (length($1) >64);
        my @templates = split(/;/, $3);

        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'label'} = $2;
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'plt'} = \@templates;
        my @templates2 = split(/;/, $4);
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$1}{'plt_paralela'} = \@templates2;
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'} = $1 if (!$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FID_DEFAULT'});
    }

    if (keys(%{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}}) < 1) {
        print STDERR "Configuración: No hay fids definidos\n";
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}} = ();
    # PORT_PLTS = 'inicio.html(vacante para portadas intercambiables, aca antes iba las spare, deprecatear eso)(plantillas paralelas)(portada para preview)'
    while ($buffer =~ m/PORT_PLTS\s*=\s*["'](.+?)\((.*?)\)\((.*?)\)(?:\(.*?\))?["']/g) {
        next if (length($1) >64);
        my $plantilla    = $1;    # plantilla principal de la portada
        my $nombre       = $2;    # contenido del primer parentesis, nombre de la portada
        my $extra_ports  = $3;    # segundo (): portadas paralelas, ej: inicio2.php;inicio3.php
        if ($plantilla !~ /^[a-zA-Z0-9_]+\.[a-zA-Z0-9_]+$/ || !defined($nombre) ) {
            print STDERR "Configuración: Plantilla de Portada [$plantilla] no es válido, debe contener sólo caracteres alfanuméricos, incluido el underscore. Además, debe tener una extensión válida.\n";
            next;
        }
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'label'} = $nombre;
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'plt'} = $plantilla;
        my @templates = split(/;/, $extra_ports);
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}{$plantilla}{'plt_paralela'} = \@templates;

    }

    if (keys(%{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_PLTS'}}) < 1) {
        print STDERR "Configuración: No hay portadas definidas\n";
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'} = [];
    while ($buffer =~ m/BASE_PORTS\s*=\s*["']([^'"]+)["']/g) {
        push @{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BASE_PORTS'}}, $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} = '';
    if ($buffer =~ m/PUBLIC_SERVER_NAME\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};
    if ($buffer =~ m/CPAN_SERVER_NAME\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} = 'NO';
    if ($buffer =~ m/CONTROL_FECHA\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} = 'NO';
    if ($buffer =~ m/MULTI_EDICION\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ADMIN_PORT'} = 'NO';
    if ($buffer =~ m/ADMIN_PORT\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ADMIN_PORT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_PORT'} = 'SI';
    if ($buffer =~ m/PUBLIC_PORT\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_PORT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDACTOR_EDITAR_ARTICULOS_AJENOS'} = 'SI';
    if ($buffer =~ m/REDACTOR_EDITAR_ARTICULOS_AJENOS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDACTOR_EDITAR_ARTICULOS_AJENOS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDACTOR_VER_ARTICULOS_AJENOS'} = 'SI';
    if ($buffer =~ m/REDACTOR_VER_ARTICULOS_AJENOS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDACTOR_VER_ARTICULOS_AJENOS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_EDITAR_ARTICULOS_AJENOS'} = 'SI';
    if ($buffer =~ m/EDITOR_EDITAR_ARTICULOS_AJENOS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_EDITAR_ARTICULOS_AJENOS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_VER_ARTICULOS_AJENOS'} = 'SI';
    if ($buffer =~ m/EDITOR_VER_ARTICULOS_AJENOS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_VER_ARTICULOS_AJENOS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_ADMINISTRAR_EDICIONES'} = 'SI';
    if ($buffer =~ m/EDITOR_ADMINISTRAR_EDICIONES\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDITOR_ADMINISTRAR_EDICIONES'} = $1;
    }

    # activar single sign-on
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} = 'NO';
    if ($buffer =~ m/PRONTUS_SSO\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} = $1;
    }

    # es el prontus que administra todos los usuarios
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER'} = 'NO';
    if ($buffer =~ m/PRONTUS_SSO_MASTER\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER'} = $1;
    }

    # id del prontus con los usuarios
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'} = '';
    if ($buffer =~ m/PRONTUS_SSO_MASTER_ID\s*=\s*["']([\w\_\-]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_INI_SELECTED'} = '';
    if ($buffer =~ m/PORT_INI_SELECTED\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_INI_SELECTED'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_HOME'} = '';
    if ($buffer =~ m/PORT_HOME\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_HOME'} = $1;
    }

    # -snp.cfg - Notificaciones push
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} = 'NO';
    if ($buffer =~ m/SNP_APP\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_SERVER_KEY'} = '';
    if ($buffer =~ m/SNP_APP_SERVER_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_SERVER_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_SERVER'} = '';
    if ($buffer =~ m/SNP_API_SERVER\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_SERVER'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_KEY'} = '';
    if ($buffer =~ m/SNP_API_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_APP_ID'} = '';
    if ($buffer =~ m/SNP_API_APP_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_APP_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_CLIENT_ID'} = '';
    if ($buffer =~ m/SNP_API_CLIENT_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_API_CLIENT_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_CUSTOM_CSS'} = '';
    if ($buffer =~ m/SNP_APP_CUSTOM_CSS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_CUSTOM_CSS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_ANALYTICS_UTM'} = '';
    if ($buffer =~ m/SNP_APP_ANALYTICS_UTM\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_ANALYTICS_UTM'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_FOTOFIJA'} = '';
    if ($buffer =~ m/SNP_APP_FOTOFIJA\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_FOTOFIJA'} = $1;
    }

    # configuración de ElasticSearch
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} = 'NO';
    if ($buffer =~ m/ELASTICSEARCH_HABILITADO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_FIDS'} = '';
    if ($buffer =~ m/ELASTICSEARCH_FIDS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_FIDS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TEXTVARS'} = '';
    if ($buffer =~ m/ELASTICSEARCH_TEXTVARS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TEXTVARS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HOST'} = '';
    if ($buffer =~ m/ELASTICSEARCH_HOST\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HOST'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HOST_PROTOCOL'} = '';
    if ($buffer =~ m/ELASTICSEARCH_HOST_PROTOCOL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HOST_PROTOCOL'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PORT'} = '';
    if ($buffer =~ m/ELASTICSEARCH_PORT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PORT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_INDEX'} = '';
    if ($buffer =~ m/ELASTICSEARCH_INDEX\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_INDEX'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_BATCH_CHUNK'} = '50';
    if ($buffer =~ m/ELASTICSEARCH_BATCH_CHUNK\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_BATCH_CHUNK'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MAXCARS'} = '100000';
    if ($buffer =~ m/ELASTICSEARCH_MAXCARS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MAXCARS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_FIDS'} = '';
    if ($buffer =~ m/ELASTICSEARCH_FIDS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_FIDS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_RATIO'} = '';
    if ($buffer =~ m/ELASTICSEARCH_RATIO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_RATIO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TIPO_PAGINACION'} = '0';
    if ($buffer =~ m/ELASTICSEARCH_TIPO_PAGINACION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TIPO_PAGINACION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PAGCORTA_MAXPAGS'} = '5';
    if ($buffer =~ m/ELASTICSEARCH_PAGCORTA_MAXPAGS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PAGCORTA_MAXPAGS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MAXRESULT'} = '5';
    if ($buffer =~ m/ELASTICSEARCH_SEARCHTIPS_MAXRESULT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MAXRESULT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MINTEXT'} = '';
    if ($buffer =~ m/ELASTICSEARCH_MINTEXT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MINTEXT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_DURACION_CACHE'} = '300';
    if ($buffer =~ m/ELASTICSEARCH_SEARCHTIPS_DURACION_CACHE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_DURACION_CACHE'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MAXREQUESTXIP'} = '99';
    if ($buffer =~ m/ELASTICSEARCH_SEARCHTIPS_MAXREQUESTXIP\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MAXREQUESTXIP'} = $1;
    }

    # iteramos sobre las variables ELASTICSEARCH_SEARCH_META*
    for (my $iter = 1; $iter <= 5; $iter++) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_SEARCH_META$iter"} = '';
        if ($buffer =~ m/ELASTICSEARCH_SEARCH_META$iter\s*=\s*["']([^'"]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_SEARCH_META$iter"} = $1;
        }
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_SEARCH_META${iter}_TIPO"} = '';
        if ($buffer =~ m/ELASTICSEARCH_SEARCH_META${iter}_TIPO\s*=\s*["']([^'"]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_SEARCH_META${iter}_TIPO"} = $1;
        }
    }

    # iteramos sobre las variables ELASTICSEARCH_METADATA*
    for (my $iter = 1; $iter <= 10; $iter++) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_METADATA$iter"} = '';
        if ($buffer =~ m/ELASTICSEARCH_METADATA$iter\s*=\s*["']([^'"]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_METADATA$iter"} = $1;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_METADATA_IMG'} = '';
    if ($buffer =~ m/ELASTICSEARCH_METADATA_IMG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_METADATA_IMG'} = $1;
    }

    # iteramos sobre las variables ELASTICSEARCH_PROMOTED_TEXT* y _TS*
    for (my $iter = 1; $iter <= 5; $iter++) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_PROMOTED_TEXT$iter"} = '';
        if ($buffer =~ m/ELASTICSEARCH_PROMOTED_TEXT$iter\s*=\s*["']([^'"]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_PROMOTED_TEXT$iter"} = $1;
        }
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_PROMOTED_TS$iter"} = '';
        if ($buffer =~ m/ELASTICSEARCH_PROMOTED_TS$iter\s*=\s*["']([^'"]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{"ELASTICSEARCH_PROMOTED_TS$iter"} = $1;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_RESPERPAG'} = '20';
    if ($buffer =~ m/ELASTICSEARCH_RESPERPAG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_RESPERPAG'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TIPO_PAGINACION'} = '0';
    if ($buffer =~ m/ELASTICSEARCH_TIPO_PAGINACION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_TIPO_PAGINACION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PAGCORTA_MAXPAGS'} = '1000';
    if ($buffer =~ m/ELASTICSEARCH_PAGCORTA_MAXPAGS\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_PAGCORTA_MAXPAGS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MAXEXEC'} = '500';
    if ($buffer =~ m/ELASTICSEARCH_MAXEXEC\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_MAXEXEC'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MINLEN'} = '2';
    if ($buffer =~ m/ELASTICSEARCH_SEARCHTIPS_MINLEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_SEARCHTIPS_MINLEN'} = $1;
    }

    # configuración de SEO
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_HABILITADO'} = 'NO';
    if ($buffer =~ m/SEO_API_HABILITADO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_HABILITADO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_TWITTER_HANDLE'} = '';
    if ($buffer =~ m/SEO_TWITTER_HANDLE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_TWITTER_HANDLE'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_FB_ID'} = '';
    if ($buffer =~ m/SEO_FB_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_FB_ID'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_GLOBAL_ID'} = '';
    if ($buffer =~ m/SEO_GLOBAL_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_GLOBAL_ID'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_KEY'} = '';
    if ($buffer =~ m/SEO_API_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_KEY'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_SERVER'} = '';
    if ($buffer =~ m/SEO_API_SERVER\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_SERVER'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_PORT'} = '';
    if ($buffer =~ m/SEO_API_PORT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_PORT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_VERSION'} = 'v1';
    if ($buffer =~ m/SEO_API_VERSION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_API_VERSION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_TITULAR'} = 'seo_titulo';
    if ($buffer =~ m/SEO_NAME_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_SLUG'} = '_slug';
    if ($buffer =~ m/SEO_NAME_SLUG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_SLUG'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_DESCRIPCION'} = 'seo_descripcion';
    if ($buffer =~ m/SEO_NAME_DESCRIPCION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_DESCRIPCION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_CUERPO'} = 'vtxt_cuerpo';
    if ($buffer =~ m/SEO_NAME_CUERPO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_NAME_CUERPO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_TITULAR'} = '25';
    if ($buffer =~ m/SEO_MIN_CARACTERES_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_TITULAR'} = '60';
    if ($buffer =~ m/SEO_MAX_CARACTERES_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_TITULAR'} = '3';
    if ($buffer =~ m/SEO_MIN_PALABRAS_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_TITULAR'} = '10';
    if ($buffer =~ m/SEO_MAX_PALABRAS_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_TITULAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_TITULAR'} = '';
    if ($buffer =~ m/SEO_MIN_CARACTERES_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_TITULAR'} = '';
    if ($buffer =~ m/SEO_MAX_CARACTERES_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_TITULAR'} = '';
    if ($buffer =~ m/SEO_MIN_PALABRAS_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_TITULAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_TITULAR'} = '';
    if ($buffer =~ m/SEO_MAX_PALABRAS_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_TITULAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_DESCRIPCION'} = '120';
    if ($buffer =~ m/SEO_MIN_CARACTERES_DESCRIPCION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_DESCRIPCION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_DESCRIPCION'} = '156';
    if ($buffer =~ m/SEO_MAX_CARACTERES_DESCRIPCION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_DESCRIPCION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_DESCRIPCION'} = '20';
    if ($buffer =~ m/SEO_MIN_PALABRAS_DESCRIPCION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_DESCRIPCION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_DESCRIPCION'} = '26';
    if ($buffer =~ m/SEO_MAX_PALABRAS_DESCRIPCION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_DESCRIPCION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_CUERPO'} = '1500';
    if ($buffer =~ m/SEO_MIN_CARACTERES_CUERPO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_CUERPO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_CUERPO'} = '30000';
    if ($buffer =~ m/SEO_MAX_CARACTERES_CUERPO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_CUERPO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_CUERPO'} = '250';
    if ($buffer =~ m/SEO_MIN_PALABRAS_CUERPO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_PALABRAS_CUERPO'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_CUERPO'} = '5000';
    if ($buffer =~ m/SEO_MAX_PALABRAS_CUERPO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_PALABRAS_CUERPO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_SLUG'} = '4';
    if ($buffer =~ m/SEO_MIN_CARACTERES_SLUG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MIN_CARACTERES_SLUG'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_SLUG'} = '90';
    if ($buffer =~ m/SEO_MAX_CARACTERES_SLUG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MAX_CARACTERES_SLUG'} = $1;
    }

    # Caja div
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_SUBSCRIBE'} = 'Suscribir notificaciones';
    if ($buffer =~ m/SNP_APP_BOXMSG_BUTTON_SUBSCRIBE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_SUBSCRIBE'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_UNSUBSCRIBE'} = 'Desuscribir notificaciones';
    if ($buffer =~ m/SNP_APP_BOXMSG_BUTTON_UNSUBSCRIBE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_BUTTON_UNSUBSCRIBE'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBE_INVITATION'} = 'Suscríbete y recibe todas nuestras notificaciones';
    if ($buffer =~ m/SNP_APP_BOXMSG_SUBSCRIBE_INVITATION\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBE_INVITATION'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBED'} = 'Ya estás suscrito a nuestras notificaciones';
    if ($buffer =~ m/SNP_APP_BOXMSG_SUBSCRIBED\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_SUBSCRIBED'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOTIFICATION_DENIED'} = 'Las notificaciones no están habilitadas para este sitio';
    if ($buffer =~ m/SNP_APP_BOXMSG_NOTIFICATION_DENIED\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOTIFICATION_DENIED'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOT_SUPPORTED'} = 'Esta opción no está soportada por tu navegador';
    if ($buffer =~ m/SNP_APP_BOXMSG_NOT_SUPPORTED\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_BOXMSG_NOT_SUPPORTED'} = $1;
    }
    # Modal
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_SHOW'} = 'NO';
    if ($buffer =~ m/SNP_APP_MODAL_SHOW\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_SHOW'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_DELAY'} = '1';
    if ($buffer =~ m/SNP_APP_MODAL_DELAY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_DELAY'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_ICON_IMG'} = '/prontus/cpan/imag/logo/prontus-logo-40x40w.png';
    if ($buffer =~ m/SNP_APP_MODAL_ICON_IMG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_ICON_IMG'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BODY_TEXT'} = '¿Deseas recibir notificaciones de nuestro sitio web?';
    if ($buffer =~ m/SNP_APP_MODAL_BODY_TEXT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BODY_TEXT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_ACCEPT'} = 'Aceptar';
    if ($buffer =~ m/SNP_APP_MODAL_BUTTON_ACCEPT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_ACCEPT'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_CANCEL'} = 'No gracias';
    if ($buffer =~ m/SNP_APP_MODAL_BUTTON_CANCEL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP_MODAL_BUTTON_CANCEL'} = $1;
    }

    # Parametros de conexion a la BD MySQL
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'} = '';
    if ($buffer =~ m/SERVER_BD\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} = '';
    if ($buffer =~ m/NOM_BD\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'} = '';
    if ($buffer =~ m/USER_BD\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'} = '';
    if ($buffer =~ m/PWD_BD\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'} = $1;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'} eq '' || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} eq '' || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'} eq '' || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'} eq '' ) {
        my $db_conn_error = "Configuración: Faltan parametros de conexion a la base de datos MySQL. Los parámetros son: SERVER_BD, NOM_BD, USER_BD y PWD_BD";
        print STDERR "$db_conn_error\n";
        return "Error: $db_conn_error";
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} = 500;
    if ($buffer =~ m/TAXPORT_MAXARTICS\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} = $1;
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} > $prontus_varglb::TAXPORT_MAXARTICS_SECURITY) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAXARTICS'} = $prontus_varglb::TAXPORT_MAXARTICS_SECURITY;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} = 4;
    if ($buffer =~ m/TAXPORT_MAX_WORKERS\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_TIPO_PAGINACION'} = '0';
    if ($buffer =~ m/TAXPORT_TIPO_PAGINACION\s*=\s*["'](0|1)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_TIPO_PAGINACION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_PAGCORTA_MAXPAGS'} = '5';
    if ($buffer =~ m/TAXPORT_PAGCORTA_MAXPAGS\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_PAGCORTA_MAXPAGS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} = 1;
    if ($buffer =~ m/TAXONOMIA_NIVELES\s*=\s*["']([0-3])["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} = $1;
    }

    # Si esto está activo no se ejecutara el cron taxport.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} = 'NO';
    if ($buffer =~ m/TAXPORT_DINAMICAS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_DINAMICAS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ARTIC_ACTUALIZA_PORTS'} = 'NO';
    if ($buffer =~ m/ARTIC_ACTUALIZA_PORTS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ARTIC_ACTUALIZA_PORTS'} = $1;
    }

    # tagonomicas
    # tagonomicas.TAGPORT_MAXARTICS
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} = 500;
    if ($buffer =~ m/TAGPORT_MAXARTICS\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} = $1;
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} > 1000) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_MAXARTICS'} = 1000;
        }
    }

    # tagonomicas.TAGPORT_ARTXPAG
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ARTXPAG'} = 20;
    if ($buffer =~ m/TAGPORT_ARTXPAG\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ARTXPAG'} = $1;
    }

    # tagonomicas.TAGPORT_ORDEN
    my $tagport_orden_error = "Configuración: la variable TAGPORT_ORDEN contiene un valor no válido. Valores posibles: 'PUBLICACION(ASC|DESC)', 'TITULAR(ASC|DESC)', 'CREACION(ASC|DESC)'. Por omisión es: 'PUBLICACION(DESC)'";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = 'ART_FECHAPHORAP DESC, ART_ID DESC';
    if ($buffer =~ m/TAGPORT_ORDEN\s*=\s*["'](PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)["']/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "ART_FECHAPHORAP $2, ART_ID $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "ART_TITU $2, ART_ID $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAGPORT_ORDEN'} = "ART_AUTOINC $2, ART_ID $2";
        } else {
            print STDERR "$tagport_orden_error [$1][$2]\n";
        }
    } else {
        print STDERR "$tagport_orden_error\n";
    }

    # ---------
    # list.LIST_PROCESO_INTERNO
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'INCLUIR_PRONTUS_ID_URL'} = 'NO';
    if ($buffer =~ m/INCLUIR_PRONTUS_ID_URL\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'INCLUIR_PRONTUS_ID_URL'} = $1;
    }

    # ---------
    # list.LIST_PROCESO_INTERNO
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PROCESO_INTERNO'} = 'SI';
    if ($buffer =~ m/LIST_PROCESO_INTERNO\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PROCESO_INTERNO'} = $1;
    }

    # list.LIST_PORT_PPROC
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PORT_PPROC'} = 'NO';
    if ($buffer =~ m/LIST_PORT_PPROC\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PORT_PPROC'} = $1;
    }

    # list.LIST_MAXARTICS
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} = 20;
    if ($buffer =~ m/LIST_MAXARTICS\s*=\s*["']([0-9]+?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} = $1;
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} > 1000) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_MAXARTICS'} = 1000;
        }
    }

    # list.LIST_ORDEN
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = 'ART_FECHAPHORAP DESC, ART_ID DESC';
    my $list_orden_error = "Configuración: la variable LIST_ORDEN contiene un valor no válido. Valores posibles: 'PUBLICACION(ASC|DESC)', 'TITULAR(ASC|DESC)', 'CREACION(ASC|DESC)'. Por omisión es: 'PUBLICACION(DESC)'";
    if ($buffer =~ m/LIST_ORDEN\s*=\s*["'](PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)["']/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "ART_FECHAPHORAP $2, ART_ID $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "ART_TITU $2, ART_ID $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_ORDEN'} = "ART_AUTOINC $2, ART_ID $2";
        } else {
            print STDERR "$list_orden_error [$1][$2]\n";
        }
    } else {
        print STDERR "$list_orden_error\n";
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PLTS'}} = ();
    # Las carga igual aunque no esté configurada friendly 5.
    while ($buffer =~ m/LIST_PLTS\s*=\s*["'](\w+\.\w+)["']/g) { # LIST_PLTS = 'articulos.html';
        next if (length($1) >64);
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'LIST_PLTS'}{$1} = 1;
    }

    # ---------
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FFMPEG_AVAILABLE'} = 0;
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'} = '';
    # si el dir se define por cfg, usa ese en lugar del por defecto
    if ($buffer =~ m/DIR_FFMPEG\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'} = $1;
        if (!-s "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg") {
            my $dir_ffmpeg_error = "Configuración: variable DIR_FFMPEG [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_FFMPEG'}/ffmpeg] no existe en el sistema.";
            print STDERR "$dir_ffmpeg_error\n";
        } else {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FFMPEG_AVAILABLE'} = 1;
        }
    }

    # Configuracion de formato de salida de html en VTXT
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORMATO_HTML_VTXT'} = 'HTML';    # XHTML | HTML
    if ($buffer =~ m/FORMATO_HTML_VTXT\s*=\s*["'](HTML|XHTML)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORMATO_HTML_VTXT'} = $1;
    }

    # Indica si se debe realizar escapeo o no
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_ENCODE_CHARS'} = 'SI';
    if ($buffer =~ m/VTXT_ENCODE_CHARS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_ENCODE_CHARS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_PRONTUS_CARDS'} = 'NO';
	if ($buffer =~ m/VTXT_PRONTUS_CARDS\s*=\s*["'](SI|NO)["']/) {
		$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_PRONTUS_CARDS'} = $1;
	}

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_IMAGE_SIZE_LIMIT'} = '0';
    if ($buffer =~ m/VTXT_IMAGE_SIZE_LIMIT\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_IMAGE_SIZE_LIMIT'} = $1;
    }

    # Habilita o deshabilita el corrector ortográfico
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ENABLE_SPELL_CHECKER'} = 'NO';
    if ($buffer =~ m/ENABLE_SPELL_CHECKER\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ENABLE_SPELL_CHECKER'} = $1;
    }

    # Habilita o deshabilita la regeneración automática de tax|tag ports
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REGENERACION_TAX_TAG'} = 'SI';
    if ($buffer =~ m/REGENERACION_TAX_TAG\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REGENERACION_TAX_TAG'} = $1;
    }

    # Single Sign-On con OpenID Connect
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} = 'NO';
    if ($buffer =~ m/OIDC_HABILITADO\s*=\s*["'](SI)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_TITULAR'} = '';
    if ($buffer =~ m/OIDC_TITULAR\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_TITULAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_ENDPOINT_URI'} = '';
    if ($buffer =~ m/OIDC_ENDPOINT_URI\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_ENDPOINT_URI'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_CLIENT_ID'} = '';
    if ($buffer =~ m/OIDC_CLIENT_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_CLIENT_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_APP_SECRET'} = '';
    if ($buffer =~ m/OIDC_APP_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_APP_SECRET'} = $1;
    }


    # actualizaciones automaticas
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTUALIZACIONES'} = 'SI';
    if ($buffer =~ m/ACTUALIZACIONES\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ACTUALIZACIONES'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_RELPATH_LINK'} = 'SI';
    if ($buffer =~ m/VTXT_RELPATH_LINK\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'VTXT_RELPATH_LINK'} = $1;
    }

    # Para saber si el FID se abre en nueva pestaña o no
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ABRIR_FIDS_NEW_TAB'} = 'SI';
    if ($buffer =~ m/ABRIR_FIDS_NEW_TAB\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ABRIR_FIDS_NEW_TAB'} = $1;
    }

    # Para usar nombre del servidor al ver un articulo desde el FID.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_PUBLIC_SERVER_NAME_VER_ARTIC'} = 'NO';
    if ($buffer =~ m/USAR_PUBLIC_SERVER_NAME_VER_ARTIC\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_PUBLIC_SERVER_NAME_VER_ARTIC'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NUM_RELAC_DEFAULT'} = 5;
    if ($buffer =~ m/NUM_RELAC_DEFAULT\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NUM_RELAC_DEFAULT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ARTXPAG'} = 20;
    if ($buffer =~ m/TAXPORT_ARTXPAG\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ARTXPAG'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_AREA_MAX_ARTICS'} = '';
    if ($buffer =~ m/PORT_AREA_MAX_ARTICS\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORT_AREA_MAX_ARTICS'} = $1;
    }

    # TAXPORT_ORDEN
    # valor por defecto.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = 'ART_FECHAPHORAP DESC, ART_ID DESC';
    if ($buffer =~ m/TAXPORT_ORDEN\s*=\s*["'](PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)["']/) {
        if ($1 eq 'PUBLICACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "ART_FECHAPHORAP $2, ART_ID $2";
        } elsif ($1 eq 'TITULAR') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "ART_TITU $2, ART_ID $2";
        } elsif ($1 eq 'CREACION') {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_ORDEN'} = "ART_AUTOINC $2, ART_ID $2";
        } else {
            print STDERR "Configuración: seteo de variable TAXPORT_ORDEN contiene un valor no válido.\nValores posibles: 'PUBLICACION(ASC|DESC)', 'TITULAR(ASC|DESC)', 'CREACION(ASC|DESC)', Por omisión es: 'PUBLICACION(DESC)'\n";
        }
    } else {
        print STDERR "Configuración: seteo de variable TAXPORT_ORDEN contiene un valor no válido.\nValores posibles: 'PUBLICACION(ASC|DESC)', 'TITULAR(ASC|DESC)', 'CREACION(ASC|DESC)', Por omisión es: 'PUBLICACION(DESC)'\n";
    }

    # EDICION BASE SELECCIONADA POR DEFECTO
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDICBASE_INI_SELECTED'} = 'NO';
    if ($buffer =~ m/EDICBASE_INI_SELECTED\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'EDICBASE_INI_SELECTED'} = $1;
    }

    # smtp
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'} = '';
    if ($buffer =~ m/SERVER_SMTP\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'} = 'altavoz.net';
    if ($buffer =~ m/PRONTUS_MAIL_DOMAIN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'} = $1;
    }

    # Para indicar si se usará https
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'} = 'http';
    if ($buffer =~ m/SERVER_PROTOCOLO_FRONTEND\s*=\s*["'](http|https)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'} = $1;
    }

    # Para indicar si se usará https en el backend
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'} = 'http';
    if ($buffer =~ m/SERVER_PROTOCOLO_CPAN\s*=\s*["'](http|https)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_CPAN'} = $1;
    }

    # Cache purge
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}} = ();
    while ($buffer =~ m/CACHE_SERVER_NAME\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}{$1} = 1;
    }

    # Subdominios a considerar al hacer purge.
    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_SUBDOMAINS'}} = ();
    while ($buffer =~ m/CACHE_PURGE_SUBDOMAINS\s*=\s*["']([^'"]+)["']/g) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_SUBDOMAINS'}{$1} = 1;
    }

    # Cache global purge
    if ($buffer =~ m/CACHE_GLOBAL_PURGE\s*=\s*["']([^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_GLOBAL_PURGE'} = $1;
    }

    # Configuracion de la codificacion del archivo de respaldo del prontus form
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_CSV_CHARSET'} = 'utf-8';
    if ($buffer =~ m/FORM_CSV_CHARSET\s*=\s*["'](utf-8|iso-8859-1)["']/i) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_CSV_CHARSET'} = $1;
    }

    # Activar funciones de SEO de Prontus
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} = 'NO';
    if ($buffer =~ m/\s*SEO_ACTIVO\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} = $1;
    }

    # Activar funciones de SEO de Prontus para multivistas
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MV'} = 'NO';
    if ($buffer =~ m/\s*SEO_MV\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MV'} = $1;
    }

    # Configuracion de la codificacion del archivo de respaldo del prontus form
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} = 'NO';
    if ($buffer =~ m/\s*HABILITAR_PLUGINS\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} = $1;
    }

    # Configuracion de la inclusión de archivo subido como adjunto al mail al admin
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_INCLUIR_ADJUNTO'} = 'SI';
    if ($buffer =~ m/FORM_INCLUIR_ADJUNTO\s*=\s*["'](SI|NO|)["']/i) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_INCLUIR_ADJUNTO'} = $1;
    }

    %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'POST_PROCESO'}} = ();
    while ($buffer =~ m/POST_PROCESO\s*=\s*["']([^'"\(\)]+)\(([^'"\(\)]+)\)["']/g) {
        # ART-BORRAR | ART-GUARDAR | ART-EDITAR | PORT_GUARDAR | etc
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'POST_PROCESO'}{$1} = $2;
    }

    # El valor por defecto es vacio
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SCRIPT_QUOTA'} = '';
    if ($buffer =~ m/SCRIPT_QUOTA\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SCRIPT_QUOTA'} = $1;
    }

    # El valor por defecto es vacio
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_VIDEO'} = '/prontus/front/player/video.html';
    if ($buffer =~ m/PLAYER_EMBED_VIDEO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_VIDEO'} = $1;
    }

    # El valor por defecto es vacio
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_AUDIO'} = '/prontus/front/player/audio.html';
    if ($buffer =~ m/PLAYER_EMBED_AUDIO\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PLAYER_EMBED_AUDIO'} = $1;
    }

    # El valor por defecto es vacio
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FOTO_MAX_PIXEL'} = '';
    if ($buffer =~ m/FOTO_MAX_PIXEL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FOTO_MAX_PIXEL'} = $1;
    }

    # El valor por defecto es NO
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDUCIR_CALIDAD_JPEGS'} = 'NO';
    if ($buffer =~ m/REDUCIR_CALIDAD_JPEGS\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'REDUCIR_CALIDAD_JPEGS'} = $1;
    }

    # El valor por defecto es 85. Sólo acepto valores en dígitos enteros entre 0 y 100. 0 es "usar el default": https://libgd.github.io/manuals/2.2.5/files/gd_jpeg-c.html
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NIVEL_OPTIMIZACION_JPG'} = '85';
    if ($buffer =~ m/NIVEL_OPTIMIZACION_JPG\s*=\s*["'](\d{1,3})["']/) {
        my $tmp_optimizacion = $1;
        if ($tmp_optimizacion <= 100 && $tmp_optimizacion >= 0) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NIVEL_OPTIMIZACION_JPG'} = $tmp_optimizacion;
        }
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'} =  500;
    if ($buffer =~ m/MAX_XCODING\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'} = $1;
    }

    # El valor por defecto es 0
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BLOQUEO_EDICION'} = 0;
    if ($buffer =~ m/BLOQUEO_EDICION\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'BLOQUEO_EDICION'} = $1;
    }

    # opciones avanzadas de transcodificacion
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_LIB_FDK'} = 'NO';
    if ($buffer =~ m/USAR_LIB_FDK\s*=\s*["'](SI|NO)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USAR_LIB_FDK'} = $1;
    }

    # Update.
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPDATE_SERVER'} = 'https://www.prontus.cl';
    if ($buffer =~ m/UPDATE_SERVER\s*=\s*["'](http[^'"]+)["']/s) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'UPDATE_SERVER'} = $1;
    }

    # Configuracion Redes Sociales
    #FACEBOOK
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FACEBOOK_POST'} = 'NO';
    if ($buffer =~ m/FACEBOOK_POST\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FACEBOOK_POST'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_PAGE_ID'} = '';
    if ($buffer =~ m/FB_PAGE_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_PAGE_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_APP_ID'} = '';
    if ($buffer =~ m/FB_APP_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_APP_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_APP_SECRET'} = '';
    if ($buffer =~ m/FB_APP_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_APP_SECRET'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_ACCESS_TOKEN'} = '';
    if ($buffer =~ m/FB_ACCESS_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_ACCESS_TOKEN'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_PICTURE_WALL'} = '';
    if ($buffer =~ m/FB_PICTURE_WALL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FB_PICTURE_WALL'} = $1;
    }

    #TWITTER
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TWITTER_POST'} = 'NO';
    if ($buffer =~ m/TWITTER_POST\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TWITTER_POST'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_TWITTER_ID'} = '';
    if ($buffer =~ m/TT_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_TWITTER_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_CONSUMER_KEY'} = '';
    if ($buffer =~ m/TT_CONSUMER_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_CONSUMER_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_CONSUMER_SECRET'} = '';
    if ($buffer =~ m/TT_CONSUMER_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_CONSUMER_SECRET'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_AUTH_TOKEN'} = '';
    if ($buffer =~ m/TT_AUTH_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_AUTH_TOKEN'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_AUTH_TOKEN_SECRET'} = '';
    if ($buffer =~ m/TT_AUTH_TOKEN_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_AUTH_TOKEN_SECRET'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_BITLY_USER'} = '';
    if ($buffer =~ m/TT_BITLY_USER\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_BITLY_USER'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_BITLY_KEY'} = '';
    if ($buffer =~ m/TT_BITLY_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_BITLY_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_GOOGL_USER'} = '';
    if ($buffer =~ m/TT_GOOGL_USER\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_GOOGL_USER'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_GOOGL_KEY'} = '';
    if ($buffer =~ m/TT_GOOGL_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_GOOGL_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_PICTURE_WALL'} = '';
    if ($buffer =~ m/TT_PICTURE_WALL\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TT_PICTURE_WALL'} = $1;
    }

    # YouTube
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_POST'} = 'NO';
    if ($buffer =~ m/YOUTUBE_POST\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_POST'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_APPNAME'} = '';
    if ($buffer =~ m/YOUTUBE_APPNAME\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_APPNAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_CLIENT_ID'} = '';
    if ($buffer =~ m/YOUTUBE_CLIENT_ID\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_CLIENT_ID'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_CLIENT_SECRET'} = '';
    if ($buffer =~ m/YOUTUBE_CLIENT_SECRET\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_CLIENT_SECRET'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_AUTHORIZATION_CODE'} = '';
    if ($buffer =~ m/YOUTUBE_AUTHORIZATION_CODE\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_AUTHORIZATION_CODE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_REFRESH_TOKEN'} = '';
    if ($buffer =~ m/YOUTUBE_REFRESH_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_REFRESH_TOKEN'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_OAUTH_TOKEN'} = '';
    if ($buffer =~ m/YOUTUBE_OAUTH_TOKEN\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'YOUTUBE_OAUTH_TOKEN'} = $1;
    }
    # fin configuración redes sociales.

    # Amazon S3
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_BUCKET_SIZE'} = 0;
    if ($buffer =~ m/S3_BUCKET_SIZE\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_BUCKET_SIZE'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} = 'NO';
    if ($buffer =~ m/S3_HABILITAR\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ANTIGUEDAD_MINIMA'} = '1';
    if ($buffer =~ m/S3_ANTIGUEDAD_MINIMA\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ANTIGUEDAD_MINIMA'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ENDPOINT'} = '';
    if ($buffer =~ m/S3_ENDPOINT\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ENDPOINT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ACCESS_KEY'} = '';
    if ($buffer =~ m/S3_ACCESS_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_ACCESS_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_SECRET_KEY'} = '';
    if ($buffer =~ m/S3_SECRET_KEY\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_SECRET_KEY'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_BUCKET_NAME'} = '';
    if ($buffer =~ m/S3_BUCKET_NAME\s*=\s*["']([^'"]+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_BUCKET_NAME'} = $1;
    }

    # configuracion sitemaps
    # máximo segun google 50.000
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NRO_MAX_PAGS'} = 1000;
    if ($buffer =~ m/SITEMAP_NRO_MAX_PAGS\s*=\s*['"](\d+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NRO_MAX_PAGS'} = $1;
    }

    # máximo segun google 1.000
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NRO_MAX_NEWS'} = 200;
    if ($buffer =~ m/SITEMAP_NRO_MAX_NEWS\s*=\s*['"](\d+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NRO_MAX_NEWS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_ARTICULOS'} = "";
    if ($buffer =~ m/SITEMAP_ARTICULOS\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_ARTICULOS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_ARTICULOS_NEWS'} = "";
    if ($buffer =~ m/SITEMAP_ARTICULOS_NEWS\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_ARTICULOS_NEWS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PORTADAS'} = "";
    if ($buffer =~ m/SITEMAP_PORTADAS\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PORTADAS'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_IMG_PORT'} = "";
    if ($buffer =~ m/SITEMAP_NEWS_IMG_PORT\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_IMG_PORT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_IMG_ART'} = "";
    if ($buffer =~ m/SITEMAP_NEWS_IMG_ART\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_IMG_ART'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_CAPTION_PORT'} = "";
    if ($buffer =~ m/SITEMAP_NEWS_CAPTION_PORT\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_CAPTION_PORT'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_CAPTION_ART'} = "";
    if ($buffer =~ m/SITEMAP_NEWS_CAPTION_ART\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS_CAPTION_ART'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_SITENAME'} = "";
    if ($buffer =~ m/SITEMAP_SITENAME\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_SITENAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS'} = "NO";
    if ($buffer =~ m/SITEMAP_PAGS\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS'} = $1;
    }
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS_SEPARAR'} = "NO";
    if ($buffer =~ m/SITEMAP_PAGS_SEPARAR\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS_SEPARAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS'} = "NO";
    if ($buffer =~ m/SITEMAP_NEWS\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_NEWS'} = $1;
    }

    # pplayer
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_ENABLE'} = "NO";
    if ($buffer =~ m/AD_ENABLE\s*=\s*['"](SI|NO)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_ENABLE'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_INFO_MSG'} = " La publicidad termina en __TIME__ segundos.";
    if ($buffer =~ m/AD_INFO_MSG\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_INFO_MSG'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_TAG'} = "";
    if ($buffer =~ m/AD_TAG\s*=\s*['"]([^'"]+)['"]/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'AD_TAG'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_SITE";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_TEMPLATE";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO'} eq 'SI' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER'} eq 'NO' && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'} ne '') {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'} = "/$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'}$prontus_varglb::DIR_PATH_CPAN/data";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MASTER_ID'};
    } else {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN/data";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_SSO_MANAGER_ID'} = $prontus_varglb::PRONTUS_ID;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}  = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'}   = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log";
    # logs de queries de elasticsearch
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}   = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log_es";
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_NRO_ARTIC'} = $prontus_varglb::MAX_NRO_ARTIC; # este valor se "copia"
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'} = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_CPAN_DATA$prontus_varglb::DIR_LOCK";

    $prontus_varglb::OIDC_HABILITADO        = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_HABILITADO'};
    $prontus_varglb::OIDC_TITULAR        = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_TITULAR'};
    $prontus_varglb::OIDC_ENDPOINT_URI        = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_ENDPOINT_URI'};
    $prontus_varglb::OIDC_CLIENT_ID        = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_CLIENT_ID'};
    $prontus_varglb::OIDC_APP_SECRET        = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'OIDC_APP_SECRET'};
    $prontus_varglb::SITEMAP_PAGS_SEPARAR       = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SITEMAP_PAGS_SEPARAR'};
    $prontus_varglb::DIR_LOG_ES    = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'};

    # configuracion cargada para este prontus
    $prontus_varglb::CONFIG_LOADED{$prontus_varglb::PRONTUS_ID} = 1;
    return '';
}

# ---------------------------------------------------------------
sub get_path_nice {
    if ($prontus_varglb::PATHNICE ne '') {
        return $prontus_varglb::PATHNICE;
    }
    $prontus_varglb::PATHNICE = '/usr/bin/nice';
    if (!-f $prontus_varglb::PATHNICE) {
        $prontus_varglb::PATHNICE = '/usr/local/bin/nice';
        if (!-f $prontus_varglb::PATHNICE) {
            $prontus_varglb::PATHNICE = '/usr/nice';
            if (!-f $prontus_varglb::PATHNICE) {
                $prontus_varglb::PATHNICE = '';
            }
        }
    }
    return $prontus_varglb::PATHNICE;
}

# ---------------------------------------------------------------
sub garbage_preview {
    my $ruta_script = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/";

    my $pathnice = get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);

    my $script = $ruta_script . "prontus_preview_garbage_collector.pl";
    my $cmd    = "$pathnice $script $prontus_varglb::PRONTUS_ID 1>/dev/null 2>/dev/null &";

    print STDERR "\ngarbage_preview cmd[$cmd]\n";

    system($cmd);
}

#------------------------------------------------------------------------#
sub call_system_and_location {
    my ($nom_script, $params, $escape, $location) = @_;
    $escape = 1 if (!defined($escape));
    # ruta_script:
    #   En unix queda algo asi como /sites/misitio.cl/web/cgi-cpn/
    my $ruta_script = $ENV{'SCRIPT_FILENAME'};
    $ruta_script =~ s/\w+\.\w+$//;
    if (!$ruta_script) {
        $ruta_script = "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/";
    }

    my $pathnice = get_path_nice();
    $pathnice = "$pathnice -n19 " if ($pathnice);
    my $script = $ruta_script . "$nom_script";

    # limpiar parametros
    $params =~ s/[#;&<>|~]+//g if ($escape);

    my $cmd    = "$pathnice $script $params 1>/dev/null 2>/dev/null &";
    print STDERR "[" . glib_hrfec_03::get_dtime_pack4() . "]$cmd\n";
    system $cmd;
    if ($location) {
        print "Status: 303 See Other\n";
        print "Location: $location\n\n";
    }
}

# ---------------------------------------------------------------
# Obtiene extension sin punto
sub get_file_extension {
    my $nomfile = shift;
    return $1 if ($nomfile =~ /\.(\w+)$/);
    return '';
}

#-----------------------------------------------------------------------
# Determina si el archivo es mas antiguo que N segundos, de acuerdo a fecha/hora de modificacion.
sub arch_antiguo {
    my ($pathArch) = shift;
    my ($n_segs)   = shift;

    # Obtener estadisticas del arch.
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $pathArch;

    # Si los seg. de antiguedad de la pagina son mayores que $taxport_refresh_segs
    if ((time - $mtime) > $n_segs) {
        return 1;    # es mas antiguo que N segundos
    }
    return 0;
}


# escribe al log de operación de prontus
# requiere glib_hrfec_03
sub write_log {
    my ($accion, $objeto, $path, $contexto) = @_;
    my ($linea, $fecha, $hora, $buf, $nom_file, $usr);
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'} =~ s/\\/\//g;
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'} =~ s/(\w)\/$/$1/;    # Borrar posible ultimo slash.

    if (-d $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'}) {
        # <nombre del publicador>_aaaammdd.log
        $fecha    = glib_hrfec_03::get_date_pack4();
        $nom_file = $prontus_varglb::PRONTUS_ID . '_' . $fecha . '.txt';

        # dd/mm/aaaa hh:mm:ss - <ip> - <accion> - <user> - <ente afectado> - <path>
        $fecha = glib_hrfec_03::des_normaliza_fecha($fecha);
        $hora = glib_hrfec_03::get_date_time('', '', '', '', 1, '', time);
        if ($prontus_varglb::USERS_USR) {
            $usr = $prontus_varglb::USERS_USR;
        } else {
            if ($contexto) {
                $usr = $contexto;
            } else {
                $usr = 'System Script';
            }
        }

        my $user_ip = $ENV{'REMOTE_ADDR'};
        if (lib_prontus::cache_active() && defined($ENV{'HTTP_X_FORWARDED_FOR'})) {
            $user_ip = $ENV{'HTTP_X_FORWARDED_FOR'};
        }

        if ($accion !~ /login/i) {
            $path =~ s/^$prontus_varglb::DIR_SERVER//;
            $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $usr . "\t" . $accion . "\t" . $objeto . "\t" . $path . "\n";
        } else {
            $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $objeto . "\t" . $accion . "\t\t\t" . $path . "\n";
        }

        my $log_file = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG'} . '/' . $nom_file;
        my $fh = lib_waitlock::lock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log.mutex");
        glib_fildir_03::append_file($log_file, $linea);
        lib_waitlock::unlock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log.mutex", $fh);
    }
}

# ---------------------------------------------------------------
# Ve si la portada esta asignada al usuario.
# Se requiere que ya se haya pasado por la funcion &check_user_session()
sub port_asoc {
    my $port = $_[0];
    my $user_id = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'};
    if (defined $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$user_id}{$port}) {
        return 1;
    }
    return 0;
}

# ---------------------------------------------------------------
sub escape_xml {    # rotulos tax
    my $toencode = $_[0];
    $toencode =~ s/&nbsp;/ /g;               # '
    # elimina entidades no numericas y las reemplaza por espacios
    $toencode =~ s/&[\w]+;/ /g;
    # reemplaza espacios dobles por uno simple
    $toencode =~ s/ +/ /g;
    # Ampersands no seguidos por "amp;"
    $toencode =~ s/&(?!amp;)/&amp;/g;
    # entidades reconocidas por xml
    $toencode =~ s/>/&gt;/g;      # >
    $toencode =~ s/</&lt;/g;      # <
    $toencode =~ s/"/&quot;/g;              # "
    $toencode =~ s/'/&#39;/g;               # '
    return $toencode;
}

# ---------------------------------------------------------------
sub unescape_xml {                # rotulos tax
    my $toencode = $_[0];
    $toencode =~ s/&gt;/>/g;      # >
    $toencode =~ s/&lt;/</g;      # <
    $toencode =~ s/&quot;/"/g;    # "
    $toencode =~ s/&#39;/'/g;
    $toencode =~ s/&apos;/'/g;
    $toencode =~ s/&amp;/&/g;
    return $toencode;
}

# ---------------------------------------------------------------
# Elimina un substring al comienzo de un string
sub remove_front_string {
    my $string    = shift;
    my $substring = shift;
    $string =~ s/^$substring//is;
    return $string;
}

# ---------------------------------------------------------------
# retorna nombre y erxtension (el ulimo punto en adelante)
sub split_nom_y_extension {
    my $nomfile = shift;
    return ($1, $2) if ($nomfile =~ /([\w\-\d]+)\.?(\w+)?$/);
    return '';
}

sub write_log_es {
    my $query = shift;
    my ($linea, $fecha, $hora, $nom_file);
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'} =~ s/\\/\//g;
    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'} =~ s/(\w)\/$/$1/;    # Borrar posible ultimo slash.

    glib_fildir_03::check_dir($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'});
    $fecha    = glib_hrfec_03::get_date_pack4();
    $nom_file = $prontus_varglb::PRONTUS_ID . '_' . $fecha . '.txt';

    # dd/mm/aaaa hh:mm:ss - <ip> - <query>
    $fecha = glib_hrfec_03::des_normaliza_fecha($fecha);
    $hora = glib_hrfec_03::get_date_time('', '', '', '', 1, '', time);

    my $user_ip = $ENV{'REMOTE_ADDR'};
    if (lib_prontus::cache_active() && defined($ENV{'HTTP_X_FORWARDED_FOR'})) {
        $user_ip = $ENV{'HTTP_X_FORWARDED_FOR'};
    }
    $linea = $fecha . "\t" . $hora . "\t" . $user_ip . "\t" . $query . "\n";

    my $log_file = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'} . '/' . $nom_file;
    my $fh = lib_waitlock::lock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log_es.mutex");
    glib_fildir_03::append_file($log_file, $linea);
    lib_waitlock::unlock_file("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOCKS'}/log_es.mutex", $fh);
}

# ---------------------------------------------------------------
# Chequea que el rango definido por la fechap/horap y la fechae/horae del artic.
# incluyan la fecha actual, para saber si hay q publicarlo o no.
sub fechas_ok {
    my ($ts_preview, $fechap, $horap, $fechae, $horae) = @_;

    $fechap = '9' x 8 if ($fechap eq '');
    $fechae = '9' x 8 if ($fechae eq '');
    $horap  = '0' x 6 if ($horap  eq '');
    $horae  = '0' x 6 if ($horae  eq '');

    $horap = $1 . $2 . '00' if ($horap =~ /^(\d\d):?(\d\d)$/);
    if ($horae =~ /^(\d\d):?(\d\d)$/) {
        $horae = $1 . $2 . '00';
    }

    # ts_now
    my $ts_now;
    if ($ts_preview ne '') {
        $ts_now = $ts_preview;
    } else {
        # TS de AHORA
        $ts_now = glib_hrfec_03::get_dtime_pack4();
        # $ts_now = '20110112000000'; # TS 0 de la mañana
    }

    # ts_publicacion
    my $ts_p = "$fechap$horap";

    # ts_expiracion
    my $ts_e = "$fechae$horae";

    if (($ts_p > $ts_now) || ($ts_e < $ts_now)) {
        # no publicar articulo
        return 0;
    }
    return 1;
}

# ---------------------------------------------------------------
# Imprime pagina con reporte de resultado.
# Param :
# 0) Titulo del mensaje
# 1) Mensaje a imprimir
# 2) 1 | 0: si no viene, se asume mostrar link para 'volver'
# 3) 1 | 0: salir, por defecto terminar ejecucion
# 4) 1 | 0: imprimir content type html, por defecto imprimir
# 5) '' | nolink, no muestra link de volver o cerrar
sub print_pag_result {
    my $tit    = $_[0];
    my $result = $_[1];
    my $cerrar = $_[2];
    my $exit   = defined($_[3]) ? $_[3] : 1;
    my $ctype  = defined($_[4]) ? $_[4] : 1;
    my $link   = defined($_[5]) ? $_[5] : '';

    binmode STDOUT, ':encoding(UTF-8)';

    # Abrir y cargar archivo corresp. a la plantilla
    my $pagina = &load_cpan_core_plt("core/prontus_msg.html");
    if ($pagina ne '') {
        $pagina = &set_coreplt_ppal_simple($pagina);
        $pagina =~ s/%%title%%/$tit/g;
        $pagina =~ s/%%msg%%/$result/g;

        if ($link eq 'nolink') {
            $pagina =~ s/<!--VOLVER-->.*?<!--\/VOLVER-->//sg;
            $pagina =~ s/<!--CERRAR-->.*?<!--\/CERRAR-->//sg;
        } else {
            if ($cerrar) {
                $pagina =~ s/<!--VOLVER-->.*?<!--\/VOLVER-->//sg;
            } else {
                $pagina =~ s/<!--CERRAR-->.*?<!--\/CERRAR-->//sg;
            }
        }
    } else {
        $pagina = $result;
    }

    print "Content-Type: text/html\r\n\r\n" if ($ctype);
    print $pagina;
    exit if ($exit);
}

# ---------------------------------------------------------------
# Imprime pagina html con headers necesarios, termina la ejecucion del script
# Param :
# 0) html completo
# 1) 1 | 0: permite decidir si enviar headers de no cache
# 2) headers adicionales que se deban enviar, cookies, status 404,
# se imprimen primero
sub print_html {
    my ($html, $nocache, $headers) = @_;
    $nocache = 1 if (!defined($nocache));

    binmode STDOUT, ':encoding(UTF-8)';

    print $headers if ($headers);
    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n" if ($nocache);
    print "Content-type: text/html;charset=utf-8\r\n\r\n";

    print $html;
    exit;
}

# ---------------------------------------------------------------
# Imprime xml con headers necesarios, termina la ejecucion del script
# Param :
# 0) xml completo
# 1) 1 | 0: permite decidir si enviar headers de no cache
# se imprimen primero
sub print_xml {
    my ($xml, $nocache) = @_;
    $nocache = 1 if (!defined($nocache));

    binmode STDOUT, ':encoding(UTF-8)';

    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n" if ($nocache);
    print "Content-type: text/xml;charset=utf-8\r\n\r\n";

    print $xml;
    exit;
}

# ---------------------------------------------------------------
# Imprime csv con headers necesarios, termina la ejecucion del script
# Param :
# 0) csv completo
# 1) nombre de archivo a descargar
# se imprimen primero
sub print_attach_csv {
    my ($csv,$filename) = @_;

    binmode STDOUT, ':encoding(UTF-8)';

    print "Cache-Control: no-cache, must-revalidate, max-age=0\r\n";
    print 'Content-Disposition: attachment; filename="'.$filename.'"\r\n';
    print "Content-type: text/csv\r\n\r\n";

    print $csv;
    exit;
}

#---------------------------------------------------------------
# Imprime mensaje json
sub print_json_string {
    my $string = $_[0];

    binmode STDOUT, ':encoding(UTF-8)';

    print "Cache-Control: no-cache, must-revalidate\r\n";
    print "Content-type: application/json; charset=utf-8\r\n\r\n";

    utf8::decode($string);
    print $string;

    exit;
}

# ---------------------------------------------------------------
sub load_cpan_core_plt {
    my $file           = $_[0];
    my $buffer         = glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/$file");

    if ($buffer eq '') {
        return "Error: no existe plantilla: $prontus_varglb::DIR_PLT_CORE/$file";
    }

    my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;
    $buffer =~ s/$crlf/\x0a/sg;
    return $buffer;
}

# ---------------------------------------------------------------
# Parsea plantilla de nivel principal en el core con reemplazos comunes.
# Asume que ya se paso por el load_config de prontus y se autentico al user.
# el 2do parametro es optativo, en caso de necesitar usar otro dir de macros
sub set_coreplt_ppal {
    my ($buffer, $pathdir_macros) = @_;

    # En primer lugar, agrega macros
    my $dir_macros = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/macros";    # ubicacion default
    $dir_macros = $pathdir_macros if (($pathdir_macros) && (-d $pathdir_macros));        # para sobreescribir ubic default

    # parsea globales de prontus
    $buffer = lib_parser::parse_globals($buffer, '');
    # agrega macros indicadas en la plantilla
    $buffer = lib_parser::add_macros($buffer, $dir_macros, '', '');
    # convierte codificacion url de %
    $buffer =~ s/%25%25/%%/g;

    # Perfilación de contenido
    $buffer = lib_profile::html_profiler($buffer);

    # oculta html correspondiente solo a admin
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} ne '1') {
        $buffer =~ s/<!--ADMIN_ONLY-->.*?<!--\/ADMIN_ONLY-->//sg;
        $buffer =~ s/%%admin_only%%.*?%%\/admin_only%%//sg;
    } else {
        $buffer =~ s/<!--ADMIN_ONLY-->//g;
        $buffer =~ s/<!--\/ADMIN_ONLY-->//g;
        $buffer =~ s/%%admin_only%%//g;
        $buffer =~ s/%%\/admin_only%%//g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} eq 'NO') {
        $buffer =~ s/<!--ELASTICSEARCH_HABILITADO-->.*?<!--\/ELASTICSEARCH_HABILITADO-->//sg;
    } else {
        $buffer =~ s/<!--\/?ELASTICSEARCH_HABILITADO-->//g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} ne 'SI') {
        $buffer =~ s/<!--PUSH_NOTIF-->.*?<!--\/PUSH_NOTIF-->//sg;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'NO') {
        $buffer =~ s/<!--ADMIN_PLUGINS-->.*?<!--\/ADMIN_PLUGINS-->//sg;
    } else {
        # despliega sistema de plugins si corresponde
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'SI') {
            my $menu_plugins = &parsea_menu_plugins();
            if ($menu_plugins eq '') {
                $buffer =~ s/<!--ADMIN_PLUGINS-->.*?<!--\/ADMIN_PLUGINS-->//sg;
            } else {
                $buffer =~ s/%%plugins_lista%%/$menu_plugins/g;
            }
        } else {
            $buffer =~ s/%%plugins_lista%%//g;
        }
        $buffer =~ s/<!--\/?ADMIN_PLUGINS-->//g;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} eq 'NO') {
         $buffer =~ s/<!--SEO_ACTIVO-->.*?<!--\/SEO_ACTIVO-->//sg;
    } else {
        $buffer =~ s/<!--\/?SEO_ACTIVO-->//g;
    }

    # oculta html correspondiente a opciones de multivistas si no está habilitado.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_MV'} eq 'NO') {
        $buffer =~ s/<!--SEO_MV-->.*?<!--\/SEO_MV-->//sg;
    } else {
        $buffer =~ s/<!--\/?SEO_MV-->//g;
    }

    # oculta html prohibido a usuarios sin portadas asignadas
    if ( &tiene_portadas_asignadas() == 0) {
        $buffer =~ s/<!--USERS_CON_PORTS-->.*?<!--\/USERS_CON_PORTS-->//sg;
    } else {
        $buffer =~ s/<!--\/?USERS_CON_PORTS-->//g;
    }

    # oculta a html al admin
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq '1') {
        $buffer =~ s/<!--HIDE_TO_ADMIN-->.*?<!--\/HIDE_TO_ADMIN-->//sg;
    } else {
        $buffer =~ s/<!--\/?HIDE_TO_ADMIN-->//g;
    }

    # Para ocultar o mostrar opciones si la multiedicion esta activa/inactiva
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        $buffer =~ s/<!--MULTI_EDICION-->.*?<!--\/MULTI_EDICION-->//sg;
        $buffer =~ s/%%multi_edicion%%.*?%%\/multi_edicion%%//sg;
        $buffer =~ s/%%\/?no_multi_edicion%%//g;
    } else {
        $buffer =~ s/%%\/?multi_edicion%%//g;
        $buffer =~ s/<!--\/?MULTI_EDICION-->//g;
        $buffer =~ s/%%no_multi_edicion%%.*?%%\/no_multi_edicion%%//sg;
    }

    # friendly v5 no activa
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} ne '5') {
        $buffer =~ s/<!--FRIENDLY_V5-->.*?<!--\/FRIENDLY_V5-->//sg;
        $buffer =~ s/<!--\/?NO_FRIENDLY_V5-->//g;
    } else {
        $buffer =~ s/<!--\/?FRIENDLY_V5-->//g;
        $buffer =~ s/<!--NO_FRIENDLY_V5-->.*?<!--\/NO_FRIENDLY_V5-->//sg;
    }

    # muestra/oculta opciones de S3
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} eq 'SI') {
        $buffer =~ s|<!--/?S3-BLOCK-->||sg;
        $buffer =~ s|%%/?s3%%||sg;
    } else {
        $buffer =~ s|<!--S3-BLOCK-->.+?<!--/S3-BLOCK-->||sg;
        $buffer =~ s|%%s3%%.+?%%/s3%%||sg;
    }

    # datos del usuario
    $buffer =~ s/%%_prontus_user_name%%/$prontus_auth::USERS_NOM/g;
    my $glosa_perfil = lib_utils::get_glosa_perfil($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'});
    $buffer =~ s/%%_prontus_user_perfil%%/$glosa_perfil/g;

    # para usar con archivos que deben recargarse siempre
    my $random = glib_str_03::random_string(10);
    $buffer =~ s/%%_random%%/$random/g;

    return $buffer;
}
# ---------------------------------------------------------------
# Parsea plantilla de nivel principal en el core con reemplazos comunes.
# No realiza perfilacion de usuario, solo procesa configuraciones de prontus
# el 2do parametro es optativo, en caso de necesitar usar otro dir de macros
sub set_coreplt_ppal_simple {
    my ($buffer, $pathdir_macros) = @_;

    # En primer lugar, agrega macros
    my $dir_macros = "$prontus_varglb::DIR_SERVER$prontus_varglb::DIR_PLT_CORE/macros";    # ubicacion default
    $dir_macros = $pathdir_macros if (($pathdir_macros) && (-d $pathdir_macros));        # para sobreescribir ubic default

    # parsea globales de prontus
    $buffer = lib_parser::parse_globals($buffer, '');
    # agrega macros indicadas en la plantilla
    $buffer = lib_parser::add_macros($buffer, $dir_macros, '', '');
    # convierte codificacion url de %
    $buffer =~ s/%25%25/%%/g;

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'ELASTICSEARCH_HABILITADO'} eq 'NO') {
        $buffer =~ s/<!--ELASTICSEARCH_HABILITADO-->.*?<!--\/ELASTICSEARCH_HABILITADO-->//sg;
    } else {
        $buffer =~ s/<!--\/?ELASTICSEARCH_HABILITADO-->//g;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SNP_APP'} ne 'SI') {
        $buffer =~ s/<!--PUSH_NOTIF-->.*?<!--\/PUSH_NOTIF-->//sg;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'NO') {
        $buffer =~ s/<!--ADMIN_PLUGINS-->.*?<!--\/ADMIN_PLUGINS-->//sg;
    } else {
        # despliega sistema de plugins si corresponde
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'HABILITAR_PLUGINS'} eq 'SI') {
            my $menu_plugins = &parsea_menu_plugins();
            if ($menu_plugins eq '') {
                $buffer =~ s/<!--ADMIN_PLUGINS-->.*?<!--\/ADMIN_PLUGINS-->//sg;
            } else {
                $buffer =~ s/%%plugins_lista%%/$menu_plugins/g;
            }
        } else {
            $buffer =~ s/%%plugins_lista%%//g;
        }
        $buffer =~ s/<!--\/?ADMIN_PLUGINS-->//g;
    }

    # oculta html correspondiente a prontus plugins si no están habilitados.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SEO_ACTIVO'} eq 'NO') {
         $buffer =~ s/<!--SEO_ACTIVO-->.*?<!--\/SEO_ACTIVO-->//sg;
    } else {
        $buffer =~ s/<!--\/?SEO_ACTIVO-->//g;
    }

    # Para ocultar o mostrar opciones si la multiedicion esta activa/inactiva
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTI_EDICION'} ne 'SI') {
        $buffer =~ s/<!--MULTI_EDICION-->.*?<!--\/MULTI_EDICION-->//sg;
        $buffer =~ s/%%multi_edicion%%.*?%%\/multi_edicion%%//sg;
        $buffer =~ s/%%\/?no_multi_edicion%%//g;
    } else {
        $buffer =~ s/%%\/?multi_edicion%%//g;
        $buffer =~ s/<!--\/?MULTI_EDICION-->//g;
        $buffer =~ s/%%no_multi_edicion%%.*?%%\/no_multi_edicion%%//sg;
    }

    # friendly v5 no activa
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} ne '5') {
        $buffer =~ s/<!--FRIENDLY_V5-->.*?<!--\/FRIENDLY_V5-->//sg;
        $buffer =~ s/<!--\/?NO_FRIENDLY_V5-->//g;
    } else {
        $buffer =~ s/<!--\/?FRIENDLY_V5-->//g;
        $buffer =~ s/<!--NO_FRIENDLY_V5-->.*?<!--\/NO_FRIENDLY_V5-->//sg;
    }

    # muestra/oculta opciones de S3
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'S3_HABILITAR'} eq 'SI') {
        $buffer =~ s|<!--/?S3-BLOCK-->||sg;
        $buffer =~ s|%%/?s3%%||sg;
    } else {
        $buffer =~ s|<!--S3-BLOCK-->.+?<!--/S3-BLOCK-->||sg;
        $buffer =~ s|%%s3%%.+?%%/s3%%||sg;
    }

    # para usar con archivos que deben recargarse siempre
    my $random = glib_str_03::random_string(10);
    $buffer =~ s/%%_random%%/$random/g;

    return $buffer;
}

# ---------------------------------------------------------------
# lee los plugins instalados en el directorio [prontus_dir]/cpan/prontus_plugins/
# y los carga en el sidebar
# soporta un solo nivel de profundidad.
sub parsea_menu_plugins {
    my %plugins = &get_plugins_paths('habilitados');

    return '' if ! %plugins; # si no hay plugins termino ahora.

    my ($output, $plugin);
    # cargamos la plantilla para la sidebar
    my $tpl_plugins_menu = &load_cpan_core_plt("prontus_plugins/plugins_menu.html");
    foreach my $nombre (sort keys %plugins) {
        my $tmp = $tpl_plugins_menu;

        $plugins{$nombre} =~ s/\s+/$plugins{$nombre}/sg;
        $plugin = URI::Escape::uri_escape_utf8($plugins{$nombre});

        $tmp =~ s/%%plugin%%/$plugin/sg;
        $tmp =~ s/%%nombre%%/$nombre/sg;
        $output .= $tmp;
    }
    $output = lib_parser::parse_globals($output, '');
    return $output;
}

# lee los archivos "plugin.json" en los subdir del directorio de plugins de Prontus
# devuelve un hash (nombre_plugin => path_plugin) con los plugins instalados y habilitados.
# parametro $estado: 'habilitados', 'todos'
sub get_plugins_paths {
    my $estado = shift;

    my @paths_plugins = glob "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/prontus_plugins/*/plugin.json";
    my %plugins;

    foreach my $json_file (@paths_plugins) {
        my $json_text = glib_fildir_03::read_file_bin($json_file);

        my $data = JSON::decode_json($json_text);

        next if ((!$data->{'habilitado'}) && (lc $estado eq 'habilitados'));
        next unless $data->{'nombre'};

        # perfil 1 (admin) puede ver todo
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} ne '1') {
            my $access_granted = 0;
            if (defined($data->{profiles})) {
                if ($data->{profiles}[0] == 0) {
                    $access_granted = 1;
                } else {
                    foreach my $profile (@{$data->{profiles}}) {
                        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq $profile) {
                            $access_granted = 1;
                            last;
                        }
                    }
                }
            }
            next if (!$access_granted);
        }

        $data->{'path'} = '#';
        if ($json_file =~ m(cpan/prontus_plugins/(.*?)/plugin.json)) {
            $data->{'path'} = $1;
        }
        $data->{'path'} =~ s/\.\.+/./sg; # evitamos que intenten cargar archivos con . o .. ???
        $plugins{$data->{'nombre'}} = $data->{'path'};
    }
    return %plugins;
}

# ---------------------------------------------------------------
# chequea si el usuario tiene portadas asignadas.
# @param void
# return bool
sub tiene_portadas_asignadas {
    # es perfil admin o esta activo el SSO
    return 1 if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'} eq '1');

    # verifica si el usuario tiene portadas asignadas en los hashes correspondientes, hash definido y con elementos
    if (defined($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}})
        && keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PORTUSERS'}{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}}}) {
        return 1;
    }
    return 0;
}

# ---------------------------------------------------------------
sub get_glosa_perfil {
    my %profiles = lib_profile::get_profiles();
    return defined($profiles{$_[0]})?$profiles{$_[0]}{'nombre'}:'';
}

return 1;
