#!/usr/bin/perl
# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use URI::Escape;
use glib_cgi_05;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_parser;
use lib_setbd;
use lib_utils;
use lib_waitlock;
##/libs##

our %FORM = ();
our $PRONTUS_DIR_CREADO_POR_CGI = 0;

main: {
    lib_utils::setUtf8();
    my ($perfil, $id_usr, $pagina);

    my $purge = $lib_prontus::DISABLE_PURGE_CACHE;
    $lib_prontus::DISABLE_PURGE_CACHE = 1; # deshabilitar purge.

    lib_utils::setUtf8();
    # Rescatar parametros recibidos.
    glib_cgi_05::new();
    $FORM{'accion'} = glib_cgi_05::param('accion');
    $prontus_varglb::PRONTUS_ID = glib_cgi_05::param('_prontus_id'); # ejemplo : /publicadores/prontus_noticias
    $FORM{'wizard'} = glib_cgi_05::param('wizard'); # 1 | 0: para saber si viene del wizard

    $pagina = lib_utils::load_cpan_core_plt("core/prontus_check_install.html");
    $pagina = lib_utils::set_coreplt_ppal_simple($pagina);

    my $tmp_result;

    # validaciones iniciales, antes de hacer nada: verifica Perl y Blowfish.
    my $version_perl = $];
    my $version_perl_minima = 5.008;
    if ($version_perl < $version_perl_minima) {
        my $error_perl = "[Error] Perl version incorrecta, se requiere Perl $version_perl_minima o superior. Se encontró la versión $version_perl. La instalación no puede continuar.";
        $pagina =~ s/%%accion%%/Verificando la versión del intérprete Perl:/g;
        $pagina =~ s/%%resultados%%/$error_perl/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
        lib_utils::print_html($pagina);
    }

    &valida_param();

    if ($FORM{'accion'} == 1) {
        $pagina =~ s/%%accion%%/Probando escritura y eliminación de archivos en $prontus_varglb::PRONTUS_ID :/g;
        $tmp_result = &check_write();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } elsif ($FORM{'accion'} == 2) {
        $pagina =~ s/%%accion%%/Probando creación de tabla de pruebas :/g;
        $tmp_result = &crear_tabla();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } elsif ($FORM{'accion'} == 3) {
        $pagina =~ s/%%accion%%/Probando insert en tabla 'PRUEBA' :/g;
        $tmp_result = &escribir_reg();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } elsif ($FORM{'accion'} == 4) {
       $pagina =~ s/%%accion%%/ Probando insert con lock\/unlock :/g;
        $tmp_result = &test_lock();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } elsif ($FORM{'accion'} == 5) {
        $pagina =~ s/%%accion%%/Probando 10 insert consecutivos en tabla 'PRUEBA' :/g;
        $tmp_result = &escribir_diez_reg();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } elsif ($FORM{'accion'} == 6) {
        $pagina =~ s/%%accion%%/Listando contenido de tabla 'PRUEBA' :/g;
        $tmp_result = &query_table();
        $pagina =~ s/%%resultados%%/$tmp_result/g;
        $pagina =~ s/%%linea_check_error%%.*%%\/linea_check_error%%//gs;
    } else {
        $pagina = lib_utils::load_cpan_core_plt("core/prontus_check_install_form.html");
        $pagina = lib_utils::set_coreplt_ppal_simple($pagina);
    }
    # restablece valor de purge original
    $lib_prontus::DISABLE_PURGE_CACHE = $purge;

    lib_utils::print_html($pagina);
}

# ---------------------------------------------------------------
# SUB-RUTINAS.
# --------------------------------------------------------------
# Chequear escritura de archivos.
#
# - Emular la escritura de un archivo con: html, xml, adjuntos, fotos y mm (todo)
# - Emular art relac
# - Emular tax
# - Emular escritura de portada
# - Escribir en carpeta temp de la cgi
# - Escribir en cpan/data
#
# Crear carpetas si esto no existe.

# Site de articulos
sub check_write {
    my $dirfecha = glib_hrfec_03::get_date_pack4();
    my $ts = glib_hrfec_03::get_dtime_pack4();
    my $reldir_pags = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/pags";
    my $reldir_img = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/imag";
    my $reldir_mmedia = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dirfecha$prontus_varglb::DIR_MMEDIA";
    my $reldir_asocfile = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ASOCFILE/$dirfecha/$ts";
    my $reldir_xml = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dirfecha/xml";

    my $plantilla = lib_utils::load_cpan_core_plt("core/check_write.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    my ($result, $falla);

    ($result, $falla) = &write_file_test($reldir_pags);
    $plantilla =~ s/%%result_reldir_pags%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_img);
    $plantilla =~ s/%%result_reldir_img%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_mmedia);
    $plantilla =~ s/%%result_reldir_media%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_asocfile);
    $plantilla =~ s/%%result_reldir_asocfile%%/$result/g;
    ($result, $falla) = &write_file_test($reldir_xml);
    $plantilla =~ s/%%result_reldir_xml%%/$result/g;

    # art. relac
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXONOMIA");
    $plantilla =~ s/%%result_art_relac%%/$result/g;

    # cpan/data y derivados
    ($result, $falla) = &write_file_test($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'});
    $plantilla =~ s/%%result_dbm%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/log");
    $plantilla =~ s/%%result_cpan_log%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/search");
    $plantilla =~ s/%%result_dbm_search%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/user_lock");
    $plantilla =~ s/%%result_dbm_user_lock%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/users");
    $plantilla =~ s/%%result_dbm_users%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs");
    $plantilla =~ s/%%result_dbm_procs%%/$result/g;

    # site ediciones
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC");
    $plantilla =~ s/%%result_dir_edic%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base");
    $plantilla =~ s/%%result_dir_edic_base%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/port");
    $plantilla =~ s/%%result_dir_edic_base_port%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/rss");
    $plantilla =~ s/%%result_dir_edic_base_rss%%/$result/g;
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EDIC/base/xml");
    $plantilla =~ s/%%result_dir_edic_base_xml%%/$result/g;

    # port. tipo tema
    ($result, $falla) = &write_file_test("$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT");
    $plantilla =~ s/%%result_dir_edic_port_tema%%/$result/g;

    # temp
    my $pesoscero = $0;

    $pesoscero =~ s/(\w+\.cgi)$/prontus_temp/;
    $pesoscero =~ s/\\/\//;
    $pesoscero =~ s/$prontus_varglb::DIR_SERVER//;
    ($result, $falla) = &write_file_test($pesoscero);
    $plantilla =~ s/%%result_dir_pesoscero%%/$result/g;

    # Si la invocacion vino del wizard es porque el dir prontus no existia
    # y lo tuve q crear para el test asi q ahora lo borro.
    if (($FORM{'wizard'}) && ($PRONTUS_DIR_CREADO_POR_CGI)) {
        glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID");
        if (-d "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID") {
            &salir("Error: no fue posible eliminar directorio $prontus_varglb::PRONTUS_ID");
        } else {
            $plantilla =~ s/%%check_result%%/[$prontus_varglb::PRONTUS_ID] ... Borrando ... OK/g;
        }
    }

    return $plantilla;
}

# ---------------------------------------------------------------
sub write_file_test {
    my $reldir = $_[0];
    my $result;
    my $falla = 0;

    if (!(glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$reldir")) ) {
        $result = "La carpeta no existe o no se pudo crear : [$reldir]";
        return $result;
    }

    my $file_test = "$prontus_varglb::DIR_SERVER$reldir/prueba.txt";
    glib_fildir_03::write_file($file_test, 'test');

    $result = "[$reldir/prueba.txt] ... ";

    if ( (-f $file_test) && (-s $file_test) )  {
        $result .=  "OK";
        $result .= " - Borrando ... ";
        if (unlink $file_test) {
            $result .= "OK";
        } else {
            $result .= "Falla";
            $falla = 1;
        }
    } else {
        $result .= "Falla";
        $falla = 1;
    }

    return ($result, $falla);
}

# ---------------------------------------------------------------
sub query_table {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_query_table.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);
    my $result;

    # Conectando con BD
    $plantilla =~ s/%%encabezado%%/Conectando con BD MySQL. Nombre BD: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    $base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}", $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) || &salir("Error: la base de datos MySQL '$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}' no existe o no fue posible conectarse a ella con los parámetros indicados en el cfg. (" . DBI->errstr . ")");

    my $sql = "SELECT PRUEBA_ID, PRUEBA_TITULAR from PRUEBA";
    my ($id, $nom);
    my $salida = &local_sql_bind($base, $sql, \($id, $nom)) || &salir("Error al realizar query: SQL [$sql] (" . DBI->errstr . ")");
    my ($loop, $looptmp, $looptot);
    $plantilla =~ /%%loop_registros%%(.*)%%\/loop_registros%%/g;
    $loop = $1;

    while ($salida->fetch) {
        $looptmp = $loop;
        $looptmp =~ s/%%registro%%/PRUEBA_ID[$id] - PRUEBA_TITULAR[$nom]/g;
        $looptot .= $looptmp;
    }

    $salida->finish;
    $base->disconnect;
    $plantilla =~ s/%%loop_registros%%.*%%\/loop_registros%%/$looptot/gs;

    return $plantilla;
}

#-------------------------------------------------------------------------#
# ejecuta una sentencia sql y devuelve su handler.
# Param entrada:
#  0) descriptor de la base de datos abierta.
#  1) sentencia SQL propiamente tal.
#  2) nombres de las variables asociadas a los campos.
#  retorno:
#  0) descriptor de la respuesta. (Usar solamente fetch para cargar las variables con las filas siguientes.
sub local_sql_bind {
    my ($dbh) = shift;
    my ($sql) = shift;
    my (@var) = (@_);
    my ($sth);

    $sth = $dbh->prepare($sql) || return 0;

    $sth->execute || return 0;
    $sth->bind_columns(undef, (@var)) || return 0;

    return($sth);
}

# ---------------------------------------------------------------
sub escribir_diez_reg {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_escribir_diez_reg.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);
    # Conectando con BD
    $plantilla =~ s/%%encabezado%%/Conectando con BD MySQL. Nombre BD: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    $base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}", $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) || &salir("Error: la base de datos MySQL '$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}' no existe o no fue posible conectarse a ella con los parámetros indicados en el cfg. (" . DBI->errstr . ")");

    my $i;
    my ($loop, $looptmp, $looptot);
    $plantilla =~ /%%loop_registros%%(.*)%%\/loop_registros%%/g;
    $loop = $1;

    for ($i = 1; $i <= 10; $i++) {
        $looptmp = $loop;
        my $sql = "insert into PRUEBA values(NULL, \"TEXTO DEL REGISTRO\")";
        $base->do($sql) || &salir("Error escribiendo registro nro. $i<br>(" . $base->errstr . ")");
        $looptmp =~ s/%%registro%%/registro nro. $i escrito OK/g;
        $looptot .= $looptmp;
    }

    $plantilla =~ s/%%loop_registros%%.*%%\/loop_registros%%/$looptot/gs;
    $base->disconnect;
    $plantilla =~ s/%%resultado%%/10 registros insertados OK/g;

    return $plantilla;
}

# ---------------------------------------------------------------
# Prueba de escribir un registro en la tabla PRUEBA.
sub escribir_reg {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_insert.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    # Conectando con BD
    $plantilla =~ s/%%encabezado%%/Conectando con BD MySQL. Nombre BD: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    $base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}", $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) || &salir("Error: la base de datos MySQL '$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}' no existe o no fue posible conectarse a ella con los parámetros indicados en el cfg. (" . DBI->errstr . ")");

    my $sql;
    # Inserta registro en la tabla.
    $plantilla =~ s/%%preliminar%%/Escribiendo un registro en tabla PRUEBA/g;
    $sql = "insert into PRUEBA values(NULL, \"TEXTO DEL REGISTRO\")";
    $base->do($sql) || &salir("Error escribiendo registro (" . $base->errstr . ")");
    $base->disconnect;
    $plantilla =~ s/%%resultado%%/Registro escrito OK/g;

    return $plantilla;
}

# ---------------------------------------------------------------
    #Crear tabla de pruebas en la BD.
    #Tabla: PRUEBA. Columnas: PRUEBA_ID y PRUEBA_TITULAR.
    #Si no existe, se crea.
    #Si ya existe, se borra la que estaba antes.
    #Con MySQL si es MySQL, tomando los datos del cfg.
sub crear_tabla {
    my $base;
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_create_table.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    # Conectando con BD
    $plantilla =~ s/%%encabezado%%/Conectando con BD MySQL. Nombre BD: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'} \/ server: $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}/g;
    $base = DBI->connect("DBI:mysql:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}:$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_BD'}", $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USER_BD'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PWD_BD'}) || &salir("Error: la base de datos MySQL '$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'NOM_BD'}' no existe o no fue posible conectarse a ella con los parámetros indicados en el cfg. (" . DBI->errstr . ")");

    # Chequeando existencia
    my $sql;
    $plantilla =~ s/%%preliminar%%/Chequeando existencia de tabla 'PRUEBA'.../g;

    if (&lib_setbd::existe_tabla($base, 'PRUEBA')) {
        $plantilla =~ s/%%existe_elimina%%/Tabla 'PRUEBA' existe, intentando eliminarla.../g;
        # SECC
        $sql = 'drop table PRUEBA';
        $base->do($sql) || &salir("Error eliminando tabla 'PRUEBA': [" . $base->errstr . "]");
        $plantilla =~ s/%%existe_elimina_result%%/Eliminada OK./g;
        $plantilla =~ s/%%check_existencia%%//g;
        $plantilla =~ s/%%\/check_existencia%%//g;
    } else {
        $plantilla =~ s/%%check_existencia%%.*%%\/check_existencia%%//gs;
    }

    # Crear tabla
    $plantilla =~ s/%%crear_tabla%%/Creando tabla 'PRUEBA'.../g;
    $sql = "CREATE TABLE PRUEBA (
                PRUEBA_ID int(5) NOT NULL auto_increment,
                PRUEBA_TITULAR varchar(128) NOT NULL default '',
                PRIMARY KEY  (PRUEBA_ID),
                KEY PRUEBA_TITULAR (PRUEBA_TITULAR)
            )";

    $base->do($sql) || &salir("Error creando tabla 'PRUEBA': [" . $base->errstr . "]");
    $plantilla =~ s/%%result_crear_tabla%%/Tabla 'PRUEBA' creada OK./g;

    $base->disconnect;
    return $plantilla;
}

# ---------------------------------------------------------------
sub test_lock {
    my $lock_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/prueba.lock";
    my $plantilla = lib_utils::load_cpan_core_plt("core/check_insert_lock.html");
    $plantilla = lib_utils::set_coreplt_ppal_simple($plantilla);

    my $preliminar = "Bloqueando [$lock_file]...";
    my $fh = lib_waitlock::lock_file($lock_file);
    $preliminar .= " Bloqueado OK.";
    $plantilla =~ s/%%preliminar%%/$preliminar/g;

    my $write_result = &escribir_reg();
    $plantilla =~ s/%%resultado%%/$write_result/g;

    my $postliminar = "Desbloqueando [$lock_file]...";
    lib_waitlock::unlock_file($lock_file, $fh);
    unlink $lock_file;

    $postliminar .= " Desbloqueado OK.";
    $plantilla =~ s/%%postliminar%%/$postliminar/g;

    return $plantilla;
}

# ---------------------------------------------------------------
sub valida_param {
    my $result;
    $prontus_varglb::PRONTUS_ID =~ s/\\/\//g;
    $prontus_varglb::PRONTUS_ID =~ s/\/$//g;

    if ( ($prontus_varglb::PRONTUS_ID eq '') || ($prontus_varglb::PRONTUS_ID eq '/') || ($prontus_varglb::PRONTUS_ID eq 'prontus') )  {
        &salir("Error: Directorio del publicador no es válido. Debe indicar el nombre del Prontus a chequear, ejemplo: _prontus_id=prontus_noticias");
    }

    if ($FORM{'wizard'}) {
        # En este caso no debe existir la carpeta del prontus asi q se intenta crear.
        if (! -d "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID") {
            if ( ! (glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID")) ) {
                &salir("Error: Directorio del publicador no existe y no fue posible crearlo.");
            } else {
                $result = "Creando directorio $prontus_varglb::PRONTUS_ID ... creado OK.";
                $PRONTUS_DIR_CREADO_POR_CGI = 1;
            }
        } else {
            &salir("Error: Directorio $prontus_varglb::PRONTUS_ID ya existe.");
        }

        # Cargar variables para emular carga de arch. de config
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_SITE";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'} = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN/data";
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}  = "/$prontus_varglb::PRONTUS_ID$prontus_varglb::DIR_PATH_CPAN";
    } else {
        if (! -d "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID") {
            &salir("Error: Directorio $prontus_varglb::PRONTUS_ID no existe.");
        }

        # Carga variables de configuracion.
        lib_utils::load_config_by_id($prontus_varglb::PRONTUS_ID);
    }
}

# ---------------------------------------------------------------
sub salir {
    lib_utils::print_html($_[0]);
}
